@@ -30,14 +30,14 @@
 init() -> true.
 
 menu({body}, Menu) ->
-    case wpc_snap:active() of
+    case wpc_snap_win:active() of
 	true ->
 	    Menu;
 	false ->
 	    Menu ++ [separator,auv_menu()]
     end;
 menu({face}, Menu) ->
-    case wpc_snap:active() of
+    case wpc_snap_win:active() of
 	true ->
 	    Menu;
 	false ->
@@ -60,6 +60,17 @@ auv_menu(1,_What) -> {?MODULE, segment};
 auv_menu(2,_) -> {?MODULE, segment_old};
 auv_menu(3,_) -> {?MODULE, force_seg}.
 
+auv_show_menu(Action) ->
+    Cmd = {show,toggle_background},
+    case Action of
+	true ->
+	    Label = ?__(1,"Show/Hide Background Image"),
+	    Help = ?__(2,"Toggle display of the background texture image"),
+	    wings_menu:update_menu(view, Cmd, {append, 0, Label},Help);
+	false ->
+	    wings_menu:update_menu(view, Cmd, delete)
+    end.
+
 command({body,{?MODULE, Op}} , St) ->
     start_uvmap(Op, St);
 command({face,{?MODULE, Op}} , St) ->
@@ -666,11 +677,6 @@ handle_event_3({action,{edit,repeat_args}}, St) ->
     repeat(args, St);
 handle_event_3({action,{edit,repeat_drag}}, St) ->
     repeat(drag, St);
-handle_event_3({action,{view,toggle_background}}, _) ->
-    Old = get({?MODULE,show_background}),
-    put({?MODULE,show_background},not Old),
-    wings_wm:dirty();
-
 handle_event_3({action,Ev}=Act, #st{selmode=AUVSel, bb=#uvstate{st=#st{selmode=GSel}}}=St) ->
     case Ev of  %% Keyboard shortcuts end up here (I believe)
 	{_, {move,_}} ->
@@ -685,6 +691,10 @@ handle_event_3({action,Ev}=Act, #st{selmode=AUVSel, bb=#uvstate{st=#st{selmode=G
 	    handle_command(slide,St);
 	{_, circularise} ->
 	    handle_command(circularise,St);
+	{view,{show,toggle_background}} ->
+	    Old = get({?MODULE,show_background}),
+	    put({?MODULE,show_background},not Old),
+	    wings_wm:dirty();
 	{view,aim} ->
 	    St1 = fake_selection(St),
 	    wings_view:command(aim, St1),
@@ -727,7 +737,11 @@ handle_event_3(got_focus, _) ->
     Msg3 = wings_msg:button_format([], [], ?__(2,"Show menu")),
     Message = wings_msg:join([Msg1,Msg2,Msg3]),
     wings_wm:message(Message, ""),
+    auv_show_menu(true),
     wings_wm:dirty();
+handle_event_3(lost_focus, _) ->
+    auv_show_menu(false),
+    keep;
 handle_event_3(_Event, _) ->
     %% io:format("MissEvent ~P~n", [_Event, 20]),
     keep.
@@ -767,7 +781,7 @@ new_state(#st{bb=#uvstate{}=Uvs}=St0) ->
     St = update_selected_uvcoords(St1),
     get_event(St).
 
-handle_command(Cmd, St0) ->    
+handle_command(Cmd, St0) ->
     case handle_command_1(Cmd,remember_command(Cmd,St0)) of
 	Drag = {drag, _} -> do_drag(Drag);
 	Result -> Result
@@ -1008,7 +1022,9 @@ fake_sel_1(St0) ->
 
 add_faces(NewFs,St0=#st{bb=ASt=#uvstate{id=Id,mode=Mode,st=GeomSt=#st{shapes=Shs0}}}) ->
     case {NewFs,Mode} of
-	{_,object} -> wings_shape:show_all(St0);
+	{_,object} ->
+            All = wings_obj:fold(fun(#{id:=I}, A) -> [I|A] end, [], St0),
+            wings_obj:unhide(All, St0);
 	{object,_} -> %% Force a chart rebuild, we are switching object mode
 	    We = gb_trees:get(Id,Shs0),
 	    Shs = gb_trees:update(Id, We#we{fs=undefined,es=array:new()}, Shs0),
@@ -1035,35 +1051,33 @@ do_drag(Other) ->
     Other.
 
 tighten(#st{selmode=vertex}=St) ->
-    tighten_1(fun vertex_tighten/3, St);
+    tighten_1(fun vertex_tighten/2, St);
 tighten(#st{selmode=body}=St) ->
-    tighten_1(fun(_, We, A) -> body_tighten(We, A) end, St).
+    tighten_1(fun(_, We) -> body_tighten(We) end, St).
 
-tighten_1(Tighten, St) ->    
-    Tvs = wings_sel:fold(Tighten, [], St),
-    wings_drag:setup(Tvs, [percent], St).
+tighten_1(Tighten, St) ->
+    wings_drag:fold(Tighten, [percent], St).
 
-vertex_tighten(Vs0, We, A) ->
+vertex_tighten(Vs0, We) ->
     Vis = gb_sets:from_ordset(wings_we:visible(We)),
     Vs = [V || V <- gb_sets:to_list(Vs0), not_bordering(V, Vis, We)],
-    wings_vertex_cmd:tighten(Vs, We, A).
+    wings_vertex_cmd:tighten_vs(Vs, We).
 
-body_tighten(#we{vp=Vtab}=We, A) ->
+body_tighten(#we{vp=Vtab}=We) ->
     Vis = gb_sets:from_ordset(wings_we:visible(We)),
     Vs = [V || V <- wings_util:array_keys(Vtab), not_bordering(V, Vis, We)],
-    wings_vertex_cmd:tighten(Vs, We, A).
+    wings_vertex_cmd:tighten_vs(Vs, We).
 
 tighten(Magnet, St) ->
-    Tvs = wings_sel:fold(fun(Vs, We, A) ->
-				 mag_vertex_tighten(Vs, We, Magnet, A)
-			 end, [], St),
     Flags = wings_magnet:flags(Magnet, []),
-    wings_drag:setup(Tvs, [percent,falloff], Flags, St).
+    wings_drag:fold(fun(Vs, We) ->
+                            mag_vertex_tighten(Vs, We, Magnet)
+                    end, [percent,falloff], Flags, St).
 
-mag_vertex_tighten(Vs0, We, Magnet, A) ->
+mag_vertex_tighten(Vs0, We, Magnet) ->
     Vis = gb_sets:from_ordset(wings_we:visible(We)),
     Vs = [V || V <- gb_sets:to_list(Vs0), not_bordering(V, Vis, We)],
-    wings_vertex_cmd:tighten(Vs, We, Magnet, A).
+    wings_vertex_cmd:tighten_vs(Vs, We, Magnet).
 
 equal_length(Op,St) ->
     wings_sel:map(fun(Es, We) ->
