@@ -12,7 +12,7 @@
 %%
 
 -module(wings).
--export([start/0,start_halt/0,start_halt/1]).
+-export([start/0,start_link/1, init/1]).
 -export([redraw/1,redraw/2,init_opengl/1,command/2]).
 -export([mode_restriction/1,clear_mode_restriction/0,get_mode_restriction/0]).
 -export([ask/3]).
@@ -32,76 +32,29 @@
 -import(lists, [foldl/3,sort/1]).
 
 start() ->
-    do_spawn(none).
+    application:start(wings).
 
-start_halt() ->
-    spawn_halt(none).
+start_link(Env) ->
+    do_spawn(Env, [link]).
 
-start_halt([File|_]) ->
-    spawn_halt(File).
-
-spawn_halt(File) ->
-    spawn(fun() ->
-		  process_flag(trap_exit, true),
-		  Wings = do_spawn(File, [link]),
-		  halt_loop(Wings)
-	  end).
-
-halt_loop(Wings) ->
-    %% Handle normal and abnormal termination of the Wings process.
-    %% None of the strings are translated because when this code
-    %% executes, the ETS table that holds the translated strings
-    %% has been deleted.
-    receive
-	{'EXIT',Wings,normal} ->
-	    %% Normal termination.
-	    init:stop();
-	{'EXIT',Wings,{window_crash,Name,Reason,StkTrace}} ->
-	    %% Crash in a window with an error reason and stack trace.
-	    Log = wings_u:crash_log(Name, Reason, StkTrace),
-	    io:format("\n\n"),
-	    %% Intentionally not translated.
-	    io:format("Fatal internal error - log written to ~s\n",
-		      [Log]),
-	    ok;
-	{'EXIT',Wings,{crash_logged, Log}} ->
-	    io:format("Fatal internal error - log written to ~s\n",
-		      [Log]),
-	    ok;
-	{'EXIT',Wings,Reason} ->
-	    %% Some other crash.
-	    Log = wings_u:crash_log("<Unknown Window Name>", Reason, []),
-	    io:format("\n\n"),
-	    %% Intentionally not translated.
-	    io:format("Fatal internal error - log written to ~s\n",
-		      [Log]),
-	    ok;
-	_Other ->
-	    %% We are not supposed to receive any other messages,
-	    %% but it is good practice to always throw away
-	    %% unexpected messages.
-	    halt_loop(Wings)
-    end.
-
-do_spawn(File) ->
-    do_spawn(File, []).
-
-do_spawn(File, Flags0) ->
+do_spawn(Env, Flags0) ->
     %% Set a minimal heap size to avoiding garbage-collecting
     %% all the time. Don't set it too high to avoid keeping binaries
     %% too long.
-    Fun = fun() -> init(File) end,
     Flags1 = case erlang:system_info(version) >= "8.1" of
                  true -> [{message_queue_data, off_heap}|Flags0];
                  false -> Flags0
              end,
     Flags = [{fullsweep_after,16384},{min_heap_size,32*1204}|Flags1],
-    spawn_opt(erlang, apply, [Fun,[]], Flags).
+    {ok, proc_lib:start_link(?MODULE, init, [Env], infinity, Flags)}.
 
-init(File) ->
+-spec init(term()) -> no_return().
+init(Env) ->
     process_flag(trap_exit, true),
     register(wings, self()),
     erlang:system_flag(backtrace_depth, 25),
+    wx:set_env(Env),
+
     wings_pref:init(),
     wings_hotkey:set_default(),
     wings_pref:load(),
@@ -109,23 +62,28 @@ init(File) ->
     wings_plugin:init(),
     wings_sel_cmd:init(),
     wings_file:init(),
+    macosx_workaround(),
+    %% Ack that we are done, need to create top window now
+    proc_lib:init_ack(self()),
+    Frame = receive {frame_created, Window} -> Window end,
+    GeomGL = wings_gl:init(Frame),
+    wx_object:get_pid(Frame) ! opengl_initialized,
+    %% Wait for other mandatory processes to become initialized
+    receive supervisor_initialization_done -> ok end,
+    init_part2(Frame, GeomGL).
 
+init_part2(Frame, GeomGL) ->
     St0 = new_st(),
     St1 = wings_sel:reset(St0),
     St2 = wings_undo:init(St1),
-    St = wings_shape:create_folder_system(St2),
+    St = wings_obj:create_folder_system(St2),
 
-    group_leader(wings_console:start(), self()),
-
-    Frame = wings_frame:start(),
-    GeomGL = wings_gl:init(Frame),
     %% Needs to be initialized before make_geom_window
     %% and before the others that use gl functions
     wings_text:init(),
     wings_wm:init(Frame),
     check_requirements(),
 
-    wings_image:init(wings_io:get_process_option()),
     wings_color:init(),
     wings_io:init(),
 
@@ -135,20 +93,19 @@ init(File) ->
     wings_view:init(),
     wings_u:caption(St),
     wings_file:init_autosave(),
-    wings_pb:start_link(Frame),
     wings_dialog:init(),
     wings_job:init(),
     wings_develop:init(),
     wings_tweak:init(),
 
-    open_file(File),
+    open_file(),
     make_geom_window(GeomGL, St),
     restore_windows(St),
     case catch wings_wm:enter_event_loop() of
-	{'EXIT',normal} ->
-	    wings_pref:finish(),
-	    wings_io:quit();
-	{'EXIT',Reason} ->
+	{'EXIT',shutdown} ->
+	    wings_io:quit(),
+            exit(shutdown);
+        {'EXIT',Reason} ->
 	    io:format("~P\n", [Reason,20]),
 	    wings_io:quit(),
 	    exit(Reason)
@@ -169,6 +126,15 @@ check_requirements() ->
     minor_gl_version(),
     have_fbo().
 
+macosx_workaround() ->
+    try 1.0/zero()
+    catch
+	error:_ -> ok
+    end.
+
+zero() ->
+    0.0.
+
 minor_gl_version() ->
     Major = 2,
     Minor = 1,
@@ -201,7 +167,7 @@ fatal(Str) ->
     Dialog = wxMessageDialog:new(Parent, Str, [{caption,"Fatal Error"}]),
     wxMessageDialog:showModal(Dialog),
     wings_io:quit(),
-    exit(normal).
+    exit(fatal).
 
 new_st() ->
     Empty = gb_trees:empty(),
@@ -243,21 +209,40 @@ free_viewer_num(N) ->
 	true -> free_viewer_num(N+1)
     end.
 
-open_file(File0) ->
+open_file() ->
     USFile = wings_file:autosave_filename(wings_file:unsaved_filename()),
-    Recovered = filelib:is_file(USFile),
-    wings_pref:set_value(file_recovered, Recovered),
+    case filelib:is_file(USFile) of
+	true ->
+	    wings_u:yes_no(?__(1,"Wings3D has detected an unsaved file.") ++"\n" ++
+			   ?__(2,"Would you like to recover it?"),
+			   fun() ->
+			       wings_pref:set_value(file_recovered, true),
+			       wings_wm:send_after_redraw(geom, {open_file,USFile})
+			   end,
+			   fun() ->
+			       wings_file:del_unsaved_file(),
+			       open_file_start()
+			   end);
+	_ -> open_file_start()
+    end.
+
+open_file_start() ->
+    File1 = case application:get_env(wings, args) of
+                undefined -> none;
+                {ok, File0} ->
+                    case filelib:is_regular(File0) of
+                        true -> File0;
+                        false -> none
+                    end
+            end,
     %% On the Mac, if Wings was started by clicking on a .wings file,
     Msgs0 = wxe_master:fetch_msgs(),
     Msgs = [F || F <- Msgs0, filelib:is_regular(F)],
     File = case Msgs of
 	       [F|_] -> F;
-	       [] -> File0
+	       [] -> File1
 	   end,
-    if Recovered ->
-	    wings_u:message(?__(1,"Wings3D has recovered an unsaved file.")),
-	    wings_wm:send_after_redraw(geom, {open_file,USFile});
-       File =:= none ->
+    if File =:= none ->
 	    timer:sleep(200), %% For splash screen :-)
 	    ignore;
        true ->
@@ -272,14 +257,13 @@ init_opengl(St) ->
     keep.
 
 redraw(St) ->
-    redraw(info(St), St).
+    redraw(wings_info:info(St), St).
 
 redraw(Info, St) ->
     Render =
 	fun() ->
 		wings_wm:clear_background(),
 		wings_render:render(St),
-		call_post_hook(St),
 		TweakInfo = wings_tweak:statusbar(),
 		case Info =/= [] andalso wings_wm:get_prop(show_info_text) of
 		    true when TweakInfo =:= [] ->
@@ -295,12 +279,6 @@ redraw(Info, St) ->
 	end,
     wings_io:batch(Render).
 
-call_post_hook(St) ->
-    case wings_wm:lookup_prop(postdraw_hook) of
-    none -> ok;
-    {value,{_Id,Fun}} -> Fun(St)
-    end.
-
 register_postdraw_hook(Window, Id, Fun) ->
     case wings_wm:lookup_prop(Window, postdraw_hook) of
     none ->
@@ -356,7 +334,7 @@ handle_event({crash_in_other_window,LogName}, St) ->
 handle_event({open_file,Name}, St0) ->
     case catch ?SLOW(wings_ff_wings:import(Name, St0)) of
 	#st{}=St1 ->
-	    St2 = wings_shape:recreate_folder_system(St1),
+	    St2 = wings_obj:recreate_folder_system(St1),
 	    USFile = wings_file:autosave_filename(wings_file:unsaved_filename()),
 	    St = case USFile of
 		     Name ->
@@ -447,6 +425,11 @@ handle_event_3(quit, St) ->
 	geom -> do_command({file,quit}, none, St);
 	_ -> keep
     end;
+handle_event_3({'EXIT',Pid,shutdown}, _St) ->
+    case whereis(wings_sup) of
+        Pid -> exit(normal);
+        _ -> keep
+    end;
 handle_event_3({new_state,St}, St0) ->
     info_line(),
     save_state(St0, St);
@@ -488,14 +471,35 @@ handle_event_3({external,not_possible_to_save_prefs}, _St) ->
     wings_help:not_possible_to_save_prefs();
 handle_event_3({external, win32_start_maximized}, _St) ->
     keep;
-handle_event_3({external, Fun}, St) 
+handle_event_3({external, Fun}, St)
   when is_function(Fun) ->
     Fun(St);
 handle_event_3({external,Op}, St) ->
     wpa:handle_external(Op,St),
     keep;
+handle_event_3(#wx{event=#wxDropFiles{files=Fs0}}, St0) ->
+    Send = fun(Op) -> wings ! {action, {file, Op}} end,
+    ImgFms = wings_image:image_formats(),
+    {Images, Fs1} = lists:partition(fun(Image) -> filter_format(Image, ImgFms) end, Fs0),
+    _ = [Send({import_image, Image}) || Image <- Images],
+    %% Merge or open wings files
+    WngFms = [{".wings", ""}],
+    {Objects, Fs2} = lists:partition(fun(Obj) -> filter_format(Obj, WngFms) end, Fs1),
+    DoOpen = wings_obj:num_objects(St0) =:= 0,
+    Open = fun(File, true)  -> Send({confirmed_open, File}), false;
+              (File, false) -> Send({merge, File}), false
+           end,
+    lists:foldl(Open, DoOpen, Objects),
+    case Fs2 of
+        [] -> keep;
+        [File] -> wings_u:error_msg(?__(1,"Unknown file format: ~s"), [filename:basename(File)]);
+        [_|_]  -> wings_u:error_msg(?__(2,"Unknown file formats"))
+    end;
 handle_event_3(ignore, _St) ->
     keep;
+handle_event_3({system,_,_}, _St) ->
+    %% observer or other system message
+    keep;
 handle_event_3({adv_menu_abort, Ev}, _St) ->
     This = wings_wm:actual_focus_window(),
     wings_wm:send(This,Ev);
@@ -523,6 +527,10 @@ handle_popup_event(Ev, Xglobal, Yglobal, St0) ->
 	    popup_menu(Xglobal, Yglobal, St0)
     end.
 
+filter_format(File, Formats) ->
+    Ext = filename:extension(File),
+    lists:keymember(Ext, 1, Formats).
+
 info_line() ->
     case wings_pref:get_value(tweak_active) of
 	false ->
@@ -676,7 +684,8 @@ command_response(keep, _, _) ->
     keep;
 command_response(quit, _, _) ->
     save_windows(),
-    exit(normal).
+    wings_pref:finish(),
+    exit(shutdown).
 
 remember_command({C,_}=Cmd, St) when C =:= vertex; C =:= edge;
                      C =:= face; C =:= body ->
@@ -1054,403 +1063,6 @@ purge_undo(St) ->
     wings_dialog:dialog("Purge Undo", Qs,
 			fun(_) -> {edit,confirmed_purge_undo} end).
 
-info(#st{sel=[]}) ->
-    [],
-    Progs = get(light_shaders),
-    NumLights = wings_pref:get_value(number_of_lights),
-    ActiveSh = wings_pref:get_value(active_shader),
-    UseProg = (Progs /= undefined) and (NumLights == 2),
-    case UseProg of
-     true ->
-	 {_Prog,Name} = element(ActiveSh, Progs),
-	 io_lib:format("Shader ~p of ~p: ~s ",[ActiveSh,tuple_size(Progs),Name]);
-     false ->
-	 []
-    end;
-info(St) ->
-    case wings_wm:get_prop(show_info_text) of
-    false -> [];
-    true -> info_1(St)
-    end.
-        
-info_1(#st{shapes=Shapes,selmode=body,sel=[{Id,_}]}) ->
-    Sh = gb_trees:get(Id, Shapes),
-    shape_info(Sh);
-info_1(#st{shapes=Shapes,selmode=body,sel=Sel}) ->
-    shape_info(Sel, Shapes);
-info_1(#st{selmode=vertex,sel=[{_Id,Sel}]}=St) ->
-    case gb_sets:size(Sel) of
-    0 -> "";
-    1 ->
-        [V] = gb_sets:to_list(Sel),
-        measure(io_lib:format(?__(1,"Vertex ~p selected"), [V]), St);
-    N when N < 5 ->
-        Vs = gb_sets:to_list(Sel),
-        measure(item_list(Vs, ?__(2,"Vertices")), St);
-    N ->
-        io_lib:format(?__(3,"~p vertices selected"), [N])
-    end;
-info_1(#st{selmode=edge,sel=[{_,Sel}]}=St) ->
-    case gb_sets:size(Sel) of
-    0 -> "";
-    1 ->
-        [Edge] = gb_sets:to_list(Sel),
-        measure(io_lib:format(?__(4,"Edge ~p selected"), [Edge]), St);
-    N when N < 5 ->
-        Edges = gb_sets:to_list(Sel),
-        measure(item_list(Edges, ?__(5,"Edges")), St);
-    N ->
-        measure(io_lib:format(?__(6,"~p edges selected"), [N]), St)
-    end;
-info_1(#st{selmode=face,sel=[{_,Sel}]}=St) ->
-    case gb_sets:size(Sel) of
-    0 -> "";
-    1 ->
-        [Face] = gb_sets:to_list(Sel),
-        measure(io_lib:format(?__(7,"Face ~p selected"), [Face]), St);
-    2 ->
-        Faces = gb_sets:to_list(Sel),
-        measure(item_list(Faces,?__(8,"Faces")), St);
-    N when N < 5 ->
-        Faces = gb_sets:to_list(Sel),
-        item_list(Faces,?__(8,"Faces"));
-    N ->
-        io_lib:format(?__(9,"~p faces selected"), [N])
-    end;
-info_1(#st{selmode=Mode,sel=Sel}=St) ->
-    On = length(Sel),
-    N = foldl(fun({_,S}, A) -> A+gb_sets:size(S) end, 0, Sel),
-    Str = case Mode of
-          vertex ->
-          io_lib:format(?__(10,"~p vertices selected in ~p objects"), [N,On]);
-          edge ->
-          io_lib:format(?__(11,"~p edges selected in ~p objects"), [N,On]);
-          face ->
-          io_lib:format(?__(12,"~p faces selected in ~p objects"), [N,On])
-      end,
-    measure(Str, St).
-
-measure(Base, #st{selmode=vertex,sel=[{Id,Vs}],shapes=Shs}) ->
-    case gb_sets:size(Vs) of
-    1 ->
-        We = gb_trees:get(Id, Shs),
-        [Va] = gb_sets:to_list(Vs),
-        {X,Y,Z} = wings_vertex:pos(Va, We),
-       [Base|io_lib:format(?__(1,". Position <~s  ~s  ~s>"),
-                          [wings_util:nice_float(X),
-                           wings_util:nice_float(Y),
-                           wings_util:nice_float(Z)])];
-    2 ->
-        We = gb_trees:get(Id, Shs),
-        [Va,Vb] = gb_sets:to_list(Vs),
-            {Xa,Ya,Za} = wings_vertex:pos(Va, We),
-            {Xb,Yb,Zb} = wings_vertex:pos(Vb, We),
-        Dist = e3d_vec:dist({Xa,Ya,Za},{Xb,Yb,Zb}),
-       [Base|io_lib:format(?__(5,". Distance ~s  <~s  ~s  ~s>"),
-                          [wings_util:nice_float(Dist),
-                           wings_util:nice_float(abs(Xb - Xa)),
-                           wings_util:nice_float(abs(Yb - Ya)),
-                           wings_util:nice_float(abs(Zb - Za))])];
-    _ -> Base
-    end;
-measure(Base, #st{selmode=vertex,sel=[{IdA,VsA},{IdB,VsB}],shapes=Shs}) ->
-    case gb_sets:size(VsA) == 1 andalso gb_sets:size(VsB) == 1 of
-    true ->
-        WeA = gb_trees:get(IdA, Shs),
-        WeB = gb_trees:get(IdB, Shs),
-        [Va] = gb_sets:to_list(VsA),
-        [Vb] = gb_sets:to_list(VsB),
-       {Xa,Ya,Za} = wings_vertex:pos(Va, WeA),
-       {Xb,Yb,Zb} = wings_vertex:pos(Vb, WeB),
-        Dist = e3d_vec:dist(wings_vertex:pos(Va, WeA),
-                            wings_vertex:pos(Vb, WeB)),
-       [Base|io_lib:format(?__(5,". Distance ~s  <~s  ~s  ~s>"),
-               [wings_util:nice_float(Dist),
-                wings_util:nice_float(Xb - Xa),
-                wings_util:nice_float(Yb - Ya),
-                wings_util:nice_float(Zb - Za)])];
-    _ -> Base
-    end;
-measure(Base, #st{selmode=edge,sel=[{Id,Es}],shapes=Shs}) ->
-    case gb_sets:size(Es) of
-        1 ->
-            We = gb_trees:get(Id, Shs),
-            [Edge] = gb_sets:to_list(Es),
-            #edge{vs=Va,ve=Vb} = array:get(Edge, We#we.es),
-            {Xa,Ya,Za} = wings_vertex:pos(Va, We),
-            {Xb,Yb,Zb} = wings_vertex:pos(Vb, We),
-            Length = e3d_vec:dist({Xa,Ya,Za}, {Xb,Yb,Zb}),
-            {X,Y,Z} = e3d_vec:average({Xa,Ya,Za}, {Xb,Yb,Zb}),
-            [Base|io_lib:format(?__(3,". Midpoint <~s  ~s  ~s>\nLength ~s") ++
-                                "  <~s  ~s  ~s>",
-                                [wings_util:nice_float(X),
-                                 wings_util:nice_float(Y),
-                                 wings_util:nice_float(Z),
-                                 wings_util:nice_float(Length),
-                                 wings_util:nice_float(abs(Xb - Xa)),
-                                 wings_util:nice_float(abs(Yb - Ya)),
-                                 wings_util:nice_float(abs(Zb - Za))])];
-        2 ->
-            We = gb_trees:get(Id, Shs),
-            [E0,E1] = gb_sets:to_list(Es),
-            #edge{vs=V0s,ve=V0e} = array:get(E0, We#we.es),
-            #edge{vs=V1s,ve=V1e} = array:get(E1, We#we.es),
-            PosA = {X0s,Y0s,Z0s} = wings_vertex:pos(V0s, We),
-            PosB = {X0e,Y0e,Z0e} = wings_vertex:pos(V0e, We),
-            PosC = {X1s,Y1s,Z1s} = wings_vertex:pos(V1s, We),
-            PosD = {X1e,Y1e,Z1e} = wings_vertex:pos(V1e, We),
-            V0 = {X0e-X0s, Y0e-Y0s, Z0e-Z0s},
-            V1 = {X1e-X1s, Y1e-Y1s, Z1e-Z1s},
-            RawAngle = e3d_vec:degrees(V0, V1),
-            Angle = case {V0s,V0e} of
-                      {V1s,_} -> RawAngle;
-                      {_,V1e} -> RawAngle;
-                      {V1e,_} -> 180.0 - RawAngle;
-                      {_,V1s} -> 180.0 - RawAngle;
-                      {_,_}   -> RawAngle   %%% unconnected
-                    end,
-            enhanced_info([Base|io_lib:format(?__(6,". Angle ~s")++"~c",
-                          [wings_util:nice_float(Angle),?DEGREE])],
-                          {PosA, PosB, PosC, PosD, E0, E1});
-        _ -> 
-            Base
-    end;
-
-measure(Base, #st{shapes=Shs, selmode=edge, sel=[{Id1,Sel1},{Id2,Sel2}]}) ->
-    case gb_sets:size(Sel1) == 1 andalso gb_sets:size(Sel2) == 1 of
-    true ->
-      WeA = gb_trees:get(Id1, Shs),
-      WeB = gb_trees:get(Id2, Shs),
-      [Es0] = gb_sets:to_list(Sel1),
-      [Es1] = gb_sets:to_list(Sel2),
-      #edge{vs=V0s,ve=V0e} = array:get(Es0, WeA#we.es),
-      #edge{vs=V1s,ve=V1e} = array:get(Es1, WeB#we.es),
-      PosA = {X0s,Y0s,Z0s} = wings_vertex:pos(V0s, WeA),
-      PosB = {X0e,Y0e,Z0e} = wings_vertex:pos(V0e, WeA),
-      PosC = {X1s,Y1s,Z1s} = wings_vertex:pos(V1s, WeB),
-      PosD = {X1e,Y1e,Z1e} = wings_vertex:pos(V1e, WeB),
-      V0 = {X0e-X0s, Y0e-Y0s, Z0e-Z0s},
-      V1 = {X1e-X1s, Y1e-Y1s, Z1e-Z1s},
-      RawAngle = e3d_vec:degrees(V0, V1),
-      Angle = case {V0s,V0e} of
-          {V1s,_} -> RawAngle;
-          {_,V1e} -> RawAngle;
-          {V1e,_} -> 180.0 - RawAngle;
-          {_,V1s} -> 180.0 - RawAngle;
-          {_,_}   -> RawAngle   %%% unconnected
-              end,
-      enhanced_info([Base|io_lib:format(?__(6,". Angle ~s")++"~c",
-                    [wings_util:nice_float(Angle),?DEGREE])],
-                    {PosA, PosB, PosC, PosD, Es0, Es1, Id1, Id2});
-    false->
-        Base
-    end;
-measure(Base, #st{selmode=face,sel=[{Id,Fs}],shapes=Shs}) ->
-    case gb_sets:size(Fs) of
-    1 ->
-        We = gb_trees:get(Id, Shs),
-        [Face] = gb_sets:to_list(Fs),
-        {X,Y,Z} = wings_face:center(Face, We),
-        Mat = wings_facemat:face(Face, We),
-        Area = area_info(Face, We),
-        [Base|wings_util:format(?__(4,". Midpoint <~s  ~s  ~s> \nMaterial ~s.")
-                            ++ Area,
-                            [wings_util:nice_float(X),
-                             wings_util:nice_float(Y),
-                             wings_util:nice_float(Z),
-                             Mat])];
-    2 ->
-      We = gb_trees:get(Id, Shs),
-      [F0,F1] = gb_sets:to_list(Fs),
-      N0 = wings_face:normal(F0, We),
-      N1 = wings_face:normal(F1, We),
-      Angle = e3d_vec:degrees(N0,N1),
-      enhanced_info([Base|io_lib:format(?__(6,". Angle ~s")++"~c",
-      [wings_util:nice_float(Angle),?DEGREE])],{We,F0,F1});
-    _ -> Base
-    end;
-measure(Base, #st{shapes=Shs, selmode=face, sel=[{Id1,Fs1},{Id2,Fs2}]}) ->
-    case gb_sets:size(Fs1) == 1 andalso gb_sets:size(Fs2) == 1 of
-    true ->
-        WeA = gb_trees:get(Id1, Shs),
-        WeB = gb_trees:get(Id2, Shs),
-        [F0] = gb_sets:to_list(Fs1),
-        [F1] = gb_sets:to_list(Fs2),
-          N0 = wings_face:normal(F0, WeA),
-          N1 = wings_face:normal(F1, WeB),
-        Angle = e3d_vec:degrees(N0,N1),
-        enhanced_info([Base|io_lib:format(?__(6,". Angle ~s")++"~c",
-                      [wings_util:nice_float(Angle),?DEGREE])],
-                      {face,WeA,WeB,F0,F1,Id1,Id2});
-    false ->
-        Base
-    end;
-
-measure(Base, _) -> Base.
-
-item_list(Items, Desc) ->
-    item_list(Items, " ", Desc).
-
-item_list([Item|Items], Sep, Desc) ->
-    item_list(Items, ", ", [Desc,Sep|integer_to_list(Item)]);
-item_list([], _Sep, Desc) -> [Desc|?__(1," selected")].
-
-shape_info(We) when ?IS_LIGHT(We) ->
-    wings_light:info(We);
-shape_info(#we{id=Id,name=Name,fs=Ftab,es=Etab,vp=Vtab}=We) ->
-    Faces = gb_trees:size(Ftab),
-    case array:size(Etab) < 50000 of
-        true ->
-            Edges = wings_util:array_entries(Etab),
-            Vertices = wings_util:array_entries(Vtab),
-            Format = ?__(new_object_info,
-                         "Object ~p \"~s\" has ~p polygons, "
-                         "~p edges, ~p vertices~s~s.");
-        false ->
-            Edges = array:size(Etab),
-            Vertices = array:size(Vtab),
-            Format = ?__(new_object_info2,
-                         "Object ~p \"~s\" has ~p polygons, "
-                         "~~~p edges, ~~~p vertices~s~s.")
-    end,
-    wings_util:format(Format,[Id,Name,Faces,Edges,Vertices,
-                              vtx_attributes(We),hole_info(We)]).
-
-vtx_attributes(We) ->
-    case wings_va:any_attributes(We) of
-	false -> "";
-	true -> ", " ++ ?__(1,"vertex attributes")
-    end.
-
-hole_info(#we{holes=[]}) ->
-    "";
-hole_info(#we{holes=Holes}) ->
-    case length(Holes) of
-	1 -> [", 1 ",?__(1,"hole")];
-	N -> [", ",integer_to_list(N)," ",?__(2,"holes")]
-    end.
-
-shape_info(Objs, Shs) ->
-    shape_info(Objs, Shs, 0, 0, 0, 0).
-
-shape_info([{Id,_}|Objs], Shs, On, Vn, En, Fn) ->
-    #we{fs=Ftab,es=Etab,vp=Vtab} = gb_trees:get(Id, Shs),
-    Faces = gb_trees:size(Ftab),
-    case array:size(Etab) < 50000 of
-        true ->
-            Edges = wings_util:array_entries(Etab),
-            Vertices = wings_util:array_entries(Vtab);
-        false ->
-            Edges = array:size(Etab),
-            Vertices = array:size(Vtab)
-    end,
-    shape_info(Objs, Shs, On+1, Vn+Vertices, En+Edges, Fn+Faces);
-shape_info([], _Shs, N, Vertices, Edges, Faces) ->
-    io_lib:format(?__(2,
-             "~p objects, ~p faces, ~p edges, ~p vertices"),
-          [N,Faces,Edges,Vertices]).
-
-enhanced_info(Basic, {PosA, PosB, PosC, PosD, E0, E1}) ->
-    case wings_pref:get_value(info_enhanced_text) of
-        true ->
-            Length1 = e3d_vec:dist(PosA,PosB),
-            Length2 = e3d_vec:dist(PosC,PosD),
-            {Xa,Ya,Za} = e3d_vec:average(PosA,PosB),
-            {Xb,Yb,Zb} = e3d_vec:average(PosC,PosD),
-               Dist = e3d_vec:dist({Xa,Ya,Za}, {Xb,Yb,Zb}),
-               Diff = abs(Length1 - Length2),
-            [Basic|io_lib:format(?__(42,"\nDistance ~s")++"  <~s  ~s  ~s>\n"++
-                                 ?__(41,"Edge~s ~s")++"  "++?__(41,"Edge~s ~s")
-                                 ++"  "++?__(45,"Difference ~s"),
-                                 [wings_util:nice_float(Dist),
-                                  wings_util:nice_float(abs(Xb - Xa)),
-                                  wings_util:nice_float(abs(Yb - Ya)),
-                                  wings_util:nice_float(abs(Zb - Za)),
-                                  wings_util:stringify(E0),
-                                  wings_util:nice_float(Length1),
-                                  wings_util:stringify(E1),
-                                  wings_util:nice_float(Length2),
-                                  wings_util:nice_float(Diff)])];
-        false -> 
-            Basic
-    end;
-enhanced_info(Basic, {PosA, PosB, PosC, PosD, Es0, Es1, Id1, Id2}) ->
-    case wings_pref:get_value(info_enhanced_text) of
-        true ->
-            Length1 = e3d_vec:dist(PosA,PosB),
-            Length2 = e3d_vec:dist(PosC,PosD),
-            {Xa,Ya,Za} = e3d_vec:average(PosA,PosB),
-            {Xb,Yb,Zb} = e3d_vec:average(PosC,PosD),
-            Dist = e3d_vec:dist({Xa,Ya,Za}, {Xb,Yb,Zb}),
-            Diff = abs(Length1 - Length2),
-            [Basic|io_lib:format(?__(42,"\nDistance ~s")++"  <~s  ~s  ~s>\n"++
-                          ?__(43,"Object~s")++" "++?__(41,"Edge~s ~s")++"  "++
-                          ?__(43,"Object~s")++" "++?__(41,"Edge~s ~s")++"  "++
-                          ?__(45,"Difference ~s"),
-                              [wings_util:nice_float(Dist),
-                               wings_util:nice_float(abs(Xb - Xa)),
-                               wings_util:nice_float(abs(Yb - Ya)),
-                               wings_util:nice_float(abs(Zb - Za)),
-                               wings_util:stringify(Id1),
-                               wings_util:stringify(Es0),
-                               wings_util:nice_float(Length1),
-                               wings_util:stringify(Id2),
-                               wings_util:stringify(Es1),
-                               wings_util:nice_float(Length2),
-                               wings_util:nice_float(Diff)])];
-        false ->
-            Basic
-    end;
-enhanced_info(Basic,{We,F0,F1}) ->
-    case wings_pref:get_value(info_enhanced_text) of
-        true ->
-            {Xa,Ya,Za} = wings_face:center(F0, We),
-            {Xb,Yb,Zb} = wings_face:center(F1, We),
-            Dist = e3d_vec:dist({Xa,Ya,Za},{Xb,Yb,Zb}),
-            Area0 = area_info(F0, We),
-            Area1 = area_info(F1, We),
-            [Basic|io_lib:format(?__(42,"\nDistance ~s")++"  <~s  ~s  ~s>\n"
-                                 ++ ?__(48,"Face~s") ++ Area0 ++ "  " 
-                                 ++ ?__(48,"Face~s") ++ Area1,
-                                [wings_util:nice_float(Dist),
-                                 wings_util:nice_float(abs(Xb - Xa)),
-                                 wings_util:nice_float(abs(Yb - Ya)),
-                                 wings_util:nice_float(abs(Zb - Za)),
-								 wings_util:stringify(F0),
-								 wings_util:stringify(F1)])];
-        false ->
-            Basic
-    end;
-enhanced_info(Basic,{face,WeA, WeB, F0, F1,Id1,Id2}) ->
-    case wings_pref:get_value(info_enhanced_text) of
-        true ->
-           {Xa,Ya,Za} = wings_face:center(F0, WeA),
-           {Xb,Yb,Zb} = wings_face:center(F1, WeB),
-           Area0 = area_info(F0, WeA),
-           Area1 = area_info(F1, WeB),
-           Dist = e3d_vec:dist({Xa,Ya,Za},{Xb,Yb,Zb}),
-           [Basic|io_lib:format(?__(42,"\nDistance ~s")++"  <~s  ~s  ~s>\n"++
-                      ?__(43,"Object~s")++" "++?__(48,"Face~s")++Area0++"  "++
-                      ?__(43,"Object~s")++" "++?__(48,"Face~s")++Area1,
-                      [wings_util:nice_float(Dist),
-                       wings_util:nice_float(abs(Xb - Xa)),
-                       wings_util:nice_float(abs(Yb - Ya)),
-                       wings_util:nice_float(abs(Zb - Za)),
-                       wings_util:stringify(Id1),
-                       wings_util:stringify(F0),
-                       wings_util:stringify(Id2),
-                       wings_util:stringify(F1)])];
-        false ->
-            Basic
-    end.
-
-area_info(Face, We) ->
-    case wings_face:vertices(Face,We) =< 50 of
-      true -> A = wings_face:area(Face, We),
-              wings_util:format(?__(40," Area ~s"), [wings_util:nice_float(A)]);
-      false -> []
-    end.
-
 command_name(_Repeat, #st{repeatable=ignore}) ->
     "("++cannot_repeat()++")";
 command_name(Repeat, #st{repeatable={_,Cmd}}=St) ->
@@ -1594,8 +1206,13 @@ drop_command(cancel_drop, St) -> St.
 %%%
 
 save_windows() ->
-    TopSize = wxWindow:getSize(?GET(top_frame)),
-    wings_pref:set_value(window_size, TopSize),
+    Frame = ?GET(top_frame),
+    case wxTopLevelWindow:isMaximized(Frame) of
+	false ->
+	    TopSize = wxWindow:getSize(?GET(top_frame)),
+	    wings_pref:set_value(window_size, TopSize);
+	true -> ignore
+    end,
     {Contained, Free} = wings_frame:export_layout(),
     wings_pref:set_value(saved_windows2, {Contained, Free}).
 
