@@ -17,7 +17,7 @@
 
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
--import(lists, [foldl/3,reverse/1,sort/1,seq/2]).
+-import(lists, [foldl/3,mapfoldl/3,reverse/1,sort/1,seq/2]).
 
 menu(X, Y, St) ->
     Dir = wings_menu_util:directions(St),
@@ -131,18 +131,14 @@ dup(Type) ->
     end.
 
 mode_dependent(St) ->
-    SelObj = wings_sel:fold(fun(_, We, A) -> [We|A] end, [], St),
-    Kind = foldl(fun(We, A) ->
-			 Type = if 
-				    ?IS_AREA_LIGHT(We) -> arealight;
-				    true -> object
-				end,
-			 case A of
-			     none -> Type;
-			     Type -> Type;
-			     _ -> mixed
-			 end
-		 end, none, SelObj),
+    MF = fun(_, We) when ?IS_AREA_LIGHT(We) -> arealight;
+            (_, _) -> object
+         end,
+    RF = fun(T, []) -> T;
+            (T, T) -> T;
+            (_, _) -> mixed
+         end,
+    Kind = wings_sel:dfold(MF, RF, [], St),
     Tail = vertex_color_item(Kind),
     arealight_conv(Kind, Tail).
 
@@ -170,13 +166,13 @@ command(invert, St) ->
 command(duplicate, St) ->
     {save_state,duplicate(none, St)};
 command({duplicate,Dir}, St) ->
-    save_state,duplicate(Dir, St);
+    duplicate(Dir, St);
 command({duplicate_object,Ids}, St) ->
     {save_state,duplicate_object(Ids, St)};
 command(delete, St) ->
-    {save_state,wings_shape:update_folders(delete(St))};
+    {save_state,delete(St)};
 command({delete_object,Ids}, St) ->
-    {save_state,wings_shape:update_folders(delete_object(Ids, St))};
+    {save_state,delete_object(Ids, St)};
 command(tighten, St) ->
     tighten(St);
 command(smooth, St) ->
@@ -206,7 +202,7 @@ command(cleanup, St) ->
 command({cleanup,Ask}, St) ->
     cleanup(Ask, St);
 command(collapse, St) ->
-    {save_state,wings_shape:update_folders(wings_collapse:collapse(St))};
+    {save_state,wings_collapse:collapse(St)};
 command(rename, St) ->
     rename(St);
 command({rename,prefix}, St) ->
@@ -215,12 +211,8 @@ command({rename,Ids}, St) ->
     rename(Ids, St);
 command(to_arealight, St) ->
     to_arealight(St);
-command({to_arealight,Ids}, St) ->
-    to_arealight(Ids, St);
 command(from_arealight, St) ->
     from_arealight(St);
-command({from_arealight,Ids}, St) ->
-    from_arealight(Ids, St);
 command({vertex_attributes,materials_to_colors}, St) ->
     {save_state,materials_to_colors(St)};
 command({vertex_attributes,colors_to_materials}, St) ->
@@ -468,14 +460,13 @@ invert_normals(St) ->
 %%% The Duplicate command.
 %%%
 
-duplicate(Dir, #st{onext=Oid0}=St0) ->
-    St1 = wings_sel:fold(fun(_, We, St) ->
-				 wings_shape:insert(We, copy, St)
-			 end, St0, St0),
-    %% Select the duplicate items, not the original items.
-    Zero = gb_sets:singleton(0),
-    Sel = [{Id,Zero} || Id <- seq(Oid0, St1#st.onext-1)],
-    St = wings_sel:set(Sel, St1),
+duplicate(Dir, St0) ->
+    CF = fun(Items, We) ->
+                 Empty = gb_sets:empty(),
+                 New = [{We,Items,copy}],
+                 {We,Empty,New}
+         end,
+    St = wings_sel:clone(CF, St0),
     case Dir of
 	none -> St;
 	_ -> wings_move:setup(Dir, St)
@@ -485,31 +476,29 @@ duplicate(Dir, #st{onext=Oid0}=St0) ->
 %%% Duplicate called from the Outliner or Object window.
 %%%
 
-duplicate_object(Objects, #st{shapes=Shs}=St) ->
-    foldl(fun(Id, S) ->
-		  We = gb_trees:get(Id, Shs),
-		  wings_shape:insert(We, copy, S)
-	  end, St, Objects).
+duplicate_object(Objects0, St0) ->
+    St = wings_sel:set(body, [{Id, gb_sets:singleton(0)} || Id <- Objects0], St0),
+    CF = fun(Items, #we{id=_Id}=We) ->
+                 {We,Items,[{We,gb_sets:empty(),copy}]}
+         end,
+    wings_sel:clone(CF, body, St).
 
 %%%
 %%% The Delete command.
 %%%
 
-delete(#st{shapes=Shapes0}=St) ->
-    Shapes = wings_sel:fold(fun(_, #we{id=Id}, Shs) ->
-				    gb_trees:delete(Id, Shs)
-			    end, Shapes0, St),
-    St#st{shapes=Shapes,sel=[]}.
+delete(St) ->
+    wings_sel:map_update_sel(
+      fun(_, _) ->
+              {#we{},gb_sets:empty()}
+      end, St).
 
 %%%
 %%% Delete called from the Outliner or Object window.
 %%%
 
-delete_object(Objects, #st{shapes=Shs0}=St) ->
-    Shs = foldl(fun(Id, Shs) ->
-			gb_trees:delete(Id, Shs)
-		end, Shs0, Objects),
-    wings_sel:valid_sel(St#st{shapes=Shs}).
+delete_object(Objects, St) ->
+    foldl(fun wings_obj:delete/2, St, Objects).
 
 %%%
 %%% The Flip command
@@ -588,12 +577,11 @@ flip_center(point, {z,{_,_,Z}}, {Cx,Cy,_}) -> {Cx,Cy,Z}.
 %%%
 
 tighten(St) ->
-    Tvs = wings_sel:fold(fun tighten/3, [], St),
-    wings_drag:setup(Tvs, [percent], St).
+    wings_drag:fold(fun tighten/2, [percent], St).
 
-tighten(_, #we{vp=Vtab}=We, A) ->
+tighten(_, #we{vp=Vtab}=We) ->
     Vs = wings_util:array_keys(Vtab),
-    wings_vertex_cmd:tighten(Vs, We, A).
+    wings_vertex_cmd:tighten_vs(Vs, We).
 
 %%%
 %%% The subdiv command.
@@ -619,32 +607,24 @@ smooth(St) ->
 
 combine(#st{sel=[]}=St) -> St;
 combine(#st{sel=[_]}=St) -> St;
-combine(#st{shapes=Shs0,sel=[{Id,_}=S|_]=Sel0}=St) ->
-    Shs1 = sofs:from_external(gb_trees:to_list(Shs0), [{id,object}]),
-    Sel1 = sofs:from_external(Sel0, [{id,dummy}]),
-    Sel2 = sofs:domain(Sel1),
-    {Wes0,Shs2} = sofs:partition(1, Shs1, Sel2),
-    Wes = sofs:to_external(sofs:range(Wes0)),
-    We = wings_we:merge(Wes),
-    Shs = gb_trees:from_orddict(sort([{Id,We}|sofs:to_external(Shs2)])),
-    wings_shape:recreate_folder_system(St#st{shapes=Shs,sel=[S]}).
+combine(#st{}=St) ->
+    CF = fun(Items, We) ->
+                 {We,Items}
+         end,
+    wings_sel:combine(CF, St).
 
 %%%
 %%% The Separate command.
 %%%
 
 separate(St) ->
-    wings_sel:fold(
-      fun(_, #we{id=Id}=We0, St0) ->
-	      case wings_we:separate(We0) of
-		  [_] -> St0;
-		  [We|Wes] ->
-		      St1 = foldl(fun(W, A) ->
-					  wings_shape:insert(W, sep, A)
-				  end, St0, Wes),
-		      wings_shape:replace(Id, We, St1)
-	      end
-      end, St, St).
+    CF = fun(Items, We0) ->
+                 Empty = gb_sets:empty(),
+                 [We|Wes] = wings_we:separate(We0),
+                 New = [{W,Empty,sep} || W <- Wes],
+                 {We,Items,New}
+         end,
+    wings_sel:clone(CF, St).
 
 %%%
 %%% The Auto-Smooth command.
@@ -693,23 +673,28 @@ cos_degrees(Angle) ->
 %%%
 
 %% used by wings_shape - Rename option - Selected
-rename_selected(Mask,St) ->
-    {_,Names,Wes} = wings_sel:fold(fun(_, We, {Idx,NAcc,WAcc}) ->
-					   Name0=get_masked_name(Mask,Idx),
-					   {Idx+1,[Name0|NAcc],[We|WAcc]}
-				   end, {1,[],[]}, St),
-    rename_1(Names, Wes, St).
+rename_selected(Mask, St) ->
+    Objs = wings_sel:fold_obj(fun(Obj, A) -> [Obj|A] end, [], St),
+    F = fun(_, I) ->
+                Name = get_masked_name(Mask, I),
+                {Name,I+1}
+        end,
+    {Names,_} = mapfoldl(F, 1, Objs),
+    rename_1(Names, Objs, St).
+
 %% used by wings_shape - Rename option - Filtered
-rename_filtered(Filter,Mask,#st{shapes=Shs}=St) ->
-    {_,Names,Wes}=foldl(fun({_,#we{name=Name}=We},{Idx,NAcc,WAcc}=Acc) ->
-				case wings_util:is_name_masked(Name,Filter) of
-				    true ->
-					Name0=get_masked_name(Mask,Idx),
-					{Idx+1,[Name0|NAcc],[We|WAcc]};
-				    false -> Acc
-				end
-			end,{1,[],[]},gb_trees:to_list(Shs)),
-    rename_1(Names, Wes, St).
+rename_filtered(Filter, Mask, St) ->
+    F = fun(#{name:=Name0}=Obj, {I,NameAcc,ObjAcc}=Acc) ->
+                case wings_util:is_name_masked(Name0, Filter) of
+                    true ->
+                        Name = get_masked_name(Mask, I),
+                        {I+1,[Name|NameAcc],[Obj|ObjAcc]};
+                    false ->
+                        Acc
+                end
+        end,
+    {_,Names,Objs} = wings_obj:fold(F, {1,[],[]}, St),
+    rename_1(Names, Objs, St).
 
 get_masked_name(Mask,SeqNum) ->
     Idx=string:chr(Mask,$%),
@@ -735,95 +720,65 @@ get_masked_name(Mask,SeqNum) ->
 	    Prefix++integer_to_list(SeqNum0)++Suffix
     end.
 
-rename_prefix(St0) ->
-    Wes = wings_sel:fold(fun(_, We, A) -> [We|A] end, [], St0),
+rename_prefix(St) ->
     Q = [{hframe,[{text,"Enter Prefix", []}]}],
-    wings_dialog:dialog(?__(1,"Prefix Selected Objects"), Q,
-			fun([Prefix]) ->
-				foldl(fun(#we{id=Id,name=Name}=We, #st{shapes=Shs0}=St) ->
-					      NewName = Prefix++Name,
-					      Shs = gb_trees:update(Id, We#we{name=NewName}, Shs0),
-					      St#st{shapes=Shs}
-				      end, St0, Wes)
-			end).
+    DF = fun([Prefix]) ->
+                 MF = fun(#{name:=Name0}=Obj) ->
+                              Name = Prefix ++ Name0,
+                              Obj#{name:=Name}
+                      end,
+                 wings_sel:map_obj(MF, St)
+         end,
+    wings_dialog:dialog(?__(1,"Prefix Selected Objects"), Q, DF).
 
 rename(St) ->
-    Wes = wings_sel:fold(fun(_, We, A) -> [We|A] end, [], St),
-    rename_1(Wes, St).
+    F = fun(Obj, A) -> [Obj|A] end,
+    Objs = wings_sel:fold_obj(F, [], St),
+    rename_1(Objs, St).
 
-rename(Objects, #st{shapes=Shs}=St) ->
-    Wes = foldl(fun(Id, A) -> [gb_trees:get(Id, Shs)|A] end, [], Objects),
-    rename_1(Wes, St).
+rename(Ids, St) ->
+    Objs = [wings_obj:get(Id, St) || Id <- Ids],
+    rename_1(Objs, St).
 
-rename_1(Wes, St) ->
-    Qs = rename_qs(Wes),
+rename_1(Objs, St) ->
+    Qs = rename_qs(Objs),
     wings_dialog:dialog(?__(1,"Rename"), Qs,
-			fun([[]]) -> ignore;
+			fun([[]]) ->
+                                ignore;
 			   (NewNames) ->
-				rename_1(NewNames, Wes, St)
+				rename_1(NewNames, Objs, St)
 			end).
 
-rename_1(Names, Wes, #st{shapes=Shs}=St) ->
-    rename_2(Names, Wes, Shs, St).
+rename_1(Names, Objs, St) ->
+    rename_2(Names, Objs, St).
 
-rename_2([[]|Ns], [#we{}|Wes], Shs0, St) ->
-    rename_2(Ns, Wes, Shs0, St);
-rename_2([N|Ns], [#we{id=Id}=We|Wes], Shs0, St) ->
-    Shs = gb_trees:update(Id, We#we{name=N}, Shs0),
-    rename_2(Ns, Wes, Shs, St);
-rename_2([], [], Shs, St) -> St#st{shapes=Shs}.
+rename_2([""|Ns], [#{}|Objs], St) ->
+    rename_2(Ns, Objs, St);
+rename_2([N|Ns], [Obj|Objs], St0) ->
+    St = wings_obj:put(Obj#{name:=N}, St0),
+    rename_2(Ns, Objs, St);
+rename_2([], [], St) -> St.
 
-rename_qs(Wes) ->
-    OldNames = [{label,Name} || #we{name=Name} <- Wes],
-    TextFields = [{text,Name,[]} || #we{name=Name} <- Wes],
+rename_qs(Objs) ->
+    OldNames = [{label,Name} || #{name:=Name} <- Objs],
+    TextFields = [{text,Name,[]} || #{name:=Name} <- Objs],
     [{hframe,
       [{vframe,OldNames},
        {vframe,TextFields}]}].
 
 %%%
-%%% Convert selected objects to area lights
+%%% Conversion to and from area lights.
 %%%
-to_arealight(#st{shapes=Shs}=St) ->
-    Wes = wings_sel:fold(fun(_, We, A) -> [We|A] end, [], St),
-    to_arealight_1(Wes, Shs, St).
+to_arealight(St) ->
+    wings_sel:map(fun to_arealight_1/2, St).
 
-to_arealight(Objects, #st{shapes=Shs}=St) ->
-    Wes = foldl(fun(Id, A) -> [gb_trees:get(Id, Shs)|A] end, [], Objects),
-    to_arealight_1(Wes, Shs, St).
-
-to_arealight_1([], Shs, St) -> 
-    St#st{shapes=Shs};
-to_arealight_1([We0|Wes], Shs, St) when ?IS_ANY_LIGHT(We0) ->
-    to_arealight_1(Wes, Shs, St);
-to_arealight_1([#we{id=Id}=We0|Wes], Shs, St) ->
+to_arealight_1(_, We) ->
+    false = ?IS_ANY_LIGHT(We),                  %Assertion.
     #we{light=Light} = wings_light:import([{opengl,[{type,area}]}]),
-    We = We0#we{light=Light},
-    to_arealight_1(Wes, gb_trees:update(Id, We, Shs), St).
+    We#we{light=Light}.
 
-
-
-%%%
-%%% Convert selected area lights to objects
-%%%
-from_arealight(#st{shapes=Shs}=St) ->
-    Wes = wings_sel:fold(
-	    fun (_, We, A) when ?IS_AREA_LIGHT(We)-> 
-		    [We|A];
-		(_, _, A) ->
-		    A
-	    end, [], St),
-    from_arealight_1(Wes, Shs, St).
-
-from_arealight(Objects, #st{shapes=Shs}=St) ->
-    Wes = foldl(fun(Id, A) -> [gb_trees:get(Id, Shs)|A] end, [], Objects),
-    from_arealight_1(Wes, Shs, St).
-
-from_arealight_1([], Shs, St) -> 
-    St#st{shapes=Shs};
-from_arealight_1([#we{id=Id}=We|Wes], Shs, St) when ?IS_AREA_LIGHT(We) ->
-    from_arealight_1(Wes, gb_trees:update(Id, We#we{light=none}, Shs), St);
-from_arealight_1([We|Wes], Shs, St) when ?IS_ANY_LIGHT(We) ->
-    to_arealight_1(Wes, Shs, St).
+from_arealight(St) ->
+    wings_sel:map(fun(_, We) -> We#we{light=none} end, St).
 
 %%%
 %%% Convert materials to vertex colors.
@@ -900,60 +855,50 @@ weld(Ask, _) when is_atom(Ask) ->
 	    {text,1.0E-3,[{range,{1.0E-5,10.0}}]}]}],
     wings_dialog:dialog(Ask, ?__(2,"Weld"), Qs,
 			fun(Res) -> {body,{weld,Res}} end);
-weld([Tolerance], #st{shapes=Shs0,sel=Sel0}=St0) ->
-    Unselected = gb_trees:keys(Shs0) -- [ Id || {Id,_} <- Sel0 ],
-    #st{shapes=Shs}=St1 = separate(St0),
-    Selected = gb_trees:keys(Shs) -- Unselected,
-    Sel = [ {Id, gb_sets:singleton(0)} || Id <- Selected ],
-    St = combine(St1#st{sel=Sel}),
-    weld_objects(Tolerance, gb_sets:empty(), status, St).
+weld([Tolerance], St) ->
+    CF = fun(_, We) ->
+                 weld_objects(Tolerance, gb_sets:empty(), We)
+         end,
+    wings_sel:combine(CF, vertex, St).
 
-%% Cycle over St until it doesn't change. This is done because matching faces
-%% that are connected by a single vertex can't be processed (as far as I've
-%% tried). So the hope is that another face pair in the shape will be connect
-%% the next time through the St, which will make the unprocessed face pair share
-%% at least one edge. All the same, not all shapes will be processed. If there
-%% only single vert matching face pairs, then nothing can be done. If you know
-%% how to weld faces with only one common vertex, append the code at
-%% "single vertex").
-weld_objects(Tolerance, SelAcc0, Status0, St0) ->
-    Empty = gb_sets:empty(),
+%% Cycle over #we{} until it doesn't change. This is done because
+%% matching faces that are connected by a single vertex can't be
+%% processed (as far as I've tried). So the hope is that another face
+%% pair in the shape will be connect the next time through the St,
+%% which will make the unprocessed face pair share at least one
+%% edge. All the same, not all shapes will be processed. If there only
+%% single vert matching face pairs, then nothing can be done.
+
+weld_objects(Tolerance, Vs0, We0) ->
     ErrorMsg = ?__(1,"Found no faces to weld."),
-    {St1,{Sel0,Status}} =
-	wings_sel:mapfold(fun(_, We0, Acc) ->
-				  case weld_1(Tolerance, We0, Acc) of
-				      {We0,_} when SelAcc0 =:= Empty ->
-					  wings_u:error_msg(ErrorMsg);
-				      Result ->
-					  Result
-				  end
-			  end, {[],Status0}, St0),
-    case Status of
-	_ when Sel0 =:= [] -> wings_u:error_msg(ErrorMsg);
-	single_vertex when St0 =/= St1 ->
-	    [{_,Vs}] = Sel0,
-	    SelAcc = gb_sets:union(SelAcc0,Vs),
-	    weld_objects(Tolerance, SelAcc, status, St1);
-	_ ->
-	    [{Id,Vs}] = Sel0,
-	    SelAcc = gb_sets:union(SelAcc0,Vs),
-	    St = wings_sel:set(vertex, [{Id,SelAcc}], St1),
-	    {save_state,wings_sel:valid_sel(St)}
+    case weld_1(Tolerance, We0) of
+        error ->
+            case gb_sets:is_empty(Vs0) of
+                true ->
+                    wings_u:error_msg(ErrorMsg);
+                false ->
+                    {We0,Vs0}
+            end;
+        {We,Vs1} ->
+            Vs = gb_sets:union(Vs0, Vs1),
+            AllVs = wings_sel:get_all_items(vertex, We),
+            SelectedVs = gb_sets:intersection(AllVs, Vs),
+            weld_objects(Tolerance, SelectedVs, We)
     end.
 
-weld_1(Tol, #we{id=Id,fs=Fs0}=We0, {Sel,Status0}) ->
+weld_1(Tol, #we{fs=Fs0}=We0) ->
     Fs = qualified_fs(gb_trees:keys(Fs0), Tol, We0, []),
     R = sofs:relation(Fs, [{key,face}]),
     F = sofs:relation_to_family(R),
-    Part0 = sofs:range(F),
-    Part1 = sofs:specification({external,fun([_]) -> false;
-					    (_) -> true end}, Part0),
-    Part = sofs:to_external(Part1),
-    case weld_part(Part, Tol, We0, Status0) of
-	{We0,Status} ->
-	    {We0,{[{Id,gb_sets:singleton(0)}],Status}}; % Nothing to weld or done
-	{We,Status} ->
-	    {We,{[{Id,weld_selection(lists:append(Part), We0, We)}|Sel],Status}}
+    Parts0 = sofs:range(F),
+    Parts1 = sofs:specification({external,fun([_]) -> false;
+                                             (_) -> true end}, Parts0),
+    Parts = sofs:to_external(Parts1),
+    case weld_parts(Parts, Tol, We0) of
+        We0 ->
+            error;
+        We ->
+            {We,weld_selection(lists:append(Parts), We0, We)}
     end.
 
 qualified_fs([F|Fs], Tol, We, Acc) ->
@@ -968,42 +913,48 @@ qualified_fs([], _, _, Acc) -> Acc.
 
 granularize(F, Tol) -> Tol*round(F/Tol).
 
-weld_part([P|Ps], Tol, We0, Status0) ->
-    {We,Status} = weld_part_1(P, Tol, We0, Status0),
-    weld_part(Ps, Tol, We, Status);
-weld_part([], _, We, Status) -> {We,Status}.
+weld_parts([P|Ps], Tol, We0) ->
+    We = weld_part(P, Tol, We0),
+    weld_parts(Ps, Tol, We);
+weld_parts([], _, We) -> We.
 
-weld_part_1([F|Fs], Tol, We, Status) ->
-    weld_part_2(F, Fs, Tol, We, [], Status);
-weld_part_1([], _, We, Status) -> {We,Status}.
+weld_part([F|Fs], Tol, We) ->
+    weld_part_1(F, Fs, Tol, We, []);
+weld_part([], _, We) -> We.
 
-weld_part_2(Fa, [Fb|Fs], Tol, We0, Acc, Status0) ->
-    case catch try_weld(Fa, Fb, Tol, We0, Status0) of
-        {We0,Status} -> weld_part_2(Fa, Fs, Tol, We0, [Fb|Acc], Status);
-        {#we{}=We,Status} -> weld_part_1(Fs++Acc, Tol, We, Status);
-        _ ->
+weld_part_1(Fa, [Fb|Fs], Tol, We0, Acc) ->
+    try try_weld(Fa, Fb, Tol, We0) of
+        error ->
+            weld_part_1(Fa, Fs, Tol, We0, [Fb|Acc]);
+        #we{}=We0 ->
+            weld_part_1(Fa, Fs, Tol, We0, [Fb|Acc]);
+        #we{}=We ->
+            weld_part(Fs++Acc, Tol, We)
+    catch
+        error:_ ->
             weld_error()
     end;
-weld_part_2(_, [], Tol, We, Acc, Status) ->
-    weld_part_1(Acc, Tol, We, Status).
+weld_part_1(_, [], Tol, We, Acc) ->
+    weld_part(Acc, Tol, We).
 
-try_weld(Fa, Fb, Tol, We, Status) ->
+try_weld(Fa, Fb, Tol, We) ->
     Na = wings_face:normal(Fa, We),
     Nb = wings_face:normal(Fb, We),
     case e3d_vec:dot(Na, Nb) of
 	Dot when Dot < -0.99 ->
 	    case wings_face:are_neighbors(Fa, Fb, We) of
 		true ->
-		    case shared_edges(Fa,Fb,We) of
-			[] ->  %io:format("~p\n",["single vertex"]),
-			    {We,single_vertex};
+		    case shared_edges(Fa, Fb, We) of
+			[] ->
+                            error;
 			CommonEs ->
-			    {weld_neighbors(Fa, Fb, CommonEs, Tol, We),Status}
+			    weld_neighbors(Fa, Fb, CommonEs, Tol, We)
 		    end;
 		false ->
-		    {try_weld_1(Fa, Fb, Tol, We),Status}
+		    try_weld_1(Fa, Fb, Tol, We)
 	    end;
-	_Dot -> {We,Status}
+	_Dot ->
+            error
     end.
 
 shared_edges(Fa, Fb, We) ->
@@ -1021,21 +972,23 @@ get_shared_edges([]) ->
 
 weld_neighbors(Fa, Fb, CommonEs, Tol, We0) ->
     Vs0 = wings_edge:to_vertices(CommonEs,We0),
-    #we{fs=Ftab}=We1 = dissolve_edges(CommonEs,We0),
+    #we{fs=Ftab}=We1 = dissolve_edges(CommonEs, We0),
     ARemains = gb_trees:is_defined(Fa,Ftab),
     BRemains = gb_trees:is_defined(Fb,Ftab),
     case {ARemains,BRemains} of
         {true,false} ->
             case check_weld_neighbors(Fa, Vs0, Tol, We1) of
                 error ->
-                    wings_dissolve:faces([Fa,Fb],We0);
-                Other -> Other
+                    wings_dissolve:faces([Fa,Fb], We0);
+                Other ->
+                    Other
             end;
         {false,true} ->
             case check_weld_neighbors(Fb, Vs0, Tol, We1) of
                 error ->
-                    wings_dissolve:faces([Fa,Fb],We0);
-                Other -> Other
+                    wings_dissolve:faces([Fa,Fb], We0);
+                Other ->
+                    Other
             end;
         {_,_} ->
             We = wings_dissolve:faces([Fa,Fb],We0),
@@ -1045,22 +998,26 @@ weld_neighbors(Fa, Fb, CommonEs, Tol, We0) ->
                         error -> We;
                         Other -> Other
                     end;
-                _ -> We
+                _ ->
+                    We
             end
     end.
 
 check_weld_neighbors(Face, Vs0, Tol, We) ->
     Vs1 = wings_face:to_vertices([Face], We),
     if
-	Vs0 =:= Vs1 -> error;
+	Vs0 =:= Vs1 ->
+            error;
 	true ->
-	    Vs =  Vs1 -- Vs0,
+	    Vs = Vs1 -- Vs0,
 	    if
-		Vs =:= [] -> error;
+		Vs =:= [] ->
+                    error;
 		true ->
 		    NVs = Vs0 -- Vs,
 		    case get_vs_pairs(Vs, Tol, Face, We) of
-			[] -> error;
+			[] ->
+                            error;
 			CPList ->
 			    connect_and_collapse(Face, CPList, NVs, [], We)
 		    end
@@ -1213,7 +1170,8 @@ weld_selection(Fs, OldWe, We) ->
 
 weld_selection([F|Fs], OldWe, #we{fs=Ftab}=We, Acc) ->
     case gb_trees:is_defined(F, Ftab) of
-	true -> weld_selection(Fs, OldWe, We, Acc);
+	true ->
+            weld_selection(Fs, OldWe, We, Acc);
 	false ->
 	    Vs = wings_face:vertices_ccw(F, OldWe),
 	    weld_selection(Fs, OldWe, We, Vs++Acc)
