@@ -19,8 +19,6 @@
 %%%  done my own solution but if an adaptiv solution shall be implemented
 %%%  check out the paper above.
 %%%  
-%%%  wings_cc_ref: contains an erlang reference implementation which
-%%%                can be debugged. Keep it up to date.
 %%%
 
 -module(wings_cc).
@@ -292,7 +290,7 @@ calc_matmap([{Mat,Fs}|Mfs], Start, Ftab0, Mul, FMap0, Acc) ->
     calc_matmap(Mfs, Next, Ftab, Mul, FMap, 
 		[{Mat, ?GL_QUADS, Start*4, (Next-Start)*4}|Acc]);
 calc_matmap([], Total, _, _, FMap, Acc) ->
-    {Total, reverse(Acc), gb_trees:from_orddict(lists:sort(FMap))}.
+    {Total, Acc, gb_trees:from_orddict(lists:sort(FMap))}.
 
 calc_matmap_1([Id|Fs], [Vs|Ftab], Mul, Count, FMap) ->
     VSize = length(Vs),
