@@ -251,7 +251,7 @@ dialog(Ask, Title, Qs0, Fun, HelpFun) ->
     {PreviewCmd, Qs} = preview_cmd(Qs0),
     dialog_1(Ask, Title, PreviewCmd, Qs, Fun, HelpFun).
 dialog_1(Ask, Title, PreviewCmd, Qs0, Fun) when is_list(Qs0) ->
-    Qs = {vframe_dialog, Qs0, [{buttons, [ok, cancel]}]},
+    Qs = {vframe_dialog, Qs0, [{buttons, [ok, cancel]}, {position, mouse}]},
     dialog_1(Ask, Title, PreviewCmd, Qs, Fun);
 dialog_1(Ask, Title, PreviewCmd, Qs, Fun) when not is_list(Qs) ->
     case element(1,Qs) of
@@ -263,7 +263,7 @@ dialog_1(Ask, Title, PreviewCmd, Qs, Fun) when not is_list(Qs) ->
     %% io:format("Enter Dialog ~p ~p ~p~n",[Ask,PreviewCmd, Fields]),
     enter_dialog(Ask, PreviewCmd, Dialog, Fields, Fun).
 dialog_1(Ask, Title, PreviewCmd, Qs0, Fun, HelpFun) when is_list(Qs0) ->
-    Qs = {vframe_dialog, Qs0, [{buttons, [ok, cancel]},{help, HelpFun}]},
+    Qs = {vframe_dialog, Qs0, [{buttons, [ok, cancel]},{help, HelpFun}, {position, mouse}]},
     dialog_1(Ask, Title, PreviewCmd, Qs, Fun).
 
 
@@ -315,13 +315,16 @@ get_widget(Key, Store) ->
     Wx.
 
 get_value(Key, Store) ->
-    [In] = ets:lookup(Store, Key),
-    get_curr_value(In).
+    try ets:lookup(Store, Key) of
+        [In] -> get_curr_value(In)
+    catch _:_ -> ok end.
 
 set_value(Key, Value, Store) ->
-    [In] = ets:lookup(Store, Key),
-    _ = set_value_impl(In, Value, Store),
-    Store.
+    try ets:lookup(Store, Key) of
+        [In] ->
+            _ = set_value_impl(In, Value, Store),
+            Store
+    catch _:_ -> ok end.
 
 set_value_impl(#in{wx=Ctrl, type=choice}, {Def, Entries}, _) when is_list(Entries) ->
     wxChoice:clear(Ctrl),
@@ -433,15 +436,15 @@ enter_dialog(true, no_preview, Dialog, Fields, Fun) -> %% No preview cmd / modal
     set_dialog_parent(Dialog),
     case wxDialog:showModal(Dialog) of
 	?wxID_CANCEL ->
-	    true = ets:delete(element(1, Fields)),
 	    reset_dialog_parent(Dialog),
 	    wxDialog:destroy(Dialog),
+	    true = ets:delete(element(1, Fields)),
 	    keep;
 	Result ->
 	    Values = get_output(Result, Fields),
-	    true = ets:delete(element(1, Fields)),
 	    reset_dialog_parent(Dialog),
 	    wxDialog:destroy(Dialog),
+	    true = ets:delete(element(1, Fields)),
 	    return_result(Fun, Values, wings_wm:this())
     end;
 enter_dialog(true, PreviewType, Dialog, Fields, Fun) ->
@@ -462,9 +465,9 @@ enter_dialog(true, PreviewType, Dialog, Fields, Fun) ->
 			     wings_wm:psend(send_after_redraw, dialog_blanket, preview),
 			     receive
 				 closed ->
-				     true = ets:delete(element(1, Fields)),
 				     reset_dialog_parent(Dialog),
-				     wxDialog:destroy(Dialog)
+				     wxDialog:destroy(Dialog),
+                                     true = ets:delete(element(1, Fields))
 			     end
 		     end),
     State = #eh{fs=Fields, apply=Fun, owner=wings_wm:this(),
@@ -713,9 +716,14 @@ setup_hook(#in{key=Key, wx=Ctrl, type=table, hook=UserHook}, Fields) ->
 setup_hook(#in{wx=Canvas, type=custom_gl, hook=CustomRedraw}, Fields) ->
     Env = wx:get_env(),
     Custom = fun() ->
-		     wxGLCanvas:setCurrent(Canvas),
-		     CustomRedraw(Canvas, Fields),
-		     wxGLCanvas:swapBuffers(Canvas)
+                     try
+                         wxGLCanvas:setCurrent(Canvas),
+                         CustomRedraw(Canvas, Fields),
+                         wxGLCanvas:swapBuffers(Canvas)
+                     catch _:Reason ->
+                             io:format("GL Custom callback crashed ~p~n~p~n",
+                                       [Reason, erlang:get_stacktrace()])
+                     end
 	     end,
     Redraw = fun(#wx{}, _) ->
 		     case os:type() of
@@ -764,7 +772,7 @@ label_col({Label,Def}) -> {Label, {text,Def}};
 label_col({Label,Def,Flags}) -> {Label, {text,Def, Flags}}.
 
 build_dialog(false, _Title, Qs) ->
-    {DialogData,_} = build(false, Qs, undefined, undefined),
+    {_,DialogData,_} = build(false, Qs, undefined, undefined),
     {undefined, DialogData};
 build_dialog(AskType, Title, Qs) ->
     wx:batch(fun() ->
@@ -781,7 +789,7 @@ build_dialog(AskType, Title, Qs) ->
 		     Top    = wxBoxSizer:new(?wxVERTICAL),
 		     Sizer  = wxBoxSizer:new(?wxVERTICAL),
 		     try build(AskType, Qs, Panel, Sizer) of
-			 {DialogData, Fs} ->
+			 {Location, DialogData, Fs} ->
 			     set_keyboard_focus(Dialog, Fs),
 			     wxWindow:setSizer(Panel, Sizer),
 			     wxSizer:add(Top, Panel, [{proportion, 1},
@@ -789,6 +797,7 @@ build_dialog(AskType, Title, Qs) ->
 						      {border, 5}]),
 			     setup_buttons(Dialog, Top, DialogData),
 			     wxWindow:setSizerAndFit(Dialog, Top),
+                             set_position(Location, Dialog),
 			     setup_hooks(DialogData),
 			     {Dialog, DialogData}
 		     catch Class:Reason ->
@@ -820,18 +829,39 @@ set_keyboard_focus(Dialog, Fields) ->
 	    ok
     end.
 
+set_position(mouse, Dialog) ->
+    {Xm,Ym} = wx_misc:getMousePosition(),
+    {Wd, Hd} = wxWindow:getSize(Dialog),
+    Ws = wxSystemSettings:getMetric(?wxSYS_SCREEN_X),
+    Hs = wxSystemSettings:getMetric(?wxSYS_SCREEN_Y),
+    if (Xm+Wd) < Ws, (Ym+Hd) < Hs ->
+            wxWindow:move(Dialog, max(Xm-100, 0), max(Ym-50, 0));
+       (Xm+Wd) < Ws ->
+            wxWindow:move(Dialog, max(Xm-100, 0), max(Hs-Hd-50, 0));
+       (Ym+Hd) < Hs ->
+            wxWindow:move(Dialog, max(Ws-Wd-100, 0), max(Ym-50, 0));
+       true ->
+            io:format("~p ~p~n",[{Xm,Wd,Ws},{Ym,Hd,Hs}]),
+            ok
+    end;
+set_position(center, Dialog) ->
+    wxTopLevelWindow:centerOnScreen(Dialog);
+set_position(_, _Dialog) ->
+    ok.
+
 build(Ask, Qs, Parent, Sizer) ->
-    Fields = build(Ask, Qs, Parent, Sizer, []),
+    {Location,Fields} = build(Ask, Qs, Parent, Sizer, []),
     {Fs, _} = lists:mapfoldl(fun(In=#in{key=undefined},N) -> {In#in{key=N}, N+1};
 				(In=#in{}, N) -> {In, N+1}
 			     end, 1, lists:reverse(Fields)),
     Table = ets:new(?MODULE, [{keypos, #in.key}, public]),
     true = ets:insert(Table, Fs),
-    {{Table, [Key || #in{key=Key} <- Fs]}, Fs}.
+    {Location, {Table, [Key || #in{key=Key} <- Fs]}, Fs}.
 
 
 build(Ask, {vframe_dialog, Qs, Flags}, Parent, Sizer, []) ->
     Def = proplists:get_value(value, Flags, ?wxID_OK),
+    Location = proplists:get_value(position, Flags, {-1,-1}),
     Buttons = proplists:get_value(buttons, Flags, [ok, cancel]),
     HelpFun = proplists:get_value(help, Flags, undefined),
     ButtMask = lists:foldl(fun(ok, Butts)     -> ?wxOK bor Butts;
@@ -870,9 +900,9 @@ build(Ask, {vframe_dialog, Qs, Flags}, Parent, Sizer, []) ->
 		     Ok
 	     end,
     In = build(Ask, {vframe, Qs, [{proportion,1}]}, Parent, Sizer, []),
-    [#in{key=proplists:get_value(key,Flags), def=Def,
-	 output= undefined =/= proplists:get_value(key,Flags),
-	 type=dialog_buttons, wx=Create}|In];
+    {Location, [#in{key=proplists:get_value(key,Flags), def=Def,
+		     output= undefined =/= proplists:get_value(key,Flags),
+		     type=dialog_buttons, wx=Create}|In]};
 
 build(Ask, {oframe, Tabs, Def, Flags}, Parent, WinSizer, In0)
   when Ask =/= false ->
