@@ -48,7 +48,7 @@ refresh_dlists(St) ->
     invalidate_dlists(St),
     build_dlists(St),
     update_sel_dlist(),
-    wings_develop:gl_error_check("Refresh of display lists").
+    wings_develop:state_check(St, "Refresh of display lists").
 
 invalidate_dlists(#st{selmode=Mode,sel=Sel}=St) ->
     prepare_dlists(St),
@@ -905,20 +905,6 @@ draw_mat_faces(Vab, Extra, MatGroups, Mtab) ->
     {call,D,Vab}.
 
 do_draw_mat_faces(MatGroups, Mtab, ActiveColor) ->
-    %% Setup shader for materials
-    Progs = get(light_shaders),
-    UseSceneLights = wings_pref:get_value(scene_lights) andalso
-	wings_light:any_enabled_lights(),
-    UseShaders = Progs =/= undefined andalso not UseSceneLights
-	andalso wings_pref:get_value(number_of_lights) =:= 2,
-    case UseShaders of
-	false -> ignore;
-	_ ->
-	    NumShaders = wings_pref:get_value(active_shader),
-	    {Prog,_Name} = element(NumShaders, Progs),
-	    put(active_shader, Prog),
-	    ok
-    end,
     %% Show materials.
     foreach(
       fun({Mat,Type,Start,NumElements}) ->
@@ -929,7 +915,6 @@ do_draw_mat_faces(MatGroups, Mtab, ActiveColor) ->
 	      DeApply(),
 	      gl:popAttrib()
       end, MatGroups),
-    put(active_shader, 0),
     ok.
 
 %%
