@@ -146,14 +146,14 @@ command(connect, St) ->
 command(connect_slide, St) ->
     connect_slide(St);
 command(clean_dissolve, St) ->
-    {save_state,wings_shape:update_folders(clean_dissolve(St))};
+    {save_state,clean_dissolve(St)};
 command(dissolve, St) ->
-    {save_state,wings_shape:update_folders(dissolve(St))};
+    {save_state,dissolve(St)};
 command(collapse, St) ->
     {save_state, wings_collapse:uniform_collapse(St)};
 command(clean_collapse, St0) ->
     St = wings_collapse:clean_uniform_collapse(St0),
-    {save_state,wings_shape:update_folders(St)};
+    {save_state,St};
 command({hardness,Type}, St) ->
     {save_state,hardness(Type, St)};
 command(loop_cut, St) ->
@@ -176,22 +176,20 @@ command(vertex_color, St) ->
 %%% The Connect command.
 %%%
 
-connect(St0) ->
-    {St,Sel} = wings_sel:mapfold(fun connect/3, [], St0),
-    wings_sel:set(Sel, St).
+connect(St) ->
+    wings_sel:map_update_sel(fun connect/2, St).
 
-connect(Es0, #we{id=Id}=We0, Acc) ->
+connect(Es0, We0) ->
     Es1 = gb_sets:to_list(Es0),
     Es = remove_nonconnectable(Es1, Es0, We0, []),
     {Vs,We1} = cut_edges(Es, We0),
     We2 = wings_vertex_cmd:connect(Vs, We1),
     Sel = wings_we:new_items_as_gbset(edge, We1, We2),
     We = wings_edge:dissolve_isolated_vs(Vs, We2),
-    {We,[{Id,Sel}|Acc]}.
+    {We,Sel}.
 
-connect_slide(St1) ->
-    {St0,Sel} = wings_sel:mapfold(fun connect/3, [], St1),
-    St=wings_sel:set(Sel, St0),
+connect_slide(St0) ->
+    St = wings_sel:map_update_sel(fun connect/2, St0),
     slide(St).
 
 cut_edges(Es, We) ->
@@ -231,14 +229,13 @@ set_color(Color, St) ->
 %%% The Cut command.
 %%%
 
-cut(N, #st{selmode=edge}=St0) when N > 1 ->
-    {St,Sel} = wings_sel:mapfold(
-		 fun(Edges, #we{id=Id}=We0, Acc) ->
-			 We = cut_edges(Edges, N, We0),
-			 S = wings_we:new_items_as_gbset(vertex, We0, We),
-			 {We,[{Id,S}|Acc]}
-		 end, [], St0),
-    wings_sel:set(vertex, Sel, St);
+cut(N, #st{selmode=edge}=St) when N > 1 ->
+    wings_sel:map_update_sel(
+      fun(Edges, We0) ->
+	      We = cut_edges(Edges, N, We0),
+	      S = wings_we:new_items_as_gbset(vertex, We0, We),
+	      {We,S}
+      end, vertex, St);
 cut(_, St) -> St.
 
 cut_edges(Edges, N, We0) ->
@@ -251,25 +248,26 @@ cut_edges(Edges, N, We0) ->
 %%% Cut at an arbitrary position.
 %%%
 
-cut_pick(St) ->
-    {Tvs,Sel} = wings_sel:fold(
-		  fun(Es, We, []) ->
-			  case gb_sets:to_list(Es) of
-			      [E] -> cut_pick_make_tvs(E, We);
-			      _ -> cut_pick_error()
-			  end;
-		     (_, _, _) ->
-			  cut_pick_error()
-		  end, [], St),
+cut_pick(#st{sel=[_]}=St0) ->
+    MF = fun(Es, We) ->
+                 case gb_sets:to_list(Es) of
+                     [E] -> cut_pick_make_tvs(E, We);
+                     _ -> cut_pick_error()
+                 end
+         end,
+    St = wings_sel:map_update_sel(MF, vertex, St0),
     Units = [{percent,{0.0,1.0}}],
     Flags = [{initial,[0]}],
-    wings_drag:setup(Tvs, Units, Flags, wings_sel:set(vertex, Sel, St)).
+    DF = fun(#we{temp=General}) -> General end,
+    wings_drag:general(DF, Units, Flags, St);
+cut_pick(#st{}) ->
+    cut_pick_error().
 
 -spec cut_pick_error() -> no_return().
 cut_pick_error() ->
     wings_u:error_msg(?__(1,"Only one edge can be cut at an arbitrary position.")).
 
-cut_pick_make_tvs(Edge, #we{id=Id,es=Etab,vp=Vtab,next_id=NewV}=We) ->
+cut_pick_make_tvs(Edge, #we{es=Etab,vp=Vtab,next_id=NewV}=We) ->
     #edge{vs=Va,ve=Vb} = array:get(Edge, Etab),
     Start = array:get(Va, Vtab),
     End = array:get(Vb, Vtab),
@@ -283,8 +281,8 @@ cut_pick_make_tvs(Edge, #we{id=Id,es=Etab,vp=Vtab,next_id=NewV}=We) ->
 	     2#10000010,
 	     2#01111100>>},
     Fun = fun(I, D) -> cut_pick_marker(I, D, Edge, We, Start, Dir, Char) end,
-    Sel = [{Id,gb_sets:singleton(NewV)}],
-    {{general,[{Id,Fun}]},Sel}.
+    Sel = gb_sets:singleton(NewV),
+    {We#we{temp=Fun},Sel}.
 
 cut_pick_marker([I], D, Edge, We0, Start, Dir, Char) ->
     {X,Y,Z} = Pos = e3d_vec:add_prod(Start, Dir, I),
@@ -380,24 +378,35 @@ hardness(hard, St) ->
 %%% The Slide command.
 %%%
 
-slide(St) ->
+slide(St0) ->
     Mode = wings_pref:get_value(slide_mode, relative),
     Stop = wings_pref:get_value(slide_stop, false),
     State = {Mode,none,Stop},
     SUp = SDown = SN = SBi = {0.0,0.0,0.0},
-    {Tvs,_,_,_,_,MinUp,MinDw} =
-	wings_sel:fold(
-	  fun(EsSet, #we{id=Id} = We, {Acc,Up0,Dw0,N0,Bi0,MinUp,MinDw}) ->
+
+    %% FIXME: The use of the process dicationary (wings_slide) will
+    %% not work when each #we{} are stored in its own process.
+    %%
+    %% FIXME: Someone who understands the Up, Dw, N, and Bi parameters
+    %% should rewrite this code in a way that can be parallelized
+    %% (avoid the use of wings_sel:mapfold/3, since it forces
+    %% sequential evaluation in each process).
+
+    {St,{_,_,_,_,MinUp,MinDw}} =
+	wings_sel:mapfold(
+	  fun(EsSet, We, {Up0,Dw0,N0,Bi0,MinUp,MinDw}) ->
 		  LofEs0 = wings_edge_loop:partition_edges(EsSet, We),
 		  LofEs = reverse(sort([{length(Es),Es} || Es <- LofEs0])),
 		  {{Slides,MUp,MDw},Up,Dw,N,Bi} =
 		      slide_setup_edges(LofEs,Up0,Dw0,N0,Bi0,We,
 					{gb_trees:empty(),MinUp,MinDw}),
-		  {[{Id,make_slide_tv(Slides, State)}|Acc],Up,Dw,N,Bi,MUp,MDw}
-	  end, {[], SUp, SDown, SN, SBi, unknown,unknown}, St),
+		  {We#we{temp=make_slide_tv(Slides, State)},
+                   {Up,Dw,N,Bi,MUp,MDw}}
+	  end, {SUp, SDown, SN, SBi, unknown,unknown}, St0),
     Units = slide_units(State,MinUp,MinDw),
     Flags = [{mode,{slide_mode(MinUp,MinDw),State}},{initial,[0]}],
-    wings_drag:setup(Tvs, Units, Flags, St).
+    DF = fun(_, #we{temp=Tv}) -> Tv end,
+    wings_drag:fold(DF, Units, Flags, St).
 
 slide_mode(MinUp,MinDw) ->
     fun(help, State)		  ->	slide_help(State);
@@ -630,16 +639,16 @@ add_slide_vertex(V,{Vpos,{Ndir,NL},{Pdir,PL}},Acc) ->
 %%% The Loop Cut command.
 %%%
 
-loop_cut(St0) ->
-    {Sel,St} = wings_sel:fold(fun loop_cut/3, {[],St0}, St0),
-    wings_sel:set(body, Sel, St).
+loop_cut(St) ->
+    wings_sel:clone(fun loop_cut/2, body, St).
 
-loop_cut(Edges, #we{name=Name,id=Id,fs=Ftab}=We0, {Sel,St0}) ->
+loop_cut(Edges, #we{id=Id,fs=Ftab}=We0) ->
     AdjFaces = wings_face:from_edges(Edges, We0),
     case loop_cut_partition(AdjFaces, Edges, We0, []) of
 	[_] ->
-	    wings_u:error_msg(?__(1,"Edge loop doesn't divide ~p into two (or more) parts."),
-			  [Name]);
+	    wings_u:error_msg(?__(1,"Edge loop doesn't divide object #~p "
+                                  "into two (or more) parts."),
+                              [Id]);
 	Parts0 ->
 	    %% We arbitrarily decide that the largest part of the object
 	    %% will be left unselected and will keep the name of the object.
@@ -657,17 +666,15 @@ loop_cut(Edges, #we{name=Name,id=Id,fs=Ftab}=We0, {Sel,St0}) ->
 	    First = ordsets:subtract(gb_trees:keys(Ftab), FirstComplement),
 
 	    We = wings_dissolve:complement(First, We0),
-	    Shs = St0#st.shapes,
-	    St = St0#st{shapes=gb_trees:update(Id, We, Shs)},
-	    loop_cut_make_copies(Parts, We0, Sel, St)
+            New = loop_cut_make_copies(Parts, We0),
+            {We,gb_sets:empty(),New}
     end.
 
-loop_cut_make_copies([P|Parts], We0, Sel0, #st{onext=Id}=St0) ->
-    Sel = [{Id,gb_sets:singleton(0)}|Sel0],
+loop_cut_make_copies([P|Parts], We0) ->
+    Sel = gb_sets:singleton(0),
     We = wings_dissolve:complement(P, We0),
-    St = wings_shape:insert(We, cut, St0),
-    loop_cut_make_copies(Parts, We0, Sel, St);
-loop_cut_make_copies([], _, Sel, St) -> {Sel,St}.
+    [{We,Sel,cut}|loop_cut_make_copies(Parts, We0)];
+loop_cut_make_copies([], _) -> [].
 
 loop_cut_partition(Faces0, Edges, We, Acc) ->
     case gb_sets:is_empty(Faces0) of
