@@ -14,7 +14,7 @@
 -module(wings_file).
 -export([init/0,init_autosave/0,menu/0,command/2]).
 -export([import_filename/2,export_filename/2,export_filename/3]).
--export([unsaved_filename/0,autosave_filename/1]).
+-export([unsaved_filename/0,del_unsaved_file/0,autosave_filename/1]).
 -export([file_filters/1]).
 
 -include("wings.hrl").
@@ -303,9 +303,10 @@ confirmed_new(#st{file=File}=St) ->
 new(#st{saved=true}=St0) ->
     St1 = clean_st(St0#st{file=undefined}),
     %% clean_st/1 will remove all saved view, but will not reset the view. For a new project we should reset it.
+    wings_frame:reinit_layout(),
     wings_view:reset(),
     St2 = clean_images(wings_undo:init(St1)),
-    St = wings_shape:create_folder_system(St2),
+    St = wings_obj:create_folder_system(St2),
     wings_u:caption(St),
     {new,St#st{saved=true}};
 new(#st{}=St0) ->		      %File is not saved or autosaved.
@@ -314,14 +315,18 @@ new(#st{}=St0) ->		      %File is not saved or autosaved.
 			  fun() -> {file,{save,{file,new}}} end,
 			  fun() -> {file,confirmed_new} end).
 
-open(#st{saved=true}) ->
-    confirmed_open_dialog();
-open(St) ->
-    wings_u:caption(St#st{saved=false}),		%Clear any autosave flag.
-    Confirmed = {file,confirmed_open_dialog},
-    wings_u:yes_no_cancel(str_save_changes(),
-			  fun() -> {file,{save,Confirmed}} end,
-			  fun() -> Confirmed end).
+open(#st{saved=Saved}=St) ->
+    case Saved =:= true orelse wings_obj:num_objects(St) =:= 0 of
+        true ->
+            confirmed_open_dialog();
+        false ->
+            %% Clear any autosave flag.
+            wings_u:caption(St#st{saved=false}),
+            Confirmed = {file,confirmed_open_dialog},
+            wings_u:yes_no_cancel(str_save_changes(),
+                                  fun() -> {file,{save,Confirmed}} end,
+                                  fun() -> Confirmed end)
+    end.
 
 confirmed_open_dialog() ->
     %% All confirmation questions asked. The former contents has either
@@ -344,12 +349,13 @@ confirmed_open(Name, St0) ->
 		  %%   Name: Original name of file to be opened.
 		  %%   File: Either original file or the autosave file
 		  St1 = clean_st(St0#st{file=undefined}),
-		  St2 = wings_shape:create_folder_system(wings_undo:init(St1)),
+		  wings_frame:reinit_layout(),
+		  St2 = wings_obj:create_folder_system(wings_undo:init(St1)),
 		  case ?SLOW(wings_ff_wings:import(File, St2)) of
 		      #st{}=St3 ->
 			  set_cwd(dirname(File)),
 			  St4 = clean_images(St3),
-			  St = wings_shape:recreate_folder_system(St4),
+			  St = wings_obj:recreate_folder_system(St4),
 			  add_recent(Name),
 			  wings_u:caption(St#st{saved=true,file=Name});
 		      {error,Reason} ->
@@ -359,14 +365,18 @@ confirmed_open(Name, St0) ->
 	  end,
     use_autosave(Name, Fun).
 
-named_open(Name, #st{saved=true}=St) ->
-    confirmed_open(Name, St);
-named_open(Name, St) ->
-    wings_u:caption(St#st{saved=false}),		%Clear any autosave flag.
-    Confirmed = {file,{confirmed_open,Name}},
-    wings_u:yes_no_cancel(str_save_changes(),
-			  fun() -> {file,{save,Confirmed}} end,
-			  fun() -> Confirmed end).
+named_open(Name, #st{saved=Saved}=St) ->
+    case Saved =:= true orelse wings_obj:num_objects(St) =:= 0 of
+        true ->
+            confirmed_open(Name, St);
+        false ->
+            %%Clear any autosave flag.
+            wings_u:caption(St#st{saved=false}),
+            Confirmed = {file,{confirmed_open,Name}},
+            wings_u:yes_no_cancel(str_save_changes(),
+                                  fun() -> {file,{save,Confirmed}} end,
+                                  fun() -> Confirmed end)
+    end.
 
 str_save_changes() ->
     ?__(1,"Do you want to save your changes?").
@@ -394,7 +404,7 @@ merge(Name, St0) ->
 		      #st{}=St ->
 			  set_cwd(dirname(Name)),
 			  wings_u:caption(St#st{saved=false}),
-			  wings_shape:recreate_folder_system(St#st{saved=false})
+			  wings_obj:recreate_folder_system(St#st{saved=false})
 		  end
 	  end,
     use_autosave(Name, Fun).
@@ -541,7 +551,8 @@ use_autosave_1(#file_info{mtime=SaveTime0}, File, Body) ->
 	    AutoTime = calendar:datetime_to_gregorian_seconds(AutoInfo0),
 	    if
 		AutoTime > SaveTime ->
-		    Msg = ?__(1,"An autosaved file with a later time stamp exists; do you want to load the autosaved file instead?"),
+		    Msg = ?__(1,"An autosaved file with a later time stamp exists;"
+                              " do you want to load the autosaved file instead?"),
 		    wings_u:yes_no(Msg, autosave_fun(Body, Auto),
 				   autosave_fun(Body, File));
 		true ->
@@ -606,6 +617,7 @@ autosave(#st{file=Name}=St) ->
     %% Fix this later
     View = wings_wm:get_prop(geom, current_view),
     wings_view:set_current(View),
+    filelib:ensure_dir(Auto),
     case ?SLOW(wings_ff_wings:export(Auto, St)) of
 	ok ->
 	    wings_u:caption(St#st{saved=auto});
@@ -678,10 +690,10 @@ recent_files_1([], _, _, Tail) -> Tail.
 
 revert(#st{file=undefined}=St) -> St;
 revert(#st{file=File}=St0) ->
-    St1 = wings_shape:create_folder_system(clean_st(St0)),
+    St1 = wings_obj:create_folder_system(clean_st(St0)),
     case ?SLOW(wings_ff_wings:import(File, St1)) of
 	#st{}=St2 ->
-	    St = wings_shape:recreate_folder_system(St2),
+	    St = wings_obj:recreate_folder_system(St2),
 	    clean_images(St);
 	{error,_}=Error ->
 	    Error
@@ -714,15 +726,19 @@ import_image(Name) ->
     case wings_image:from_file(Name) of
 	Im when is_integer(Im) ->
 	    keep;
-	{error,Error} ->
-		case Error of
-		100902 -> % GLU_OUT_OF_MEMORY
-			wings_u:error_msg(?__(2,"The image cannot be loaded.~nFile: \"~s\"~n GLU Error: ~p - ~s~n"),
-				  	[Name,Error, glu:errorString(Error)]);
-		_ ->
-	    	wings_u:error_msg(?__(1,"Failed to load \"~s\": ~s\n"),
-				     [Name,file:format_error(Error)])
-		end
+	{error,100902=Error} ->  % GLU_OUT_OF_MEMORY
+            wings_u:error_msg(?__(2,"The image cannot be loaded.~nFile: "
+                                  "\"~ts\"~n GLU Error: ~p - ~s~n"),
+                              [Name, Error, glu:errorString(Error)]);
+        {error,Error} ->
+            case file:format_error(Error) of
+                "unknown" ++ _ ->
+                    wings_u:error_msg(?__(1,"Failed to load") ++ " \"~ts\": ~p\n",
+                                      [Name,Error]);
+                ErrStr ->
+                    wings_u:error_msg(?__(1,"Failed to load") ++ " \"~ts\": ~s\n",
+                                      [Name,ErrStr])
+            end
     end.
 
 %%
@@ -814,7 +830,7 @@ clean_st(St) ->
 				sel=[],ssels=Empty,saved=Limit}).
 
 clean_images(#st{saved=Limit}=St) when is_integer(Limit) ->
-    wings_dl:map(fun(D, _) -> D#dlo{proxy_data=none} end, []),
+    wings_dl:map(fun(D, _) -> D#dlo{proxy_data=none, proxy=false} end, []),
     wings_image:delete_older(Limit),
     St#st{saved=false}.
 
