@@ -12,29 +12,30 @@
 %%
 
 -module(wings_image).
--export([init/1,
-	 from_file/1,new/2,new_temp/2,new_hidden/2, create/1,
+-export([from_file/1,new/2,new_temp/2,new_hidden/2, create/1,
 	 rename/2,txid/1,info/1,images/0,
 	 screenshot/2,screenshot/1,viewport_screenshot/1,
-	 bumpid/1, default/1,
-	 is_normalmap/1, normal_cubemapid/0,
+	 bumpid/1,
+	 is_normalmap/1,
 	 next_id/0,delete_older/1,delete_from/1,delete/1,
-	 update/2,update_filename/2,
-	 window/1]).
+         filter_images/1,
+	 update/2,update_filename/2,find_image/2,
+	 window/1, debug_display/2]).
 -export([image_formats/0,image_read/1,image_write/1,
 	 e3d_to_wxImage/1, wxImage_to_e3d/1]).
--export([loop/1]).
 -export([maybe_exceds_opengl_caps/1]).
 
+-behavior(gen_server).
+-export([start_link/0, init/1, handle_call/3, handle_cast/2]).
+
 -define(NEED_OPENGL, 1).
 -include("wings.hrl").
 -include("e3d_image.hrl").
--import(lists, [reverse/1,foreach/2,flatten/1]).
+-import(lists, [reverse/1]).
 
--define(DEFAULT, '$Default Img').
-
-init(Opt) ->
-    spawn_opt(fun() -> server(Opt) end, [link,{fullsweep_after,0}]).
+start_link() ->
+    Env = wings_io:get_process_option(),
+    gen_server:start_link({local, ?MODULE}, ?MODULE, [Env], [{spawn_opt, [{fullsweep_after,0}]}]).
 
 %%%
 %%% Interface against plug-ins.
@@ -47,43 +48,15 @@ image_read(Ps) ->
     OptDir   = proplists:get_value(opt_dir,  Ps, undefined),
     AbsFile  = proplists:get_value(filename, Ps),
     FileName = filename:basename(AbsFile),
-    Dirs     = filename:split(filename:dirname(AbsFile)),    
+    Dirs     = filename:split(filename:dirname(AbsFile)),
     case wings_plugin:call_ui({image,read,Ps}) of
-	{error, enoent} -> 	    
+	{error, enoent} ->
 	    try_relative_paths(Dirs,FileName,OptDir,CurrDir,Ps);
 	Ok = #e3d_image{} ->
 	    Ok#e3d_image{filename=AbsFile};
 	Other ->
 	    Other
     end.
-		
-try_relative_paths(Dirs,FileName,undefined,CurrDir,Ps) ->
-    try_relative_paths(CurrDir,Dirs,FileName,Ps);
-try_relative_paths(Dirs,FileName,OptDir,CurrDir,Ps) ->
-    case try_relative_paths(OptDir,Dirs,FileName,Ps) of
-	{error, enoent} ->
-	    try_relative_paths(Dirs,FileName,undefined,CurrDir,Ps);
-	Other -> 
-	    Other
-    end.
-
-try_relative_paths(Start, Rel, File, Ps0) ->
-    Abs = case Rel of 
-	      [] -> filename:join(Start,File);
-	      _  -> filename:join(filename:join(Start,filename:join(Rel)),File)
-	  end,
-    Ps = lists:keyreplace(filename,1,Ps0,{filename,Abs}),
-    case wings_plugin:call_ui({image,read,Ps}) of
-	Err = {error, enoent} ->
-	    case Rel of
-		[] -> Err;
-		[_|Rest] -> try_relative_paths(Start,Rest,File,Ps0)
-	    end;
-	Ok = #e3d_image{} ->
-	    Ok#e3d_image{filename=Abs};
-	Other ->
-	    Other
-    end.
 
 image_write(Ps) ->
     case catch wings_plugin:call_ui({image,write,Ps}) of
@@ -92,38 +65,8 @@ image_write(Ps) ->
 	Result -> Result
     end.
 
-e3d_to_wxImage(I = #e3d_image{bytes_pp=4, width=W, height=H}) ->
-    #e3d_image{image=RGB} = e3d_image:convert(I, r8g8b8, 1, upper_left),
-    Wx = wxImage:new(W,H,RGB),
-    #e3d_image{image=Alpha} = e3d_image:convert(I, a8, 1, upper_left),
-    wxImage:setAlpha(Wx, Alpha),
-    Wx;
-e3d_to_wxImage(I = #e3d_image{bytes_pp=3, width=W, height=H}) ->
-    #e3d_image{image=RGB} = e3d_image:convert(I, r8g8b8, 1, upper_left),
-    wxImage:new(W,H,RGB);
-e3d_to_wxImage(I = #e3d_image{bytes_pp=1, width=W, height=H}) ->
-    #e3d_image{image=RGB} = e3d_image:convert(I, r8g8b8, 1, upper_left),
-    wxImage:new(W,H,RGB).
-
-wxImage_to_e3d(Wx) ->
-    wxImage = wx:getObjectType(Wx), %% Assert
-    E3d0 = #e3d_image{image=wxImage:getData(Wx),
-		      width=wxImage:getWidth(Wx),
-		      height=wxImage:getHeight(Wx),
-		      order = upper_left
-		     },
-    case wxImage:hasAlpha(Wx) of
-	true ->
-            e3d_image:add_alpha(E3d0, wxImage:getAlpha(Wx));
-	false ->
-            case wxImage:hasMask(Wx) of
-                true ->
-                    wxImage:initAlpha(Wx),
-                    e3d_image:add_alpha(E3d0, wxImage:getAlpha(Wx));
-                false ->
-                    E3d0
-            end
-    end.
+e3d_to_wxImage(I) ->
+    e3d_to_wxImage_1(I).
 
 %%%
 %%% Client API.
@@ -147,6 +90,9 @@ new_temp(Name, E3DImage) ->
 new_hidden(Name, E3DImage) ->
     req({new,E3DImage#e3d_image{name=Name},true, true}).
 
+debug_display(Id, Img) ->
+    Display = fun(_) -> wings_image_viewer:new({image, Id}, Img), keep end,
+    wings ! {external, Display}.
 
 create(St) ->
     create_image(),
@@ -158,6 +104,54 @@ rename(Id, NewName) ->
 txid(Id) ->
     req({txid,Id}, false).
 
+is_normalmap(Id) ->
+    req({is_normalmap,Id},false).
+
+bumpid(Id) ->
+    req({bumpid,Id}, false).
+
+info(Id) ->
+    req({info,Id}, false).
+
+images() ->
+    req(images, false).
+
+filter_images(Bool) ->
+    req({filter_images, Bool}, false).
+
+next_id() ->
+    req(next_id, false).
+
+delete_older(Id) ->
+    req({delete_older,Id}).
+
+delete_from(Id) ->
+    req({delete_from,Id}).
+
+delete(Id) ->
+    req({delete,Id}).
+
+update(Id, Image) ->
+    req({update,Id,Image}).
+
+update_filename(Id, Filename) ->
+    gen_server:cast(?MODULE, {update_filename,Id,Filename}).
+
+find_image(Dir, Filename) ->
+    req({find_image,Dir,Filename}, false).
+
+req(Req) ->
+    req(Req, true).
+
+req(Req, Notify) ->
+    Reply = gen_server:call(?MODULE, Req),
+    Running = get(wings_not_running) == undefined,
+    case Notify andalso Running of
+        false -> ok;
+        true -> wings_wm:notify(image_change)
+    end,
+    Reply.
+
 screenshot(Ask, _) when is_atom(Ask) ->
     ViewPortOnly = wings_pref:get_value(screenshot_viewport_only, false),
     SaveView = wings_pref:get_value(screenshot_save_current_view, false),
@@ -205,192 +199,134 @@ viewport_screenshot(Name) ->
     Id = new_temp(Name, Image),
     window(Id).
 
-is_normalmap(Id) ->
-    req({is_normalmap,Id},false).
-
-bumpid(Id) ->
-    req({bumpid,Id}, false).
-
-normal_cubemapid() ->
-    req(normalCM, false).
-
-default(Type) ->
-    req({default,Type}, false).
-
-info(Id) ->
-    req({info,Id}, false).
-
-images() ->
-    req(images, false).
-
-next_id() ->
-    req(next_id, false).
-
-delete_older(Id) ->
-    req({delete_older,Id}).
-
-delete_from(Id) ->
-    req({delete_from,Id}).
-
-delete(Id) ->
-    req({delete,Id}).
-
-update(Id, Image) ->
-    req({update,Id,Image}).
-
-update_filename(Id, Filename) ->
-    req({update_filename,Id,Filename}).
-
-req(Req) ->
-    req(Req, true).
-
-req(Req, Notify) ->
-    Self = self(),
-    Ref = make_ref(),
-    wings_image ! {Self,Ref,Req},
-    receive
-	{Ref,Answer} ->
-	    Running = get(wings_not_running) == undefined,
-	    case Notify andalso Running of
-		false -> ok;
-		true -> wings_wm:notify(image_change)
-	    end,
-	    Answer
-    end.
-
 %%%
 %%% Server implementation.
 %%%
 %%% Reason for using a server: Convenient encapsulation of images.
 %%% We'll get an entire process dictionary to use for texture ids.
-%%% 
+%%%
 
 -record(ist,
 	{next=0,				%Next image ID.
 	 images					%All images (gb_trees).
 	}).
 
-server(Opt) ->
-    register(wings_image, self()),
+init([Env]) ->
     process_flag(trap_exit, true),
-    case Opt of
-	wings_not_running ->
-	    put(wings_not_running, true);
-	_ ->
-	    wings_io:set_process_option(Opt),
-	    init_background_tx()
-    end,
-    loop(#ist{images=gb_trees:empty()}).
+    wings_io:set_process_option(Env),
+    {ok, #ist{images=gb_trees:empty()}}.
 
-loop(S0) ->
-    receive
-	{'EXIT', _Wings, _} ->
-	    %% Time to die
-	    exit(normal);
-	{Client,Ref,Req} ->
-	    case handle(Req, S0) of
-		{{error,_GlErr}=Err,S} ->
-		    Client ! {Ref,Err};
-		#ist{}=S ->
-		    Client ! {Ref,ok};
-		{Resp,S} ->
-		    Client ! {Ref,Resp}
-	    end,
-	    loop(S);
-	reload ->
-	    ?MODULE:loop(S0);
-	Other ->
-	    exit({bad_message_to_wings_image,Other})
-    end.
-
-handle({new,#e3d_image{name=Name0}=Im0,false,Hide}, #ist{next=Id,images=Images0}=S) ->
-    Name = make_unique(Name0, Images0),
-    Im = maybe_convert(Im0#e3d_image{name=Name}),
-    Images = gb_trees:insert(Id, hide(Im, Hide), Images0),
-    case make_texture(Id, Im) of
-	{error,_GlErr}=Err ->
-	    {Err,S};
-	_ ->
-	    {Id,S#ist{next=Id+1,images=Images}}
+handle_call({new,#e3d_image{name=Name0}=Im0,false,Hide}, _From, #ist{next=Id,images=Images0}=S) ->
+    try texture_format(Im0) of
+        _ ->
+            Name = make_unique(Name0, Images0),
+            Im = maybe_convert(Im0#e3d_image{name=Name}),
+            Images = gb_trees:insert(Id, hide(Im, Hide), Images0),
+            case make_texture(Id, Im) of
+                {error,_GlErr}=Err ->
+                    {reply, Err, S};
+                _ ->
+                    {reply, Id, S#ist{next=Id+1,images=Images}}
+            end
+    catch _:_ -> %% texture_format is used to check validity
+            {reply, {error, unknown_image_format}, S}
     end;
-handle({new,#e3d_image{name=Name}=Im,true,Hide}, #ist{images=Images}=S0) ->
+handle_call({new,#e3d_image{name=Name}=Im,true,Hide}, From, #ist{images=Images}=S0) ->
     Exist = fun({_, #e3d_image{name=N}}) when N =:= Name -> true;
 	       ({_, {hidden, #e3d_image{name=N}}}) when N =:= Name -> true;
 	       (_) -> false
 	    end,
     case lists:filter(Exist, gb_trees:to_list(Images)) of
 	[] ->
-	    handle({new,Im,false,Hide}, S0);
+	    handle_call({new,Im,false,Hide}, From, S0);
 	[{Id,_}] ->
-	    S = handle({delete,Id}, S0),
-	    handle({new,Im,false,Hide}, S)
+	    {reply, ok, S} = handle_call({delete,Id}, From, S0),
+	    handle_call({new,Im,false,Hide}, From, S)
     end;
-
-handle({rename,Id,Name0}, #ist{images=Images0}=S) ->
+handle_call({rename,Id,Name0}, _From, #ist{images=Images0}=S) ->
     Name = make_unique(Name0, gb_trees:delete(Id, Images0)),
     Im0 = gb_trees:get(Id, Images0),
     Im = (image_rec(Im0))#e3d_image{name=Name},
     Images = gb_trees:update(Id, hide(Im, is_hidden(Im0)), Images0),
-    {Id,S#ist{images=Images}};
-handle({txid,Id}, S) ->
-    {case get(Id) of
-	 undefined -> none;
-	 TxId -> TxId
-     end,S};
-handle({bumpid,Id}, S) ->
-    {case get({Id,bump}) of
-	 undefined ->
-	     create_bump(Id,undefined,S);
-	 TxId -> 
-	     TxId
-     end,S};
-handle({is_normalmap,Id},S) ->
-    {case {get({Id,bump}),get(Id)} of
-	 {undefined,undefined} -> none;
-	 {undefined, ImId} -> create_bump(Id,ImId,S);
-	 {TxId,_} -> TxId
-     end,S};
-
-handle(normalCM, S) ->
-    {case get(normalCM) of
-	 undefined -> 
-	     create_normal_cube_map();
-	 TxId -> 
-	     TxId
-     end,S};
-handle({default,all},S) ->
-    All = [diffuse, bump, gloss],
-    Ids = [{Type,element(1, handle({default,Type},S))} || Type <- All],
-    {Ids, S};
-handle({default,Type},S) ->
-    {case get({?DEFAULT, Type}) of
-	 undefined -> 
-	     create_default(Type);
-	 TxId -> 
-	     TxId
-     end,S};
-handle({info,Id}, #ist{images=Images}=S) ->
-    case gb_trees:lookup(Id, Images) of
-	{value,E3D} -> {image_rec(E3D),S};
-	none -> {none,S}
+    {reply, Id,S#ist{images=Images}};
+handle_call({txid,Id}, _From, S) ->
+    case get(Id) of
+        undefined -> {reply, none, S};
+        TxId -> {reply, TxId, S}
     end;
-handle(images, #ist{images=Images}=S) ->
-    {[NotHidden || NotHidden = {_, #e3d_image{}} <- gb_trees:to_list(Images)],S};
-handle(next_id, #ist{next=Id}=S) ->
-    {Id,S};
-handle({delete,Id}, S) ->
-    delete(Id, S);
-handle({delete_older,Id}, S) ->
-    delete_older(Id, S);
-handle({delete_from,Id}, S) ->
-    delete_from(Id, S);
-handle({update,Id,Image}, S) ->
+handle_call({bumpid,Id}, _From, S) ->
+    case get({Id,bump}) of
+        undefined -> {reply, create_bump(Id,undefined,S), S};
+        TxId -> {reply, TxId, S}
+    end;
+handle_call({is_normalmap,Id}, _From, S) ->
+    case {get({Id,bump}),get(Id)} of
+        {undefined,undefined} -> {reply, none, S};
+        {undefined, ImId} -> {reply, create_bump(Id,ImId,S), S};
+        {TxId,_} -> {reply, TxId, S}
+    end;
+handle_call({info,Id}, _From, #ist{images=Images}=S) ->
+    case gb_trees:lookup(Id, Images) of
+	{value,E3D} -> {reply, image_rec(E3D),S};
+	none -> {reply, none,S}
+    end;
+handle_call(images, _From, #ist{images=Images}=S) ->
+    {reply, [NotHidden || NotHidden = {_, #e3d_image{}} <- gb_trees:to_list(Images)],S};
+
+handle_call({filter_images, Bool}, _From, #ist{images=Images}=S) ->
+    Ft = case Bool of
+             true -> ?GL_LINEAR;
+             false -> ?GL_NEAREST
+         end,
+    Filter = fun(Id) ->
+                     TxId = get(Id),
+                     gl:bindTexture(?GL_TEXTURE_2D, TxId),
+                     gl:texParameteri(?GL_TEXTURE_2D, ?GL_TEXTURE_MAG_FILTER, Ft),
+                     gl:bindTexture(?GL_TEXTURE_2D, 0)
+             end,
+    [Filter(NotHidden) || {NotHidden, #e3d_image{}} <- gb_trees:to_list(Images)],
+    {reply, ok, S};
+
+handle_call(next_id, _From, #ist{next=Id}=S) ->
+    {reply,Id,S};
+handle_call({delete,Id}, _From, #ist{images=Images0}=S) ->
+    delete_bump(Id),
+    wings_gl:deleteTextures([erase(Id)]),
+    Images = gb_trees:delete(Id, Images0),
+    {reply, ok, S#ist{images=Images}};
+handle_call({delete_older,Id}, _From, #ist{images=Images0}=S) ->
+    Images1 = delete_older_1(gb_trees:to_list(Images0), Id),
+    Images = gb_trees:from_orddict(Images1),
+    {reply, ok, S#ist{images=Images}};
+handle_call({delete_from,Id}, _From, #ist{images=Images0}=S) ->
+    Images1 = delete_from_1(gb_trees:to_list(Images0), Id, []),
+    Images = gb_trees:from_orddict(Images1),
+    {reply, ok, S#ist{images=Images}};
+handle_call({update,Id,Image}, _From, S) ->
     do_update(Id, Image, S);
-handle({update_filename,Id,NewName}, #ist{images=Images0}=S) ->
+handle_call({find_image, Dir, File}, _From, #ist{images=Ims}=S) ->
+    AbsName = filename:join(Dir, File),
+    Find = fun(Fn) -> Fn == AbsName end,
+    Found = [Id || {Id, #e3d_image{filename=FN}} <-
+                       gb_trees:to_list(Ims), Find(FN)],
+    case Found of
+        [] -> {reply, false, S};
+        [Id|_] -> {reply, {true, Id}, S}
+    end;
+
+handle_call(Req, _From, S) ->
+    io:format("~w: Bad request: ~w~n", [?MODULE, Req]),
+    {reply, error, S}.
+
+handle_cast({update_filename,Id,NewName}, #ist{images=Images0}=S) ->
     Im0 = gb_trees:get(Id, Images0),
     Im = (image_rec(Im0))#e3d_image{filename=NewName},
     Images = gb_trees:update(Id, hide(Im, is_hidden(Im0)), Images0),
-    S#ist{images=Images}.
+    {noreply, S#ist{images=Images}}.
+
+
+%%%%%%%%%%%%%%%%% Internal Functions %%%%%%%%%%%%%%%
+
 
 create_bump(Id, BumpId, #ist{images=Images0}) ->
     delete_bump(Id),  %% update case..
@@ -429,31 +365,6 @@ create_bump(Id, BumpId, #ist{images=Images0}) ->
 	    none
     end.
 
-create_default(bump) ->
-    Img = #e3d_image{width=1,height=1,        % Default direction
-		     image= <<127,127,255>>}, % Normal points to +Z
-    make_texture({?DEFAULT,bump},Img);
-create_default(Type) when Type == diffuse; Type == gloss ->
-    Img = #e3d_image{width=1,height=1,        % Default diffuse/spec
-		     image= <<255,255,255>>}, % White and full specular.
-    make_texture({?DEFAULT,Type},Img);
-create_default(Type) -> 
-    io:format(?__(1,"~p Don't know about the type ~p; ignoring\n"),
-	      [?MODULE,Type]),
-    none.
-
-create_normal_cube_map() ->
-    case wings_gl:is_ext('GL_ARB_texture_cube_map') of
-	true ->	
-	    [CubeMap] = gl:genTextures(1),
-	    gl:bindTexture(?GL_TEXTURE_CUBE_MAP, CubeMap),
-	    make_normalize_vector_cubemap(32),
-	    put(normalCM,CubeMap),
-	    CubeMap;
-	false ->
-	    none
-    end.
-
 update_mipmaps(TxId, MipMaps) ->
     gl:enable(?GL_TEXTURE_2D),
     gl:bindTexture(?GL_TEXTURE_2D, TxId),
@@ -478,17 +389,6 @@ img_type(b8g8r8) -> r8g8b8;
 img_type(b8g8r8a8) -> r8g8b8a8;
 img_type(Type) -> Type.
 
-init_background_tx() ->
-    White = [255,255,255],
-    Grey = [204,204,204],
-    EightWhite = pattern_repeat(8, White),
-    EightGrey = pattern_repeat(8, Grey),
-    B0 = [pattern_repeat(8, [EightGrey|EightWhite])|
-	  pattern_repeat(8, [EightWhite|EightGrey])],
-    B = list_to_binary(B0),
-    Im = #e3d_image{width=16,height=16,image=B},
-    put(background, init_texture(Im)).
-
 make_texture(Id, Image) ->
     case init_texture(Image) of
 	{error,_}=Error ->
@@ -624,28 +524,12 @@ internal_format(?GL_BGR) -> ?GL_RGB;
 internal_format(?GL_BGRA) -> ?GL_RGBA;
 internal_format(Else) -> Else.
 
-delete(Id, #ist{images=Images0}=S) ->
-    delete_bump(Id),
-    wings_gl:deleteTextures([erase(Id)]),
-    Images = gb_trees:delete(Id, Images0),
-    S#ist{images=Images}.
-
-delete_older(Id, #ist{images=Images0}=S) ->
-    Images1 = delete_older_1(gb_trees:to_list(Images0), Id),
-    Images = gb_trees:from_orddict(Images1),
-    S#ist{images=Images}.
-
 delete_older_1([{Id,_}|T], Limit) when Id < Limit ->
     delete_bump(Id),
     wings_gl:deleteTextures([erase(Id)]),
     delete_older_1(T, Limit);
 delete_older_1(Images, _) -> Images.
 
-delete_from(Id, #ist{images=Images0}=S) ->
-    Images1 = delete_from_1(gb_trees:to_list(Images0), Id, []),
-    Images = gb_trees:from_orddict(Images1),
-    S#ist{images=Images}.
-
 delete_from_1([{Id,_}=Im|T], Limit, Acc) when Id < Limit ->
     delete_from_1(T, Limit, [Im|Acc]);
 delete_from_1([{Id,_}|T], Limit, Acc) ->
@@ -709,7 +593,6 @@ is_hidden(#e3d_image{}) -> false.
 
 window(Id) ->
     Name = {image,Id},
-    %% This need to be fixed for wx!!
     case wings_wm:is_window(Name) of
 	true ->
 	    wings_wm:raise(Name);
@@ -836,54 +719,6 @@ truncate(B0, Sz) ->
     <<B:Sz/binary,_/binary>> = list_to_binary(B0),
     B.
 
-%% Creating Normal-Cubemap
-
-%% Initialize a cube map texture object that generates RGB values
-%% that when expanded to a [-1,1] range in the texture-unit
-%% form a normalized vector matching the per-pixel vector used to
-%% access the cube map.
-
-make_normalize_vector_cubemap(Size) ->
-    gl:texParameteri(?GL_TEXTURE_CUBE_MAP, ?GL_TEXTURE_WRAP_S, ?GL_CLAMP_TO_EDGE),
-    gl:texParameteri(?GL_TEXTURE_CUBE_MAP, ?GL_TEXTURE_WRAP_T, ?GL_CLAMP_TO_EDGE),
-    gl:texParameteri(?GL_TEXTURE_CUBE_MAP, ?GL_TEXTURE_WRAP_R, ?GL_CLAMP_TO_EDGE),
-    gl:texParameteri(?GL_TEXTURE_CUBE_MAP, ?GL_TEXTURE_MAG_FILTER, ?GL_LINEAR),
-    gl:texParameteri(?GL_TEXTURE_CUBE_MAP, ?GL_TEXTURE_MIN_FILTER, ?GL_LINEAR),
-
-    Sides = [?GL_TEXTURE_CUBE_MAP_POSITIVE_X, ?GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
-	     ?GL_TEXTURE_CUBE_MAP_POSITIVE_Y, ?GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
-	     ?GL_TEXTURE_CUBE_MAP_POSITIVE_Z, ?GL_TEXTURE_CUBE_MAP_NEGATIVE_Z],
-    lists:foreach(fun(Side) -> make_cube_map(Side,Size-1,Size-1,Size, []) end, Sides).
-
-make_cube_map(Side,-1,-1,Size,Acc) ->
-    Image = list_to_binary(Acc),
-    gl:texImage2D(Side, 0, ?GL_RGB8, Size, Size, 0, 
-		  ?GL_RGB, ?GL_UNSIGNED_BYTE, Image);
-make_cube_map(Side,-1,Y,Size,Acc) ->
-    make_cube_map(Side,Size-1,Y-1,Size,Acc);
-make_cube_map(Side,X,Y,Size,Acc) ->
-    Vec = get_cube_vec(Side,Size,X,Y),
-    make_cube_map(Side,X-1,Y,Size,[Vec|Acc]).
-
-%% Given a cube map face index, cube map size, and integer 2D face position,
-%% return the cooresponding normalized vector.
-get_cube_vec(Side, Size, X, Y) ->
-    S = (X + 0.5) / Size,
-    T = (Y + 0.5) / Size,    
-    SC = S*2.0 - 1.0,    
-    TC = T*2.0 - 1.0,
-    Vec = 
-	case Side of
-	    ?GL_TEXTURE_CUBE_MAP_POSITIVE_X -> {1.0,  -TC, -SC};
-	    ?GL_TEXTURE_CUBE_MAP_NEGATIVE_X -> {-1.0, -TC,  SC};
-	    ?GL_TEXTURE_CUBE_MAP_POSITIVE_Y -> {SC,   1.0,  TC};
-	    ?GL_TEXTURE_CUBE_MAP_NEGATIVE_Y -> {SC,  -1.0, -TC};
-	    ?GL_TEXTURE_CUBE_MAP_POSITIVE_Z -> {SC,   -TC, 1.0};
-	    ?GL_TEXTURE_CUBE_MAP_NEGATIVE_Z -> {-SC,  -TC,-1.0}
-	end,
-    {RX,RY,RZ} = e3d_vec:norm(Vec),
-    [round(128+127*RX),round(128+127*RY),round(128+127*RZ)].
-
 %% Run a computation in a worker process with a generous heap size
 %% and the default generational garbage collector. Before R12B,
 %% heap fragments would allow the heap to be over-committed, but in
@@ -899,3 +734,67 @@ worker_process(WorkFun) ->
 	{'DOWN',Ref,process,Pid,Reason} ->
 	    exit(Reason)
     end.
+
+e3d_to_wxImage_1(I = #e3d_image{bytes_pp=4, width=W, height=H}) ->
+    #e3d_image{image=RGB} = e3d_image:convert(I, r8g8b8, 1, upper_left),
+    Wx = wxImage:new(W,H,RGB),
+    #e3d_image{image=Alpha} = e3d_image:convert(I, a8, 1, upper_left),
+    wxImage:setAlpha(Wx, Alpha),
+    Wx;
+e3d_to_wxImage_1(I = #e3d_image{bytes_pp=3, width=W, height=H}) ->
+    #e3d_image{image=RGB} = e3d_image:convert(I, r8g8b8, 1, upper_left),
+    wxImage:new(W,H,RGB);
+e3d_to_wxImage_1(I = #e3d_image{bytes_pp=1, width=W, height=H}) ->
+    #e3d_image{image=RGB} = e3d_image:convert(I, r8g8b8, 1, upper_left),
+    wxImage:new(W,H,RGB).
+
+wxImage_to_e3d(Wx) ->
+    wxImage_to_e3d_1(Wx).
+
+wxImage_to_e3d_1(Wx) ->
+    wxImage = wx:getObjectType(Wx), %% Assert
+    E3d0 = #e3d_image{image=wxImage:getData(Wx),
+		      width=wxImage:getWidth(Wx),
+		      height=wxImage:getHeight(Wx),
+		      order = upper_left
+		     },
+    case wxImage:hasAlpha(Wx) of
+	true ->
+            e3d_image:add_alpha(E3d0, wxImage:getAlpha(Wx));
+	false ->
+            case wxImage:hasMask(Wx) of
+                true ->
+                    wxImage:initAlpha(Wx),
+                    e3d_image:add_alpha(E3d0, wxImage:getAlpha(Wx));
+                false ->
+                    E3d0
+            end
+    end.
+
+try_relative_paths(Dirs,FileName,undefined,CurrDir,Ps) ->
+    try_relative_paths(CurrDir,Dirs,FileName,Ps);
+try_relative_paths(Dirs,FileName,OptDir,CurrDir,Ps) ->
+    case try_relative_paths(OptDir,Dirs,FileName,Ps) of
+	{error, enoent} ->
+	    try_relative_paths(Dirs,FileName,undefined,CurrDir,Ps);
+	Other ->
+	    Other
+    end.
+
+try_relative_paths(Start, Rel, File, Ps0) ->
+    Abs = case Rel of
+	      [] -> filename:join(Start,File);
+	      _  -> filename:join(filename:join(Start,filename:join(Rel)),File)
+	  end,
+    Ps = lists:keyreplace(filename,1,Ps0,{filename,Abs}),
+    case wings_plugin:call_ui({image,read,Ps}) of
+	Err = {error, enoent} ->
+	    case Rel of
+		[] -> Err;
+		[_|Rest] -> try_relative_paths(Start,Rest,File,Ps0)
+	    end;
+	Ok = #e3d_image{} ->
+	    Ok#e3d_image{filename=Abs};
+	Other ->
+	    Other
+    end.
