@@ -13,9 +13,9 @@
 
 -module(wings_light).
 -export([light_types/0,menu/3,command/2,is_any_light_selected/1,
-	 any_enabled_lights/0,info/1,
-	 create/2,update_dynamic/2,update_matrix/2,update/1,render/1,
-	 global_lights/0,camera_lights/0,camera_lights/1,
+	 any_enabled_lights/0,info/1,setup_light/1,
+	 create/2,update_dynamic/2,update_matrix/2,update/1,
+	 global_lights/1,
 	 export/1,export_bc/1,export_camera_lights/0,
 	 import/2,import/1,shape_materials/2,
 	 light_pos/1]).
@@ -24,7 +24,7 @@
 -include("wings.hrl").
 -include("e3d.hrl").
 
--import(lists, [reverse/1,foldl/3,member/2,keydelete/3,sort/1]).
+-import(lists, [reverse/1,foldl/3,foldr/3,member/2,keydelete/3,sort/1]).
 
 -define(DEF_X, 0.0).
 -define(DEF_Y, 3.0).
@@ -98,21 +98,24 @@ body_menu(Dir, #st{selmode=body}) ->
       ?STR(menu,21,"Delete seleced lights")}];
 body_menu(_, _) -> [].
 
-filter_menu(Menu0, St) ->
-    T = wings_sel:fold(
-	  fun(_, #we{light=#light{type=Type}}, none) -> Type;
-	     (_, _, _) -> mixed
-	  end, none, St),
-    Menu = foldl(fun({one_light,_}, A) when T == mixed -> A;
-		    ({one_light,Entry}, A) -> [Entry|A];
-		    ({{iff,[_|_]=Types},Entry}, A) ->
-			 case member(T, Types) of
-			     true -> [Entry|A];
-			     false -> A
-			 end;
-		    (Entry, A) -> [Entry|A]
-		 end, [], Menu0),
-    reverse(Menu).
+filter_menu(Menu, St) ->
+    MF = fun(_, #we{light=#light{type=Type}}) -> Type;
+            (_, #we{}) -> not_light
+         end,
+    RF = fun(Type, []) -> Type;
+            (Type, Type) -> Type;
+            (_, _) -> mixed
+         end,
+    T = wings_sel:dfold(MF, RF, [], St),
+    foldr(fun({one_light,_}, A) when T =:= mixed -> A;
+             ({one_light,Entry}, A) -> [Entry|A];
+             ({{iff,[_|_]=Types},Entry}, A) ->
+                  case member(T, Types) of
+                      true -> [Entry|A];
+                      false -> A
+                  end;
+             (Entry, A) -> [Entry|A]
+          end, [], Menu).
 
 command({move_light,Type}, St) ->
     wings_move:setup(Type, St);
@@ -131,48 +134,41 @@ command(edit, St) ->
 command({edit,Id}, St) ->
     edit(Id, St);
 command(delete, St) ->
-    {save_state,wings_shape:update_folders(delete(St))};
+    {save_state,delete(St)};
 command({duplicate,Dir}, St) ->
     duplicate(Dir, St).
-    
-is_any_light_selected(#st{sel=Sel,shapes=Shs}) ->
-    is_any_light_selected(Sel, Shs).
-is_any_light_selected([{Id,_}|Sel], Shs) ->
-    case gb_trees:get(Id, Shs) of
-	#we{}=We when ?IS_LIGHT(We) -> true;
-	#we{} -> is_any_light_selected(Sel, Shs)
-    end;
-is_any_light_selected([], _) -> false.
+
+-spec is_any_light_selected(#st{}) -> boolean().
+
+is_any_light_selected(St) ->
+    MF = fun(_, We) -> ?IS_LIGHT(We) end,
+    RF = fun erlang:'or'/2,
+    wings_sel:dfold(MF, RF, false, St).
 
 any_enabled_lights() ->
     wings_dl:fold(fun(#dlo{src_we=We}, Bool) ->
 			  Bool orelse ?IS_ANY_LIGHT(We)
 		  end, false).
 
+-spec info(#we{}) -> iolist().
+
 info(#we{name=Name,light=#light{type=Type}=L}=We) ->
-    Info0 = wings_util:format(?__(1,"Light ~s"), [Name]),
+    Info0 = io_lib:format(?__(1,"Light ~ts"), [Name]),
     case Type of
 	ambient -> Info0;
 	_ ->
-	    {X,Y,Z} = Pos = light_pos(We),
-	    Info = [Info0|io_lib:format(?__(2,": Pos ~s ~s ~s"),
-					[wings_util:nice_float(X),
-					 wings_util:nice_float(Y),
-					 wings_util:nice_float(Z)])],
+	    Pos = light_pos(We),
+	    Info = [Info0|io_lib:format(?__(2,": Pos ~s"),
+					[wings_util:nice_vector(Pos)])],
 	    [Info|info_1(Type, Pos, L)]
     end.
 
 info_1(point, _, _) -> [];
 info_1(Type, Pos, #light{aim=Aim,spot_angle=A}) ->
-    {Ax,Ay,Az} = Aim,
-    {Dx,Dy,Dz} = e3d_vec:norm_sub(Aim, Pos),
-    Info = io_lib:format(?__(1,". Aim ~s ~s ~s. Dir ~s ~s ~s"),
-			 [wings_util:nice_float(Ax),
-			  wings_util:nice_float(Ay),
-			  wings_util:nice_float(Az),
-			  wings_util:nice_float(Dx),
-			  wings_util:nice_float(Dy),
-			  wings_util:nice_float(Dz)]),
+    Dir = e3d_vec:norm_sub(Aim, Pos),
+    Info = io_lib:format(?__(1,". Aim ~s. Dir ~s"),
+			 [wings_util:nice_vector(Aim),
+			  wings_util:nice_vector(Dir)]),
     [Info|case Type of
 	      spot -> io_lib:format(?__(2,". Angle ~s~c"),
 				    [wings_util:nice_float(A),?DEGREE]);
@@ -183,30 +179,26 @@ info_1(Type, Pos, #light{aim=Aim,spot_angle=A}) ->
 %%% Light Commands.
 %%%
 
-color(St) ->
-    Drag = wings_sel:fold(
-	     fun(_, #we{id=Id,light=L}=We, none) when ?IS_LIGHT(We) ->
-		     {R,G,B,A} = get_light_color(L),
-		     {H,S,V} = wings_color:rgb_to_hsv(R, G, B),
-		     ColorFun = fun({finish,C}, D) -> color(C, D, A);
-				   (C, D) -> color(C, D, A)
-				end,
-		     Tvs = {general,[{Id,ColorFun}]},
-		     Units = [{angle,{0.0,359.9999}},
-			      {percent,{0.0,1.0}},
-			      {percent,{0.0,1.0}}
-			     ],
-		     Flags = [{initial,[H,V,S]}],
-		     {Tvs,Units,Flags};
-		(_, We, _) when ?IS_LIGHT(We) ->
-		     wings_u:error_msg(?__(1,"Select only one light."));
-		(_, _, A) -> A
-	     end, none, St),
-    case Drag of
-	none -> St;
-	{Tvs,Units,Flags} ->
-	    wings_drag:setup(Tvs, Units, Flags, St)
-    end.
+color(St0) ->
+    {St,Flags} =
+        wings_sel:mapfold(
+          fun(_, #we{light=L}=We, []) when ?IS_LIGHT(We) ->
+                  {R,G,B,A} = get_light_color(L),
+                  {H,S,V} = wings_color:rgb_to_hsv(R, G, B),
+                  ColorFun = fun({finish,C}, D) -> color(C, D, A);
+                                (C, D) -> color(C, D, A)
+                             end,
+                  Flags = [{initial,[H,V,S]}],
+                  {We#we{temp=ColorFun},Flags};
+             (_, We, _) when ?IS_LIGHT(We) ->
+                  wings_u:error_msg(?__(1,"Select only one light."));
+             (_, _, A) -> A
+           end, [], St0),
+    Units = [{angle,{0.0,359.9999}},
+             {percent,{0.0,1.0}},
+             {percent,{0.0,1.0}}],
+    DF = fun(#we{temp=General}) -> General end,
+    wings_drag:general(DF, Units, Flags, St).
 
 color([H,V,S], #dlo{src_we=#we{light=L0}=We0}=D, A) ->
     {R,G,B} = wings_color:hsv_to_rgb(H, S, V),
@@ -242,40 +234,37 @@ position_highlight_1(Center, #we{light=L0}=We) ->
 
 spot_angle(St) ->
     case selected_light(St) of
-	{Id,#light{type=spot,spot_angle=SpotAngle}} ->
+	#light{type=spot,spot_angle=SpotAngle} ->
 	    SpotFun0 = fun([Angle|_], L) -> L#light{spot_angle=Angle} end,
-	    SpotFun = adjust_fun(SpotFun0),
-	    Tvs = {general,[{Id,SpotFun}]},
+            DF = fun(_) -> adjust_fun(SpotFun0) end,
 	    Units = [{angle,{0.1,89.9}}],
 	    Flags = [{initial,[SpotAngle]}],
-	    wings_drag:setup(Tvs, Units, Flags, St);
-	{_,_} ->
+	    wings_drag:general(DF, Units, Flags, St);
+	_ ->
 	    wings_u:error_msg(?__(1,"Not a spotlight."))
     end.
 
 spot_falloff(St) ->
     case selected_light(St) of
-	{Id,#light{type=spot,spot_exp=SpotExp}} ->
+	#light{type=spot,spot_exp=SpotExp} ->
 	    SpotFun0 = fun([Exp|_], L) -> L#light{spot_exp=Exp} end,
-	    SpotFun = adjust_fun(SpotFun0),
-	    Tvs = {general,[{Id,SpotFun}]},
+	    DF = fun(_) -> adjust_fun(SpotFun0) end,
 	    Units = [{number,{0.0,128.0}}],
 	    Flags = [{initial,[SpotExp]}],
-	    wings_drag:setup(Tvs, Units, Flags, St);
-	{_,_} ->
+	    wings_drag:general(DF, Units, Flags, St);
+	_ ->
 	    wings_u:error_msg(?__(1,"Not a spotlight."))
     end.
 
 attenuation(Type, St) ->
     case selected_light(St) of
-	{Id,#light{type=Ltype}=L} when Ltype == point; Ltype == spot ->
+	#light{type=Ltype}=L when Ltype =:= point; Ltype =:= spot ->
 	    Initial = att_initial(Type, L),
-	    Fun = adjust_fun(att_fun(Type)),
-	    Tvs = {general,[{Id,Fun}]},
+	    DF = fun(_) -> adjust_fun(att_fun(Type)) end,
 	    Units = [{dx,att_range(Type)}],
 	    Flags = [{initial,[Initial]}],
-	    wings_drag:setup(Tvs, Units, Flags, St);
-	{_,_} ->
+	    wings_drag:general(DF, Units, Flags, St);
+	_ ->
 	    wings_u:error_msg(?__(1,"Not a point light or spotlight."))
     end.
 
@@ -289,13 +278,18 @@ att_range(linear) -> {0.0,1.0};
 att_range(quadratic) -> {0.0,0.5}.
 
 selected_light(St) ->
-    wings_sel:fold(fun(_, #we{id=Id,light=L}=We, none) when ?IS_LIGHT(We) ->
-			   {Id,L};
-		      (_, We, _) when ?IS_LIGHT(We) ->
-			   wings_u:error_msg(?__(1,
-						"Select only one light."));
-		      (_, _, A) -> A
-		   end, none, St).
+    MF = fun(_, #we{light=L}=We) when ?IS_LIGHT(We) ->
+                 [L];
+            (_, #we{}) ->
+                 []
+         end,
+    RF = fun erlang:'++'/2,
+    case wings_sel:dfold(MF, RF, [], St) of
+        [Selected] ->
+            Selected;
+        [_|_] ->
+            wings_u:error_msg(?__(1,"Select only one light."))
+    end.
 
 adjust_fun(AdjFun) ->
     fun({finish,Ds}, D) -> adjust_fun_1(AdjFun, Ds, D);
@@ -413,25 +407,28 @@ edit_specific(More, L) -> {L,More}.
 %%% The Delete command.
 %%%
 
-delete(#st{shapes=Shs0}=St) ->
-    Shs = wings_sel:fold(fun(_, #we{id=Id}, Shs) ->
-				 gb_trees:delete(Id, Shs)
-			 end, Shs0, St),
-    St#st{shapes=Shs,sel=[]}.
+delete(St) ->
+    wings_sel:map_update_sel(
+      fun(_, _) ->
+              {#we{},gb_sets:empty()}
+      end, St).
 
 %%%
 %%% The Duplicate command.
 %%%
 
 duplicate(Dir, St0) ->
-    {St,Sel} = wings_sel:fold(
-		 fun(Elems, We, {#st{onext=Id}=S0,Sel}) ->
-			 S = wings_shape:insert(We, copy, S0),
-			 {S,[{Id,Elems}|Sel]};
-		    (_, _, A) -> A
-		 end, {St0,[]}, St0),
-    wings_move:setup(Dir, wings_sel:set(Sel, St)).
-    
+    CF = fun(Items, We) ->
+                 Empty = gb_sets:empty(),
+                 New = [{We,Items,copy}],
+                 {We,Empty,New}
+         end,
+    St = wings_sel:clone(CF, St0),
+    case Dir of
+	none -> St;
+	_ -> wings_move:setup(Dir, St)
+    end.
+
 %%%
 %%% Creating lights.
 %%%
@@ -475,22 +472,19 @@ update_fun(infinite, SelColor, #we{light=#light{aim=Aim}}=We) ->
     LightCol = get_light_col(We),
     Vec = e3d_vec:norm_sub(Aim, LightPos),
     Data = [e3d_vec:mul(Vec, 0.2),e3d_vec:mul(Vec, 0.6)],
+    {Len, Tris,_} = wings_shapes:tri_sphere(#{subd=>3, scale=>0.08}),
     D = fun() ->
-		gl:lineWidth(1.0),
-		gl:color4fv(LightCol),
+		gl:lineWidth(1.5),
 		gl:pushMatrix(),
 		{X,Y,Z} = LightPos,
 		gl:translatef(X, Y, Z),
-		Obj = glu:newQuadric(),
-		glu:quadricDrawStyle(Obj, ?GLU_FILL),
-		glu:quadricNormals(Obj, ?GLU_SMOOTH),
-		glu:sphere(Obj, 0.08, 25, 25),
-		glu:deleteQuadric(Obj),
+		gl:color4fv(LightCol),
+                gl:drawArrays(?GL_TRIANGLES, 2, Len*3),
 		gl:color3fv(SelColor),
-		gl:drawArrays(?GL_LINES, 0, 2),
+                gl:drawArrays(?GL_LINES, 0, 2),
 		gl:popMatrix()
 	end,
-    wings_vbo:new(D, Data);
+    wings_vbo:new(D, Data++Tris);
 update_fun(point, SelColor, We) ->
     LightPos = light_pos(We),
     LightCol = get_light_col(We),
@@ -502,22 +496,19 @@ update_fun(point, SelColor, We) ->
 	     {0.0,0.71,0.71}],
     N = length(Data0) * 4,
     Data = lines(Data0),
+    {Len, Tris,_} = wings_shapes:tri_sphere(#{subd=>3, scale=>0.08}),
     D = fun() ->
 		gl:lineWidth(1.0),
 		gl:color4fv(LightCol),
 		gl:pushMatrix(),
 		{X,Y,Z} = LightPos,
 		gl:translatef(X, Y, Z),
-		Obj = glu:newQuadric(),
-		glu:quadricDrawStyle(Obj, ?GLU_FILL),
-		glu:quadricNormals(Obj, ?GLU_FLAT),
-		glu:sphere(Obj, 0.08, 25, 25),
-		glu:deleteQuadric(Obj),
+                gl:drawArrays(?GL_TRIANGLES, N, Len*3),
 		gl:color3fv(SelColor),
 		gl:drawArrays(?GL_LINES, 0, N),
 		gl:popMatrix()
 	end,
-    wings_vbo:new(D, Data);
+    wings_vbo:new(D, Data++Tris);
 update_fun(spot, SelColor, #we{light=#light{aim=Aim,spot_angle=Angle}}=We) ->
     Top = light_pos(We),
     LightCol = get_light_col(We),
@@ -531,25 +522,25 @@ update_fun(spot, SelColor, #we{light=#light{aim=Aim,spot_angle=Angle}}=We) ->
     H = math:cos(Rad),
     Translate = e3d_vec:mul(SpotDir, H),
     Rot = e3d_mat:rotate_s_to_t({0.0,0.0,1.0}, e3d_vec:neg(SpotDir)),
-    fun() ->
-	    gl:lineWidth(1.0),
-	    gl:color4fv(LightCol),
-	    gl:pushMatrix(),
-	    Obj = glu:newQuadric(),
-	    {Tx,Ty,Tz} = Top,
-	    gl:translatef(Tx, Ty, Tz),
-	    glu:quadricDrawStyle(Obj, ?GLU_FILL),
-	    glu:quadricNormals(Obj, ?GLU_SMOOTH),
-	    glu:sphere(Obj, 0.08, 25, 25),
-	    gl:color3fv(SelColor),
-	    {Dx,Dy,Dz} = Translate,
-	    gl:translatef(Dx, Dy, Dz),
-	    gl:multMatrixd(Rot),
-	    glu:quadricDrawStyle(Obj, ?GLU_LINE),
-	    glu:cylinder(Obj, R, 0.08, H, 12, 1),
-	    glu:deleteQuadric(Obj),
-	    gl:popMatrix()
-    end;
+    {Len, Tris,_} = wings_shapes:tri_sphere(#{subd=>3, scale=>0.08}),
+    D = fun() ->
+                gl:lineWidth(1.0),
+                gl:color4fv(LightCol),
+                gl:pushMatrix(),
+                {Tx,Ty,Tz} = Top,
+                gl:translatef(Tx, Ty, Tz),
+                gl:drawArrays(?GL_TRIANGLES, 0, Len*3),
+                gl:color3fv(SelColor),
+                {Dx,Dy,Dz} = Translate,
+                gl:translatef(Dx, Dy, Dz),
+                gl:multMatrixd(Rot),
+                Obj = glu:newQuadric(),
+                glu:quadricDrawStyle(Obj, ?GLU_LINE),
+                glu:cylinder(Obj, R, 0.08, H, 12, 1),
+                glu:deleteQuadric(Obj),
+                gl:popMatrix()
+        end,
+    wings_vbo:new(D, Tris);
 update_fun(ambient, _, _) ->
     fun() -> ok end.
 
@@ -563,9 +554,6 @@ lines([]) -> [].
 get_light_col(#we{light=#light{diffuse=Diff}}) ->
     Diff.
 
-render(#dlo{work=Light}) ->
-    wings_dl:call(Light).
-
 %%%
 %%% Exporting lights.
 %%%
@@ -639,8 +627,6 @@ get_light_1(#we{light=L,pst=Pst}=We, BC) ->
 	     end,
     [{opengl,OpenGL}|Prop].
 
-export_perm([], Ps) ->
-    [{visible,false},{locked,false}|Ps];
 export_perm({_,_}, Ps) ->
     [{visible,false},{locked,false}|Ps];
 export_perm(P, Ps) when is_integer(P) ->
@@ -682,7 +668,7 @@ import(Lights, St) ->
     foldl(fun import_fun/2, St, Lights).
 
 import_fun({Name,Ps}, St) ->
-    wings_shape:new(Name, import(Ps), St).
+    wings_obj:new(Name, import(Ps), St).
 
 import(Ps) ->
     Visible = proplists:get_value(visible, Ps, []),
@@ -745,13 +731,13 @@ import_we(#light{}=Light, OpenGL, {X,Y,Z}) ->
 import_perm([]=_Visible,[]=_Locked) ->  % it will match when a new light is added to project  
     import_perm(true,false);
 import_perm(false,false) ->
-    [];
+    ?PERM_HIDDEN_BIT;
 import_perm(true,false) ->
     0;
 import_perm(true,true) ->
-    1;
+    ?PERM_LOCKED_BIT;
 import_perm(false,true) ->
-    3.
+    ?PERM_HIDDEN_BIT bor ?PERM_LOCKED_BIT.
 
 import_fix_mesh(#e3d_mesh{fs=Fs0}=Mesh0) ->
     Fs = [import_fix_face(F) || F <- Fs0],
@@ -775,49 +761,11 @@ import_fix_face(FaceRec) when is_tuple(FaceRec) ->
 %%% Setting up lights.
 %%%
 
-camera_lights() ->
-    camera_lights(wings_pref:get_value(number_of_lights)).
-
-camera_lights(Type) ->
-    gl:enable(?GL_LIGHT0),
-    gl:disable(?GL_LIGHT2),
-    gl:disable(?GL_LIGHT3),
-    gl:disable(?GL_LIGHT4),
-    gl:disable(?GL_LIGHT5),
-    gl:disable(?GL_LIGHT6),
-    gl:disable(?GL_LIGHT7),
-    Amb = camera_ambient(),
-    gl:lightModelfv(?GL_LIGHT_MODEL_AMBIENT, Amb#light.ambient),
-    case Type of
-	1 ->
-	    L = camera_infinite_1_0(),
-	    gl:disable(?GL_LIGHT1),
-	    gl:lightfv(?GL_LIGHT0, ?GL_DIFFUSE,  L#light.diffuse),
-	    gl:lightfv(?GL_LIGHT0, ?GL_SPECULAR, L#light.specular),
-	    gl:lightfv(?GL_LIGHT0, ?GL_POSITION, infinite(L#light.aim));     
-	2 ->
-	    L20 = camera_infinite_2_0(),
-	    L21 = camera_infinite_2_1(),
-	    gl:enable(?GL_LIGHT1),
-	    gl:lightfv(?GL_LIGHT0, ?GL_DIFFUSE,  L20#light.diffuse), 
-	    gl:lightfv(?GL_LIGHT0, ?GL_SPECULAR, L20#light.specular),
-	    gl:lightfv(?GL_LIGHT0, ?GL_POSITION, infinite(L20#light.aim)),
-	    gl:lightfv(?GL_LIGHT1, ?GL_DIFFUSE,  L21#light.diffuse), 
-	    gl:lightfv(?GL_LIGHT1, ?GL_SPECULAR, L21#light.specular),
-	    gl:lightfv(?GL_LIGHT1, ?GL_POSITION, infinite(L21#light.aim));     
-	mat_preview ->
-	    D = 0.8,
-	    S = 0.7,
-	    gl:disable(?GL_LIGHT1),
-	    gl:lightfv(?GL_LIGHT0, ?GL_DIFFUSE, {D,D,D,1}),
- 	    gl:lightfv(?GL_LIGHT0, ?GL_SPECULAR, {S,S,S,1}),
-	    gl:lightfv(?GL_LIGHT0, ?GL_POSITION, {1,1,2,0})
-    end.
-
-global_lights() ->
+global_lights(Lights0) ->
     gl:lightModelfv(?GL_LIGHT_MODEL_AMBIENT, {0.0,0.0,0.0,1.0}),
-    Lnum = wings_dl:fold(fun scene_lights_fun/2, ?GL_LIGHT0),
-    disable_from(Lnum).
+    Lights = lists:map(fun scene_lights_fun/1, Lights0),
+    IsAL = fun(#{light:=#light{type=Type}}) -> Type =:= ambient end,
+    lists:partition(IsAL, Lights).
 
 camera_ambient() ->
     #light{type = ambient, 
@@ -845,21 +793,11 @@ camera_infinite_2_1() ->
 	   aim      = {-0.71,-0.71,0.0}
 	  }.
 
-infinite({X,Y,Z}) -> {X,Y,Z,0.0}.
-
-disable_from(Lnum) when Lnum > ?GL_LIGHT7 -> ok;
-disable_from(Lnum) ->
-    gl:disable(Lnum),
-    disable_from(Lnum+1).
-
-scene_lights_fun(_, Lnum) when Lnum > ?GL_LIGHT7 -> Lnum;
-scene_lights_fun(#dlo{src_we=We}, Lnum) 
-  when not ?IS_ANY_LIGHT(We)-> Lnum;
-scene_lights_fun(#dlo{transparent=#we{light=L}=We}, Lnum) ->
+scene_lights_fun(#dlo{transparent=#we{light=L}=We}) ->
     %% This happens when dragging a light in Body selection mode.
     %% (Not area light.)
-    setup_light(Lnum, L, We, none);
-scene_lights_fun(#dlo{drag=Drag,src_we=We0}=D, Lnum) ->
+    prepare_light(L, We, none);
+scene_lights_fun(#dlo{drag=Drag,src_we=We0}=D) ->
     %% Area lights handled here in all selection modes +
     %% other lights in vertex/edge/face modes.
     We = case We0 of
@@ -870,9 +808,9 @@ scene_lights_fun(#dlo{drag=Drag,src_we=We0}=D, Lnum) ->
 		 %% updating would use the resulting #we{}, but it does not
 		 %% exist until the drag is done.
 		 wings_draw:original_we(D);
-	     _ -> 
+	     _ ->
 		 %% Non-area lights drag the whole shape so they can use
-		 %% the dynamic part of the splitted shape 
+		 %% the dynamic part of the splitted shape
 		 %% (which is the whole shape).
 		 We0
 	 end,
@@ -880,70 +818,78 @@ scene_lights_fun(#dlo{drag=Drag,src_we=We0}=D, Lnum) ->
 	    {matrix,_Tr,_M0,M1} -> M1;
  	    _ -> none
  	end,
-    setup_light(Lnum, We#we.light, We, M).
+    prepare_light(We#we.light, We, M).
 
-setup_light(Lnum, #light{type=ambient,ambient=Amb}, _We, _M) ->
+prepare_light(#light{type=ambient,ambient=Amb}=L, _We, _M) ->
     gl:lightModelfv(?GL_LIGHT_MODEL_AMBIENT, Amb),
-    Lnum;
-setup_light(Lnum, #light{type=infinite,aim=Aim}=L, We, _M) ->
+    #{light=>L};
+prepare_light(#light{type=infinite,aim=Aim}=L, We, _M) ->
     {X,Y,Z} = e3d_vec:norm_sub(light_pos(We), Aim),
-    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,0.0}),
-    setup_color(Lnum, L),
-    gl:enable(Lnum),
-    Lnum+1;
-setup_light(Lnum, #light{type=point}=L, We, _M) ->
+    #{light=>L, pos=>{X,Y,Z,0.0}};
+prepare_light(#light{type=point}=L, We, _M) ->
     {X,Y,Z} = light_pos(We),
-    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,1.0}),
-    gl:lightf(Lnum, ?GL_SPOT_CUTOFF, 180.0),
-    setup_color(Lnum, L),
-    setup_attenuation(Lnum, L),
-    gl:enable(Lnum),
-    Lnum+1;
-setup_light(Lnum, #light{type=spot,aim=Aim,spot_angle=Angle,spot_exp=Exp}=L, 
-	    We, _M) ->
+    #{light=>L, pos=>{X,Y,Z,1.0}};
+prepare_light(#light{type=spot,aim=Aim}=L, We, _M) ->
     Pos = {X,Y,Z} = light_pos(We),
     Dir = e3d_vec:norm_sub(Aim, Pos),
-    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,1.0}),
-    gl:lightf(Lnum, ?GL_SPOT_CUTOFF, Angle),
-    gl:lightf(Lnum, ?GL_SPOT_EXPONENT, Exp),
-    gl:lightfv(Lnum, ?GL_SPOT_DIRECTION, Dir),
-    setup_color(Lnum, L),
-    setup_attenuation(Lnum, L),
-    gl:enable(Lnum),
-    Lnum+1;
-setup_light(Lnum, #light{type=area}=L, We, M) ->
+    #{light=>L, pos=>{X,Y,Z,1.0}, dir=>Dir};
+prepare_light(#light{type=area}=L, We, M) ->
     Apde = arealight_posdirexp(We),
-    setup_arealight(Lnum, L, Apde, M).
+    prepare_arealight(L, Apde, M).
 
-setup_arealight(Lnum, #light{type=area}=L, {Pos0,{0.0,0.0,0.0},_}, M) ->
-    {X,Y,Z} = 
-	case M of
-	    none -> Pos0;
-	    _ -> mul_point(M, Pos0)
-	end,
-    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,1}),
-    gl:lightf(Lnum, ?GL_SPOT_CUTOFF, 180.0),
-    setup_color(Lnum, L),
-    setup_attenuation(Lnum, L),
-    gl:enable(Lnum),
-    Lnum+1;
-setup_arealight(Lnum, #light{type=area}=L, {Pos0,Dir0,Exp}, M) ->
-    {{X,Y,Z},Dir} = 
+prepare_arealight(#light{type=area}=L, {Pos0,{0.0,0.0,0.0},_}, M) ->
+    {X,Y,Z} = case M of
+                  none -> Pos0;
+                  _ -> mul_point(M, Pos0)
+              end,
+    #{light=>L, pos=>{X,Y,Z,1.0}};
+prepare_arealight(#light{type=area}=L, {Pos0,Dir0,Exp}, M) ->
+    {{X,Y,Z},Dir} =
 	case M of
 	    none -> {Pos0,Dir0};
-	    _ -> 
+	    _ ->
 		Pos = mul_point(M, Pos0),
 		Aim = mul_point(M, e3d_vec:add(Dir0, Pos0)),
 		{Pos,e3d_vec:sub(Aim, Pos)}
 	end,
-    gl:lightfv(Lnum, ?GL_POSITION, {X,Y,Z,1}),
-    gl:lightf(Lnum, ?GL_SPOT_CUTOFF, 90.0),
-    gl:lightf(Lnum, ?GL_SPOT_EXPONENT, Exp),
-    gl:lightfv(Lnum, ?GL_SPOT_DIRECTION, Dir),
-    setup_color(Lnum, L),
-    setup_attenuation(Lnum, L),
-    gl:enable(Lnum),
-    Lnum+1.
+    #{light=>L, pos=>{X,Y,Z,1.0}, dir=>Dir, exp=>Exp}.
+
+setup_light(#{light:=#light{type=ambient,ambient=Amb}}) ->
+    gl:lightModelfv(?GL_LIGHT_MODEL_AMBIENT, Amb),
+    wings_shaders:use_prog(ambient_light);
+setup_light(#{light:=#light{type=infinite}=L, pos:=Pos}) ->
+    gl:lightfv(?GL_LIGHT0, ?GL_POSITION, Pos),
+    setup_color(?GL_LIGHT0, L),
+    wings_shaders:use_prog(infinite_light);
+setup_light(#{light:=#light{type=point}=L,pos:=Pos}) ->
+    gl:lightfv(?GL_LIGHT0, ?GL_POSITION, Pos),
+    gl:lightf(?GL_LIGHT0, ?GL_SPOT_CUTOFF, 180.0),
+    setup_color(?GL_LIGHT0, L),
+    setup_attenuation(?GL_LIGHT0, L),
+    wings_shaders:use_prog(point_light);
+setup_light(#{light:=#light{type=spot,spot_angle=Angle,spot_exp=Exp}=L,
+              pos:=Pos, dir:=Dir}) ->
+    gl:lightfv(?GL_LIGHT0, ?GL_POSITION, Pos),
+    gl:lightf(?GL_LIGHT0, ?GL_SPOT_CUTOFF, Angle),
+    gl:lightf(?GL_LIGHT0, ?GL_SPOT_EXPONENT, Exp),
+    gl:lightfv(?GL_LIGHT0, ?GL_SPOT_DIRECTION, Dir),
+    setup_color(?GL_LIGHT0, L),
+    setup_attenuation(?GL_LIGHT0, L),
+    wings_shaders:use_prog(spot_light);
+setup_light(#{light:=#light{type=area}=L, pos:=Pos, dir:=Dir, exp:=Exp}) ->
+    gl:lightfv(?GL_LIGHT0, ?GL_POSITION, Pos),
+    gl:lightf(?GL_LIGHT0, ?GL_SPOT_CUTOFF, 90.0),
+    gl:lightf(?GL_LIGHT0, ?GL_SPOT_EXPONENT, Exp),
+    gl:lightfv(?GL_LIGHT0, ?GL_SPOT_DIRECTION, Dir),
+    setup_color(?GL_LIGHT0, L),
+    setup_attenuation(?GL_LIGHT0, L),
+    wings_shaders:use_prog(spot_light);
+setup_light(#{light:=#light{type=area}=L, pos:=Pos}) ->
+    gl:lightfv(?GL_LIGHT0, ?GL_POSITION, Pos),
+    gl:lightf(?GL_LIGHT0, ?GL_SPOT_CUTOFF, 180.0),
+    setup_color(?GL_LIGHT0, L),
+    setup_attenuation(?GL_LIGHT0, L),
+    wings_shaders:use_prog(point_light).
 
 setup_color(Lnum, #light{diffuse=Diff,ambient=Amb,specular=Spec}) ->
     gl:lightfv(Lnum, ?GL_DIFFUSE, Diff),
