@@ -40,6 +40,7 @@
 -define(VIEW_ORTHO, 91).
 -define(VIEW_AXES, 90).
 -define(VIEW_GROUND, 89).
+-define(CHECK_MARK, [10003,32]).  % 10003 = unicode for check mark character
 
 %% menu entries, the name, type and opts are used differently
 %% depending on if they are used in top_level menus or pop menus
@@ -198,6 +199,7 @@ setup_dialog(Parent, Entries0, Magnet, {X0,Y0}=ScreenPos) ->
     X  = X0-20,
     Y1 = Y0-10,
     Dialog = wxPopupTransientWindow:new(Parent, [{style, ?wxBORDER_SIMPLE}]),
+    wxWindow:setExtraStyle(Dialog, ?wxWS_EX_PROCESS_IDLE),
     Panel = wxPanel:new(Dialog),
     wxPanel:setFont(Panel, ?GET(system_font_wx)),
     %% wxPanel:setBackgroundStyle(Panel, ?wxBG_STYLE_TRANSPARENT),
@@ -411,11 +413,15 @@ popup_result(#menu{type=menu,name=Name,opts=Opts}, {Click,MagnetClick}, Names, O
     Cmd =:= ignore orelse wings_wm:send_after_redraw(Owner, {action,Cmd}),
     pop.
 
-calc_min_sizes([#menu{type=Type, desc=Desc, hk=HK}|Es], Win, C1, C2)
+calc_min_sizes([#menu{type=Type, desc=Desc, opts=Ps, hk=HK}|Es], Win, C1, C2)
   when Type=:=menu;Type=:=submenu ->
+    case proplists:get_value(crossmark, Ps) of
+	true -> {WChk, _, _, _} = wxWindow:getTextExtent(Win, ?CHECK_MARK);
+	_ -> WChk = 0
+    end,
     {WStr, _, _, _} = wxWindow:getTextExtent(Win, Desc),
     {WHK, _, _, _} = wxWindow:getTextExtent(Win, get_hotkey(1,HK)),
-    calc_min_sizes(Es, Win, max(WStr+5, C1), max(WHK+5, C2));
+    calc_min_sizes(Es, Win, max(WChk+WStr+5, C1), max(WHK+5, C2));
 calc_min_sizes([#menu{}|Es], Win, C1, C2) ->
     calc_min_sizes(Es, Win, C1, C2);
 calc_min_sizes([], _, C1, C2) ->
@@ -464,8 +470,10 @@ setup_popup([#menu{type=menu, wxid=Id, desc=Desc, help=Help, opts=Props, hk=HK}=
     Panel = wxPanel:new(Parent, [{winid, Id}]),
     setup_colors([Panel], Cs),
     Line  = wxBoxSizer:new(?wxHORIZONTAL),
+    Checked = proplists:get_value(crossmark, Props) =:= true,
     wxSizer:addSpacer(Line, 3),
-    wxSizer:add(Line, T1 = wxStaticText:new(Panel, Id, Desc),[{proportion, 0},{flag, ?wxALIGN_CENTER}]),
+    ChkM = if Checked -> ?CHECK_MARK; true -> "" end,
+    wxSizer:add(Line, T1 = wxStaticText:new(Panel, Id, ChkM++Desc),[{proportion, 0},{flag, ?wxALIGN_CENTER}]),
     wxSizer:setItemMinSize(Line, T1, Sz1, -1),
     wxSizer:addSpacer(Line, 10),
     wxSizer:addStretchSpacer(Line),
@@ -663,21 +671,29 @@ update_menu(Menu, Item, delete, _) ->
 	    end
     end;
 update_menu(Menu, Item, {append, Pos0, Cmd0}, Help) ->
+    update_menu(Menu, Item, {append, Pos0, Cmd0, []}, Help);
+update_menu(Menu, Item, {append, Pos0, Cmd0, Props}, Help) ->
     case menu_item_id(Menu, Item) of
 	false ->
 	    AddItem =
 		fun(SubMenu, Name) ->
 		    Pos =
 			if Pos0 >= 0 -> min(wxMenu:getMenuItemCount(SubMenu), Pos0);
-                           true -> wxMenu:getMenuItemCount(SubMenu)
+			    true -> wxMenu:getMenuItemCount(SubMenu)
 			end,
-		    MO = wxMenu:insert(SubMenu, Pos, -1, [{text, Cmd0}]),
+		    {Type,Check} = case proplists:get_value(crossmark, Props) of
+				       undefined -> {?wxITEM_NORMAL, false};
+				       false -> {?wxITEM_CHECK, false};
+				       _ -> {?wxITEM_CHECK, true} %% grey or true
+				   end,
+		    MO = wxMenu:insert(SubMenu, Pos, -1, [{text, Cmd0},{kind, Type}]),
 		    Id = wxMenuItem:getId(MO),
 		    ME=#menu{name=Name, object=MO,
-			     wxid=Id, type=?wxITEM_NORMAL},
+			     wxid=Id, type=Type},
 		    true = ets:insert(wings_menus, ME),
 		    Cmd = setup_hotkey(MO, Cmd0),
 		    wxMenuItem:setText(MO, Cmd),
+		    (Type==?wxITEM_CHECK) andalso wxMenuItem:check(MO,[{check, Check}]),
 		    is_list(Help) andalso wxMenuItem:setHelp(MO, Help)
 		end,
 
