@@ -24,7 +24,7 @@
 -define(NEED_OPENGL, 1).
 -include("wings.hrl").
 -include("e3d_image.hrl").
--import(lists, [keydelete/3]).
+-import(lists, [keydelete/3,reverse/1]).
 
 -dialyzer({nowarn_function, fake_enter_window/2}).
 
@@ -49,7 +49,7 @@ window(St) ->
 window(Pos, Size, Ps0, St) ->
     Shapes = get_state(St),
     {Frame,Ps} = wings_frame:make_win(title(), [{size, Size}, {pos, Pos}|Ps0]),
-    Window = wx_object:start_link(?MODULE, [Frame, Ps, Shapes], []),
+    Window = wings_sup:window(undefined, ?MODULE, [Frame, Ps, Shapes]),
     Fs = [{display_data, geom_display_lists}|Ps],
     wings_wm:toplevel(?MODULE, Window, Fs, {push, change_state(Window, St)}),
     keep.
@@ -233,13 +233,14 @@ rename(Id, File) ->
     Name = filename:basename(File),
     wings_image:rename(Id, Name).
 
-rename_obj(Id, NewName, #st{shapes=Shs}=St) ->
-    case gb_trees:get(Id, Shs) of
-	#we{name=NewName} -> ignore;
-	We0 ->
-	    We = We0#we{name=NewName},
-	    Shapes = gb_trees:update(Id, We, Shs),
-	    wings_wm:send(geom, {new_state, St#st{shapes=Shapes}})
+rename_obj(Id, NewName, St0) ->
+    case wings_obj:get(Id, St0) of
+        #{name:=NewName} ->
+            ignore;
+        Obj0 ->
+            Obj = Obj0#{name:=NewName},
+            St = wings_obj:put(Obj, St0),
+	    wings_wm:send(geom, {new_state,St})
     end.
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -277,16 +278,17 @@ forward_event(Ev, Window, _) ->
     wx_object:cast(Window, Ev),
     keep.
 
-get_state(#st{mat=Mat,shapes=Shs0}) ->
-    Lights = [light_info(We) || We <- gb_trees:values(Shs0),
-				?IS_ANY_LIGHT(We)],
+get_state(#st{mat=Mat}=St) ->
+    F = fun(#{light:=_,id:=Id,name:=Name}, A) ->
+                [#{type=>light,id=>Id,name=>Name}|A];
+           (#{}, A) ->
+                A
+        end,
+    Lights = reverse(wings_obj:fold(F, [], St)),
     Materials = [mat_info(M) || M <- gb_trees:to_list(Mat)],
     Images = [image_info(Im) || Im <- wings_image:images()],
     Lights ++ Materials ++ Images.
 
-light_info(#we{id=Id,name=Name}) ->
-    #{type=>light,id=>Id,name=>Name}.
-
 mat_info({Name,Mp}) ->
     OpenGL = proplists:get_value(opengl, Mp),
     Maps = proplists:get_value(maps, Mp, []),
@@ -311,6 +313,8 @@ init([Frame,  _Ps, Os]) ->
     Szr = wxBoxSizer:new(?wxVERTICAL),
     wxPanel:setBackgroundColour(Panel, BG),
     TC = make_tree(Panel, Cs, IL),
+    wxPanel:dragAcceptFiles(Panel, true),
+    wxPanel:connect(Panel, drop_files),
     wxSizer:add(Szr, TC, [{proportion,1}, {flag, ?wxEXPAND}]),
     wxPanel:setSizer(Panel, Szr),
     {Shown,IMap} = update_object(Os, TC, IL, IMap0),
@@ -337,8 +341,28 @@ handle_sync_event(#wx{event=#wxTree{item=Indx}}, Drag,
 	    wx_object:get_pid(TC) ! {drag, Obj};
 	_ -> ignore %% for now
     end,
+    ok;
+
+handle_sync_event(#wx{event=#wxKey{type=char, keyCode=KC}}, EvObj, #state{tc=TC, shown=Shown}) ->
+    Indx = wxTreeCtrl:getSelection(TC),
+    case {key_to_op(KC), obj_to_type_and_param(lists:keyfind(Indx, 1, Shown))} of
+	{Act, {Elm, Param}} when Act =/= ignore ->
+	    Cmd = list_to_atom(Act++"_"++Elm),
+	    wings_wm:psend(?MODULE, {action, {?MODULE, {Cmd, Param}}});
+	_ ->  % false is returned for the root items: 'Lights', 'Materials', 'Images'
+	    wxEvent:skip(EvObj)
+    end,
     ok.
 
+key_to_op(?WXK_DELETE) -> "delete";
+key_to_op(?WXK_F2) -> "rename";
+key_to_op(_) -> ignore.
+
+obj_to_type_and_param({_, #{type:=mat, name:=Name}}) -> {"material", Name};
+obj_to_type_and_param({_, #{type:=image, id:=Id}}) -> {"image", Id};
+obj_to_type_and_param({_, #{type:=light, id:=Id}}) -> {"object", Id};
+obj_to_type_and_param(_) -> ignore.
+
 handle_event(#wx{event=#wxMouse{type=right_up, x=X, y=Y}}, #state{tc=TC} = State) ->
     {Indx, _} = wxTreeCtrl:hitTest(TC, {X,Y}),
     make_menus(Indx, wxWindow:clientToScreen(TC, {X,Y}), State),
@@ -429,6 +453,10 @@ handle_event(#wx{event=#wxFocus{}}=Ev, #state{top=Obj} = State) ->
     %% fake a enter_window event
     wings_frame ! fake_enter_window(Ev, Obj),
     {noreply, State};
+handle_event(#wx{event=#wxDropFiles{}}=Ev, State) ->
+    %% Let wings deal with this
+    wings_wm:psend(geom,Ev),
+    {noreply, State};
 handle_event(#wx{event=_Ev}, State) ->
     %% io:format("~p:~p Got unexpected event ~p~n", [?MODULE,?LINE, _Ev])
     {noreply, State}.
@@ -438,8 +466,14 @@ handle_call(_Req, _From, State) ->
     {reply, ok, State}.
 
 handle_cast({new_state, Os}, #state{tc=TC, il=IL, imap=IMap0} = State) ->
-    {Shown,IMap} = update_object(Os, TC, IL, IMap0),
-    {noreply, State#state{os=Os, shown=Shown, imap=IMap}};
+    try
+        {Shown,IMap} = update_object(Os, TC, IL, IMap0),
+        {noreply, State#state{os=Os, shown=Shown, imap=IMap}}
+    catch _:Reason ->
+            io:format("~p:~p: crashed ~P~n",[?MODULE,?LINE,Reason,30]),
+            io:format(" at: ~P~n",[Reason,30]),
+            {noreply, State}
+    end;
 handle_cast(quit, State) ->
     {noreply, State};
 handle_cast(_Req, State) ->
@@ -495,6 +529,7 @@ make_tree(Parent, #{bg:=BG, text:=FG}, IL) ->
     wxWindow:connect(TC, command_tree_item_activated, []),
     wxWindow:connect(TC, enter_window, [{userData, {win, Parent}}]),
     wxWindow:connect(TC, motion, [{skip, true}]),
+    wxWindow:connect(TC, char, [callback]),
     case os:type() of
 	{win32, _} ->
 	    wxWindow:connect(TC, command_tree_item_menu, [{skip, false}]),
@@ -537,12 +572,12 @@ update_object(Os, TC, IL, Imap0) ->
 		     end,
 		 Acc = [{Item, O}|Acc0],
 		 if Type =:= mat ->
-		     case maps:get(maps, O, []) of
-			 [] -> {Acc, Imap};
-			 Maps ->
-			     add_maps(Maps, TC, Item, Name, IL, Imap, Os, Acc)
-		     end;
-		 true -> {Acc, Imap}
+                         case maps:get(maps, O, []) of
+                             [] -> {Acc, Imap};
+                             Maps ->
+                                 add_maps(Maps, TC, Item, Name, IL, Imap, Os, Acc)
+                         end;
+                    true -> {Acc, Imap}
 		 end
 		 %% {Node,_} = lists:keyfind(Curr, 2, All),
 		 %% wxTreeCtrl:selectItem(TC, Node),
@@ -605,10 +640,13 @@ image_maps_index(Type) ->
     case Type of
     	diffuse -> 4;
 	gloss -> 5;
+        metallic -> 5;
 	bump -> 6;
 	normal -> 7;
-	material -> 8;
-	_ -> undefined
+        occlusion -> 8;
+        emission -> 9;
+	material -> 10;
+        _unknown -> 4
     end.
 
 load_icons() ->
@@ -629,6 +667,7 @@ load_icons() ->
 		     small_image,small_image2,perspective, %small_object,
 		     small_light,
 		     small_diffuse,small_gloss,small_bump,small_normal,
+                     small_aocc, small_emission,
 		     material
 		    ]),
     {IL, #{image=>0, internal_image=>1, object=>2, light=>3, mat=>8}}.
@@ -687,16 +726,16 @@ do_menu(#{type:=mat, name:=Name}) ->
      {?__(7,"Duplicate"),menu_cmd(duplicate_material, Name),
       ?__(8,"Duplicate this material")},
      {?__(9,"Delete"),menu_cmd(delete_material, Name),
-      ?__(10,"Delete this material")},
+      ?__(10,"Delete this material"),[{hotkey,wings_hotkey:format_hotkey({?SDLK_DELETE,[]},pretty)}]},
      {?__(11,"Rename"),menu_cmd(rename_material, Name),
-      ?__(12,"Rename this material")}];
+      ?__(12,"Rename this material"),[{hotkey,wings_hotkey:format_hotkey({?SDLK_F2,[]},pretty)}]}, []];
 do_menu(#{type:=object, id:=Id}) ->
     [{?__(13,"Duplicate"),menu_cmd(duplicate_object, Id),
       ?__(14,"Duplicate this object")},
      {?__(15,"Delete"),menu_cmd(delete_object, Id),
-      ?__(16,"Delete this object")},
+      ?__(16,"Delete this object"),[{hotkey,wings_hotkey:format_hotkey({?SDLK_DELETE,[]},pretty)}]},
      {?__(17,"Rename"),menu_cmd(rename_object, Id),
-      ?__(18,"Rename this object")}];
+      ?__(18,"Rename this object"),[{hotkey,wings_hotkey:format_hotkey({?SDLK_F2,[]},pretty)}]}];
 do_menu(#{type:=light, id:=Id}) ->
     [{?__(19,"Edit Light..."),menu_cmd(edit_light, Id),
       ?__(20,"Edit light properties")},
@@ -704,9 +743,9 @@ do_menu(#{type:=light, id:=Id}) ->
      {?__(21,"Duplicate"),menu_cmd(duplicate_object, Id),
       ?__(22,"Duplicate this light")},
      {?__(23,"Delete"),menu_cmd(delete_object, Id),
-      ?__(24,"Delete this light")},
+      ?__(24,"Delete this light"),[{hotkey,wings_hotkey:format_hotkey({?SDLK_DELETE,[]},pretty)}]},
      {?__(25,"Rename"),menu_cmd(rename_object, Id),
-      ?__(26,"Rename this light")}];
+      ?__(26,"Rename this light"),[{hotkey,wings_hotkey:format_hotkey({?SDLK_F2,[]},pretty)}]}];
 do_menu(#{type:=image, id:=Id, image:=Im}=Map) ->
     image_menu(Id, Im, maps:get(mat, Map, unused)).
 
@@ -724,7 +763,8 @@ image_menu_1(Id, _, Mat) ->
 
 image_menu_2(Id, unused) ->
     [separator,
-     {?__(1,"Delete"),menu_cmd(delete_image, Id), ?__(2,"Delete selected image")}
+     {?__(1,"Delete"),menu_cmd(delete_image, Id),
+      ?__(2,"Delete selected image"),[{hotkey,wings_hotkey:format_hotkey({?SDLK_DELETE,[]},pretty)}]}
      |command_image_menu(Id)];
 image_menu_2(Id, {Mat, Type}) ->
     [separator,
@@ -743,16 +783,19 @@ command_image_menu(Id) ->
      {?__(3,"Duplicate"),menu_cmd(duplicate_image, Id),
       ?__(4,"Duplicate selected image")},
      {?__(7,"Rename"),menu_cmd(rename_image, Id),
-      ?__(8,"Rename selected image")}
+      ?__(8,"Rename selected image"),[{hotkey,wings_hotkey:format_hotkey({?SDLK_F2,[]},pretty)}]}
     ].
 
 handle_drop(#{type:=image, id:=Id}, #{type:=mat, name:=Name}) ->
     [{?__(1,"Texture Type"),ignore},
      separator,
      {?__(2,"Diffuse"),tx_cmd(diffuse, Id, Name)},
-     {?__(3,"Gloss"),tx_cmd(gloss, Id, Name)},
+     {?__(8,"Metal&Roughness"),tx_cmd(metallic, Id, Name)},
+     {?__(6,"Ambient Occlusion"),tx_cmd(occlusion, Id, Name)},
      {?__(4,"Bump (HeightMap)"),tx_cmd(bump, Id, Name)},
-     {?__(5,"Bump (NormalMap)"),tx_cmd(normal, Id, Name)}].
+     {?__(5,"Bump (NormalMap)"),tx_cmd(normal, Id, Name)},
+     {?__(7,"Emission"),tx_cmd(emission, Id, Name)}
+    ].
 
 tx_cmd(Type, Id, Mat) ->
     {'VALUE',{assign_texture,Type,Id,Mat}}.
