@@ -251,7 +251,7 @@ check_result(_M, {new_shape,Prefix,#e3d_object{}=Obj,Mat}, St) ->
 check_result(_M, {new_shape,Prefix,Fs,Vs}, St) ->
     Name = object_name(Prefix, St),
     #we{} = We = wings_we:build(Fs, Vs),
-    wings_shape:new(Name, We, St);
+    wings_obj:new(Name, We, St);
 check_result(_M, aborted, _St) -> aborted;
 check_result(_M, {drag,_}=Drag, _) -> Drag;
 check_result(_M, #st{}=St, _) -> St;
@@ -259,6 +259,7 @@ check_result(_M, {save_state,#st{}}=SS, _) -> SS;
 check_result(_M, {push,_}=Push, _) -> Push;
 check_result(_M, {seq,_,_}=Seq, _) -> Seq;
 check_result(_M, keep, _) -> keep;
+check_result(_M, ok, _) -> keep; %% Exporters have always returned ok
 check_result(M, Other, St) ->
     io:format("~w:command/3: bad return value: ~P\n", [M,Other,20]),
     St.
@@ -311,8 +312,11 @@ install_beam(Name) ->
 			  [filename:basename(Name)])
     end.
 
+erl_tar() -> %% Fool dialyzer the spec is wrong for erl_tar:table() in 20.0-20.2
+    list_to_atom("erl_tar"). 
+
 install_tar(Name) ->
-    {ok,Files} = erl_tar:table(Name, [compressed]),
+    {ok,Files} = (erl_tar()):table(Name, [compressed]),
     install_verify_files(Files, Name),
     case erl_tar:extract(Name, [compressed,{cwd,plugin_dir()}]) of
 	ok -> ok;
@@ -689,6 +693,12 @@ check_each_plugin(_, [], _, Acc) -> Acc.
 
 % Further restrictions to bar the output may be stated from within the plugin.
 % See wpc_magnet_mask.erl
+check_plugin_against_flag(_Flag, _PData, wings_shape, Acc) ->
+    %% FIXME: In the future, we really should stop using the plug-in
+    %% mechanism for handling folders (a built-in feature).
+    %% For now, make sure that we fail quickly without attempting
+    %% to load the non-existing wings_shape module from disk.
+    Acc;
 check_plugin_against_flag(Flag, PData, Plugin, Acc) ->
     case catch Plugin:get_data(Flag,PData,Acc) of
       {ok, Result} -> Result;
