@@ -437,6 +437,7 @@ language_name("cs") -> [268,"esk",253]; %Czech
 language_name("en") -> "English";
 language_name("de") -> "Deutsch";
 language_name("es") -> ["Espa",241,"ol"];
+language_name("es-ES") -> ["Espa",241,"ol"];
 language_name("fi") -> "Suomi";
 language_name("fr") -> ["Fran",231,"ais"];
 language_name("hu") -> "Hungarian";
@@ -477,7 +478,7 @@ misc_prefs() ->
 	       {text,num_undo_levels,[{range,{1,128}}]}]},
       {vframe,
        [{label_column,
-	 [{?__(5,"Angle"),auto_rotate_angle}]}],
+	 [{?__(5,"Angle per second"),auto_rotate_angle}]}],
        [{title,?__(7,"Auto Rotate")}]},
       {vframe,
        [{vframe,
@@ -578,6 +579,7 @@ smart_set_value_1(Key, Val, St) ->
 			    "effect the next time Wings 3D is started."),
 		    wings_u:message(Str);
                 show_toolbar ->
+                    delayed_set_value(Key, OldVal, Val),
                     wings_frame:show_toolbar(Val);
 		extended_toolbar ->
 		    delayed_set_value(Key, OldVal, Val),
