@@ -12,8 +12,8 @@
 %%
 
 -module(wings_proxy).
--export([setup/1,quick_preview/1,update/2,draw/3,draw_smooth_edges/1,
-	 smooth/2, smooth_dl/1, invalidate/2,
+-export([setup/1,quick_preview/1,update/2,draw_smooth_edges/2,
+	 smooth/2, smooth_dl/1, flat_dl/1, invalidate/2,
 	 split_proxy/3, update_dynamic/3, reset_dynamic/1]).
 
 -define(NEED_OPENGL, 1).
@@ -61,7 +61,7 @@ invalidate(#sp{}=Pd, 'vab') ->
     %% Used when there are changes having to do with UV coordinates and/or
     %% vertex colors (including toggling their visibility using
     %% View|Show Colors or View|Show Textures).
-    Pd#sp{faces=none,smooth=none,vab=none};
+    Pd#sp{faces=none,smooth=none,vab=none,proxy_edges=none};
 invalidate(#sp{}=Pd, 'dl') ->
     %% Invalidate displays lists.
     Pd#sp{faces=none,smooth=none};
@@ -78,13 +78,15 @@ smooth_dl(#sp{smooth=Smooth}) when Smooth =/= none -> Smooth;
 smooth_dl(#sp{smooth=none, faces=FL}) when FL =/= none -> {[FL,[]], false};
 smooth_dl(_) -> {none, false}.
 
+flat_dl(#sp{faces=FL}) -> FL.
+
 setup(#st{sel=OrigSel}=St) ->
     wings_dl:map(fun(D, Sel) -> setup_1(D, Sel) end, OrigSel),
     {save_state,wings_sel:reset(St)}.
 
 setup_1(#dlo{src_we=#we{id=Id}=We}=D, [{Id,_}|Sel]) when ?IS_ANY_LIGHT(We) ->
     %% Never use proxies on lights.
-    {D,Sel};
+    {D#dlo{proxy=false},Sel};
 setup_1(#dlo{src_we=#we{id=Id},proxy=false, proxy_data=Pd}=D, [{Id,_}|Sel]) ->
     case Pd of
 	none ->
@@ -110,7 +112,7 @@ setup_all(Activate) ->
 
 setup_all(#dlo{src_we=We}=D, _) when ?IS_ANY_LIGHT(We) ->
     %% Never use proxies on lights.
-    D;
+    D#dlo{proxy=false};
 setup_all(#dlo{src_we=#we{id=Id},proxy_data=none}=D, true) ->
     Wire0 = wings_wm:get_prop(wings_wm:this(), wireframed_objects),
     Wire = gb_sets:add(Id, Wire0),
@@ -130,6 +132,8 @@ setup_all(#dlo{src_we=#we{id=Id}}=D, false) ->
 setup_all(D, _) -> D.
 
 update(#dlo{proxy=false}=D, _) -> D;
+update(#dlo{src_we=We}=D, _) when ?IS_ANY_LIGHT(We) ->
+    D#dlo{proxy=false}; %% Never use proxies on lights.
 %% Proxy data is not up to date. Recalculate!
 update(#dlo{proxy_data=#sp{faces=[_]}=Pd0}=D, St) ->
     update(D#dlo{proxy_data=Pd0#sp{faces=none}},St);
@@ -153,8 +157,13 @@ update_edges(D, Pd) ->
 
 update_edges_1(_, _, cage) -> none;
 update_edges_1(_, #sp{vab=#vab{mat_map=MatMap}=Vab}, all) ->
-    [{_Mat,_Type,Start,MCount}|_] = MatMap,
-    Count = Start+MCount,
+    %% expects the materials to be reversed
+    Count = case MatMap of
+                [{_Mat,_Type,0,MCount}] -> MCount;
+                [{_Mat,_Type,Start,MCount}|_]
+                  when Start =/= 0 ->  %% Assert order of material faces
+                    Start + MCount
+            end,
     fun() ->
 	    Extra = [face_normals],
 	    wings_draw_setup:enable_pointers(Vab, Extra),
@@ -230,81 +239,21 @@ any_proxy() ->
     wings_dl:fold(fun(#dlo{proxy=false}, A) -> A;
 		     (#dlo{}, _) -> true end, false).
 
+draw_smooth_edges(#dlo{drag=none}=D, Style) ->
+    draw_edges(D, Style);
+draw_smooth_edges(D, _) ->
+    draw_edges(D, cage).
 
-draw(#dlo{proxy=false}, _Wire, _) -> ok;
-draw(#dlo{proxy_data=#sp{faces=Dl},drag=none}=D, Wire, SceneLights) ->
-    draw_1(D, Dl, Wire, proxy_static_opacity, cage, SceneLights);
-draw(#dlo{proxy_data=#sp{faces=Dl}}=D, _Wire, SceneLights) ->
-    draw_1(D, Dl, true, proxy_moving_opacity, cage, SceneLights).
-
-draw_1(#dlo{proxy_data=#sp{src_we=We}} = D, Dl, Wire,
-       Key, EdgeStyleKey, SceneLights) ->
-    gl:shadeModel(?GL_SMOOTH),
-    wings_render:enable_lighting(SceneLights),
-    gl:enable(?GL_POLYGON_OFFSET_FILL),
-    wings_render:polygonOffset(3.0),
-    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
-    case wings_gl:is_ext('GL_ARB_imaging') of
-	false -> ok;
-	true ->
-	    case wings_pref:get_value(Key) of
-		1.0 -> ok;
-		Opacity ->
-		    gl:enable(?GL_BLEND),
-		    gl:blendFunc(?GL_CONSTANT_ALPHA, ?GL_ONE_MINUS_CONSTANT_ALPHA),
-		    gl:blendColor(0.0, 0.0, 0.0, Opacity)
-	    end
-    end,
-    case wings_we:is_open(We) andalso wings_pref:get_value(show_backfaces) of
-	true -> gl:disable(?GL_CULL_FACE);
-	false -> ignore
-    end,
-    wings_dl:call(Dl),
-    gl:enable(?GL_CULL_FACE),
-    gl:disable(?GL_POLYGON_OFFSET_FILL),
-    wings_render:disable_lighting(),
-    gl:shadeModel(?GL_FLAT),
-    draw_edges(D, Wire, EdgeStyleKey),
-    gl:disable(?GL_BLEND).
-
-draw_smooth_edges(#dlo{drag=none}=D) ->
-    draw_edges(D, true, wings_pref:get_value(proxy_shaded_edge_style));
-draw_smooth_edges(D) ->
-    draw_edges(D, true, cage).
-
-draw_edges(_, false, _) -> ok;
-draw_edges(D, true, EdgeStyle) ->
-    case wings_pref:get_value(aa_edges) of
-	true ->
-	    gl:enable(?GL_LINE_SMOOTH),
-	    gl:enable(?GL_BLEND),
-	    gl:blendFunc(?GL_SRC_ALPHA, ?GL_ONE_MINUS_SRC_ALPHA),
-	    gl:hint(?GL_LINE_SMOOTH_HINT, ?GL_NICEST);
-	false ->
-	    ok
-    end,
-    draw_edges_1(D, EdgeStyle).
-
-draw_edges_1(#dlo{edges=Edges}, cage) ->
-    gl:color3fv(wings_pref:get_value(edge_color)),
-    gl:lineWidth(1.0),
-    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
-    gl:disable(?GL_CULL_FACE),
-    wings_dl:call(Edges),
-    gl:enable(?GL_CULL_FACE);
-draw_edges_1(#dlo{proxy_data=#sp{proxy_edges=ProxyEdges}}, _) ->
-    gl:color3fv(wings_pref:get_value(edge_color)),
-    gl:lineWidth(1.0),
-    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
+draw_edges(#dlo{edges=Edges}, cage) ->
+    wings_dl:call(Edges);
+draw_edges(#dlo{proxy_data=#sp{proxy_edges=ProxyEdges}}, _) ->
     wings_dl:call(ProxyEdges).
 
 proxy_smooth(We0, Pd0, St) ->
     Level = wings_pref:get_value(proxy_opencl_level),
-    if is_integer(Level),Level > 0 ->
-	    Impl = wings_cc;
-       true ->
-	    Impl = ?MODULE
-    end,
+    Impl = if is_integer(Level),Level > 0 -> wings_cc;
+              true ->?MODULE
+           end,
     case proxy_needs_update(We0, Pd0) of
 	{false,_} ->
 	    Pd0;
