@@ -21,7 +21,7 @@
 -include("wings.hrl").
 
 init() ->
-    init_shaders(),
+    wings_shaders:init(),
     wings_pref:set_default(multisample, true),
     init_polygon_stipple().
 
@@ -35,8 +35,7 @@ render(#st{selmode=Mode}=St) ->
     ?CHECK_ERROR(),
     gl:pushAttrib(?GL_CURRENT_BIT bor ?GL_ENABLE_BIT bor
 		  ?GL_TEXTURE_BIT bor ?GL_POLYGON_BIT bor
-		  ?GL_LINE_BIT bor ?GL_COLOR_BUFFER_BIT bor
-		  ?GL_LIGHTING_BIT),
+		  ?GL_LINE_BIT bor ?GL_COLOR_BUFFER_BIT),
     gl:enable(?GL_DEPTH_TEST),
     gl:enable(?GL_CULL_FACE),
     case wings_pref:get_value(multisample) of
@@ -54,8 +53,15 @@ render(#st{selmode=Mode}=St) ->
     axis_letters(PM,MM,Yon),
     show_camera_image_plane(),
     gl:popAttrib(),
+    call_post_hook(St),
     wings_develop:gl_error_check("Rendering scene").
 
+call_post_hook(St) ->
+    case wings_wm:lookup_prop(postdraw_hook) of
+        none -> ok;
+        {value,{_Id,Fun}} -> Fun(St)
+    end.
+
 polygonOffset(M) ->
     case get(polygon_offset) of
 	undefined ->
@@ -81,11 +87,6 @@ non_polygon_offset(Offset, TPM) ->
 %%% Internal functions follow.
 %%%
 
-init_shaders() ->
-    try wings_shaders:init()
-    catch _:_Err -> ok
-    end.
-
 init_polygon_stipple() ->
     P = <<16#DD,16#DD,16#DD,16#DD,16#77,16#77,16#77,16#77,
 	 16#DD,16#DD,16#DD,16#DD,16#77,16#77,16#77,16#77,
@@ -105,96 +106,209 @@ init_polygon_stipple() ->
 	 16#DD,16#DD,16#DD,16#DD,16#77,16#77,16#77,16#77>>,
     gl:polygonStipple(P).
 
-render_objects(Mode, PM, SceneLights) ->
-    Dls = wings_dl:display_lists(),
-    case wings_wm:get_prop(workmode) of
-	false ->
-	    render_smooth_objects(Dls, Mode, false, PM, SceneLights),
-	    render_smooth_objects(Dls, Mode, true, PM, SceneLights);
-	true ->
-	    render_work_objects(Dls, Mode, PM, SceneLights)
+setup_scene_lights(false, _) ->
+    false;
+setup_scene_lights(true, Lights) ->
+    {Amb0, SL} = wings_light:global_lights(Lights),
+    case Amb0 of
+        [] ->
+            gl:lightModelfv(?GL_LIGHT_MODEL_AMBIENT, {0.0,0.0,0.0,1.0}),
+            wings_shaders:use_prog(ambient_light),
+            SL;
+        [Amb|RestAmb] ->
+            wings_light:setup_light(Amb),
+            RestAmb ++ SL
     end.
 
-render_smooth_objects([D|Dls], Mode, RenderTrans, PM, SceneLights) ->
-    render_object(D, Mode, false, RenderTrans, PM, SceneLights),
-    render_smooth_objects(Dls, Mode, RenderTrans, PM, SceneLights);
-render_smooth_objects([], _, _, _, _) -> ok.
+render_objects(Mode, PM, UseSceneLights) ->
+    Dls = wings_dl:display_lists(),
+    {Open,Closed,Lights} = split_objects(Dls, [], [], []),
+    render_lights(Lights),
+    SL = setup_scene_lights(UseSceneLights, Lights),
+    case wings_wm:get_prop(workmode) of
+	false ->
+            case UseSceneLights of
+                true -> render_smooth_objects(Open, Closed, ambient); %% amb pass
+                false -> ignore
+            end,
+	    render_smooth_objects(Open, Closed, SL),
+            render_wire(Dls, Mode, true),
+            render_sel_highlight(Dls, Mode, true, PM);
+	true ->
+            case UseSceneLights of
+                true -> render_work_objects(Open, Closed, ambient); %% amb pass
+                false -> ignore
+            end,
+            render_work_objects(Open, Closed, SL),
+            render_wire(Dls, Mode, false),
+            render_sel_highlight(Dls, Mode, false, PM)
+    end.
 
-render_work_objects([D|Dls], Mode, PM, SceneLights) ->
-    render_object(D, Mode, true, false, PM, SceneLights),
-    render_work_objects(Dls, Mode, PM, SceneLights);
-render_work_objects([], _, _, _) -> ok.
+render_lights(Lights) ->
+    wings_shaders:use_prog(1),
+    render_work_objects_0(Lights, false),
+    gl:color4ub(255, 255, 255, 255).
 
-render_object(#dlo{drag={matrix,_,_,Matrix}}=D, Mode, Work, RT, PM, SceneLights) ->
+render_work_objects(Open, Closed, SceneLights) ->
+    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
+    gl:enable(?GL_POLYGON_OFFSET_FILL),
+    polygonOffset(2.0),
+    gl:shadeModel(?GL_SMOOTH),
+    enable_lighting(SceneLights),
+    render_work_objects_0(Closed, SceneLights),
+    wings_pref:get_value(show_backfaces) andalso gl:disable(?GL_CULL_FACE),
+    render_work_objects_0(Open, SceneLights),
+    gl:enable(?GL_CULL_FACE),
+    disable_lighting(),
+    gl:shadeModel(?GL_FLAT),
+    ok.
+
+render_work_objects_0([D|Dls], SceneLights) ->
+    render_object(D, true, false, SceneLights),
+    render_work_objects_0(Dls, SceneLights);
+render_work_objects_0([], _) -> ok.
+
+render_smooth_objects(Open, Closed, SceneLights) ->
+    gl:shadeModel(?GL_SMOOTH),
+    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
+    enable_lighting(SceneLights),
+    gl:enable(?GL_POLYGON_OFFSET_FILL),
+    polygonOffset(2.0),
+    gl:enable(?GL_CULL_FACE),
+    render_smooth_objects_0(Closed, false, SceneLights),
+    wings_pref:get_value(show_backfaces) andalso gl:disable(?GL_CULL_FACE),
+    render_smooth_objects_0(Open, false, SceneLights),
+    gl:enable(?GL_BLEND),
+    gl:blendFunc(?GL_SRC_ALPHA, ?GL_ONE_MINUS_SRC_ALPHA),
+    gl:depthMask(?GL_FALSE),
+    gl:enable(?GL_CULL_FACE),
+    render_smooth_objects_0(Closed, true, SceneLights),
+    wings_pref:get_value(show_backfaces) andalso gl:disable(?GL_CULL_FACE),
+    render_smooth_objects_0(Open, true, SceneLights),
+    gl:enable(?GL_CULL_FACE),
+    gl:disable(?GL_BLEND),
+    gl:depthMask(?GL_TRUE).
+
+render_smooth_objects_0([D|Dls], RenderTrans, SceneLights) ->
+    render_object(D, false, RenderTrans, SceneLights),
+    render_smooth_objects_0(Dls, RenderTrans, SceneLights);
+render_smooth_objects_0([], _, _) -> ok.
+
+render_object(#dlo{drag={matrix,_,_,Matrix}}=D, Work, RT, SceneLights) ->
     gl:pushMatrix(),
     gl:multMatrixf(Matrix),
-    render_object_1(D, Mode, Work, RT, PM, SceneLights),
+    render_object_1(D, Work, RT, SceneLights),
     gl:popMatrix();
-render_object(D, Mode, Work, RT, PM, SceneLights) ->
-    render_object_1(D, Mode, Work, RT, PM, SceneLights).
+render_object(D, Work, RT, SceneLights) ->
+    render_object_1(D, Work, RT, SceneLights).
 
-render_object_1(#dlo{mirror=none}=D, Mode, Work, RenderTrans, PM, SceneLights) ->
-    render_object_2(D, Mode, Work, RenderTrans, PM, SceneLights);
-render_object_1(#dlo{mirror=Matrix}=D, Mode, Work, RenderTrans, PM, SceneLights) ->
-    render_object_2(D, Mode, Work, RenderTrans, PM, SceneLights),
+render_object_1(#dlo{mirror=none}=D, Work, RenderTrans, SceneLights) ->
+    render_object_2(D, Work, RenderTrans, SceneLights);
+render_object_1(#dlo{mirror=Matrix}=D, Work, RenderTrans, SceneLights) ->
+    render_object_2(D, Work, RenderTrans, SceneLights),
     gl:frontFace(?GL_CW),
     gl:pushMatrix(),
     gl:multMatrixf(Matrix),
-    render_object_2(D, Mode, Work, RenderTrans, PM, SceneLights),
+    render_object_2(D, Work, RenderTrans, SceneLights),
     gl:popMatrix(),
     gl:frontFace(?GL_CCW).
 
-render_object_2(#dlo{src_we=We}=D, _, _, false, _, _) when ?IS_LIGHT(We) ->
-    wings_light:render(D);
-render_object_2(#dlo{src_we=We}, _, _, true, _, _) when ?IS_LIGHT(We) ->
+render_object_2(D, true, _, SceneLights) ->
+    render_plain(D, SceneLights);
+render_object_2(#dlo{transparent=true}=D, false, false, SceneLights) ->
+    render_smooth(D, false, SceneLights);
+render_object_2(D, false, RenderTrans, SceneLights) ->
+    render_smooth(D, RenderTrans, SceneLights).
+
+render_plain(#dlo{work=Faces,src_we=We,proxy=false}, SceneLights) ->
+    case wire(We) of
+	false -> render_lighted(Faces, SceneLights);
+        true -> ok
+    end;
+render_plain(#dlo{proxy_data=PD, drag=Drag}, SceneLights) ->
+    polygonOffset(3.0),
+    Faces = wings_proxy:flat_dl(PD),
+    Key = case Drag =:= none of
+              true  -> proxy_static_opacity;
+              false -> proxy_moving_opacity
+          end,
+    if SceneLights =:= false ->
+            Blend = wings_gl:is_ext('GL_ARB_imaging') andalso wings_pref:get_value(Key),
+            case Blend of
+                false ->
+                    render_lighted(Faces, SceneLights);
+                1.0 ->
+                    render_lighted(Faces, SceneLights);
+                _ ->
+                    gl:enable(?GL_BLEND),
+		    gl:blendFunc(?GL_CONSTANT_ALPHA, ?GL_ONE_MINUS_CONSTANT_ALPHA),
+		    gl:blendColor(0.0, 0.0, 0.0, Blend),
+                    render_lighted(Faces, SceneLights),
+                    gl:disable(?GL_BLEND)
+            end;
+       true ->
+            render_lighted(Faces, SceneLights)
+    end.
+
+render_smooth(#dlo{work=Work,smooth=Smooth0,transparent=Trans0,
+		   proxy=Proxy,proxy_data=PD},
+	      RenderTrans, SceneLights) ->
+    {Smooth, _Trans} =
+        case Proxy of
+            true -> wings_proxy:smooth_dl(PD);
+            false -> {Smooth0,Trans0}
+        end,
+
+    Draw = case {Smooth,RenderTrans} of
+               {none,false}   -> Work;
+               {[Op,_],false} -> Op;
+               {[_,Tr],true}  -> Tr;
+               {_,_} -> none
+           end,
+    render_lighted(Draw, SceneLights).
+
+render_lighted(Faces, false) ->
+    wings_dl:call(Faces);
+render_lighted(Faces, ambient) ->
+    wings_dl:call(Faces);
+render_lighted(Faces, Lights) ->
+    Do = fun(Light) ->
+                 wings_light:setup_light(Light),
+                 wings_dl:call(Faces)
+         end,
+    _ = [Do(Light) || Light <- Lights],
+    ok.
+
+enable_lighting(false) ->
+    Lighting = wings_pref:get_value(number_of_lights),
+    gl:color4ub(255, 255, 255, 255), %% Needed when vertex colors are not set
+    wings_shaders:use_prog(Lighting);
+enable_lighting(ambient) ->
     ok;
-render_object_2(D, Mode, true, _, PM, SceneLights) ->
-    render_plain(D, Mode, PM, SceneLights);
-render_object_2(#dlo{transparent=true}=D, _, false, false, PM, SceneLights) ->
-    gl:disable(?GL_CULL_FACE),
-    render_smooth(D, false, PM, SceneLights),
-    gl:enable(?GL_CULL_FACE);
-render_object_2(#dlo{transparent=true}=D, _, false, true, PM, SceneLights) ->
-    render_smooth(D, true, PM, SceneLights);
-render_object_2(#dlo{transparent=false}=D, _, false, RenderTrans, PM, SceneLights) ->
-    render_smooth(D, RenderTrans, PM, SceneLights).
+enable_lighting(_) ->
+    gl:color4ub(255, 255, 255, 255),
+    gl:depthFunc(?GL_LEQUAL),
+    gl:enable(?GL_BLEND),
+    gl:blendFunc(?GL_ONE, ?GL_ONE).
 
-render_plain(#dlo{work=Faces,edges=Edges,open=Open,
-		  src_we=We,proxy=false}=D, SelMode, PM, SceneLights) ->
-    %% Draw faces for winged-edge-objects.
-    Wire = wire(We),
-    case Wire of
-	false ->
-	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
-	    gl:enable(?GL_POLYGON_OFFSET_FILL),
-	    polygonOffset(2.0),
-	    gl:shadeModel(?GL_SMOOTH),
-	    enable_lighting(SceneLights),
-	    case wings_pref:get_value(show_backfaces) of
-		true when Open ->
-		    gl:lightModeli(?GL_LIGHT_MODEL_TWO_SIDE, ?GL_TRUE),
-		    gl:disable(?GL_CULL_FACE),
-		    wings_dl:call(Faces),
-		    gl:enable(?GL_CULL_FACE);
-		_ ->
-		    wings_dl:call(Faces)
-	    end,
-	    disable_lighting(),
-	    gl:shadeModel(?GL_FLAT);
-	true -> ok
-    end,
+disable_lighting() ->
+    wings_gl:use_prog(0),
+    gl:disable(?GL_BLEND),
+    gl:depthMask(?GL_TRUE).
 
-    %% Draw edges.
-    case Wire orelse wings_pref:get_value(show_edges) of
-	false -> ok;
-	true ->
-	    case {SelMode,wings_pref:get_value(edge_color)} of
+render_wire(Dls, SelMode, false) ->
+    WOs = wings_wm:get_prop(wireframed_objects),
+    Show = wings_pref:get_value(show_edges),
+    case split_wires(Dls, WOs, Show, [], [], []) of
+        {[],[],[]} -> ok;
+        {Ws,Os,PWs} ->
+            case {SelMode,wings_pref:get_value(edge_color)} of
 		{body,{0.0,0.0,0.0}} ->
 		    gl:color3f(0.3, 0.3, 0.3);
 		{_,EdgeColor} ->
 		    gl:color3fv(EdgeColor)
 	    end,
-	    case wings_pref:get_value(aa_edges) of
+            case wings_pref:get_value(aa_edges) of
 		true ->
 		    gl:enable(?GL_LINE_SMOOTH),
 		    gl:enable(?GL_BLEND),
@@ -207,37 +321,133 @@ render_plain(#dlo{work=Faces,edges=Edges,open=Open,
 	    gl:lineWidth(edge_width(SelMode)),
 	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
             gl:depthFunc(?GL_LEQUAL),
-	    case Wire andalso wings_wm:get_prop(show_wire_backfaces) of
-		true ->
-		    gl:disable(?GL_CULL_FACE),
-		    wings_dl:call(Edges),
-		    gl:enable(?GL_CULL_FACE);
-		false ->
-		    wings_dl:call(Edges)
-	    end,
-            gl:depthFunc(?GL_LESS),
-            ok
-    end,
-    render_plain_rest(D, Wire, SelMode, PM);
-render_plain(#dlo{src_we=We}=D, SelMode, PM, SceneLights) ->
-    Wire = wire(We),
-    wings_proxy:draw(D, Wire, SceneLights),
-    render_plain_rest(D, Wire, SelMode, PM).
+            wings_wm:get_prop(show_wire_backfaces)
+                andalso gl:disable(?GL_CULL_FACE),
+            _ = [render_wire_object(D, cage) || D <- Ws],
+            gl:disable(?GL_CULL_FACE),
+            _ = [render_wire_object(D, cage) || D <- Os],
+            gl:enable(?GL_CULL_FACE),
+            gl:color3fv(wings_pref:get_value(edge_color)),
+            gl:lineWidth(1.0),
+            _ = [render_wire_object(D, cage) || D <- PWs],
+            gl:enable(?GL_CULL_FACE),
+            gl:depthFunc(?GL_LESS)
+    end;
+render_wire(Dls, _, true) ->
+    gl:enable(?GL_CULL_FACE),
+    gl:disable(?GL_POLYGON_OFFSET_FILL),
+    gl:depthMask(?GL_TRUE),
+    disable_lighting(),
+    gl:shadeModel(?GL_FLAT),
+    gl:depthFunc(?GL_LEQUAL),
+    WOs = wings_wm:get_prop(wireframed_objects),
+    {Ws,[],PWs} = split_wires(Dls, WOs, false, [], [], []),
+    gl:color3fv(wings_pref:get_value(edge_color)),
+    gl:lineWidth(1.0),
+    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
+    _ = [render_wire_object(D, cage) || D <- Ws],
+    Style = wings_pref:get_value(proxy_shaded_edge_style),
+    _ = [render_wire_object(D, Style) || D <- PWs],
+    ok.
 
-render_plain_rest(#dlo{}=D, Wire, SelMode, PM) ->
+render_wire_object(#dlo{drag={matrix,_,_,Matrix}}=D, PStyle) ->
+    gl:pushMatrix(),
+    gl:multMatrixf(Matrix),
+    render_wire_object_1(D, PStyle),
+    gl:popMatrix();
+render_wire_object(D, PStyle) ->
+    render_wire_object_1(D, PStyle).
+
+render_wire_object_1(#dlo{mirror=none, edges=Edges, proxy=false}, _) ->
+    wings_dl:call(Edges);
+render_wire_object_1(#dlo{mirror=Matrix, edges=Edges, proxy=false}, _) ->
+    wings_dl:call(Edges),
+    gl:frontFace(?GL_CW),
+    gl:pushMatrix(),
+    gl:multMatrixf(Matrix),
+    wings_dl:call(Edges),
+    gl:popMatrix(),
+    gl:frontFace(?GL_CCW);
+render_wire_object_1(#dlo{mirror=none}=D, PStyle) ->
+    wings_proxy:draw_smooth_edges(D, PStyle);
+render_wire_object_1(#dlo{mirror=Matrix}=D, PStyle) ->
+    wings_proxy:draw_smooth_edges(D, PStyle),
+    gl:frontFace(?GL_CW),
+    gl:pushMatrix(),
+    gl:multMatrixf(Matrix),
+    wings_proxy:draw_smooth_edges(D, PStyle),
+    gl:popMatrix(),
+    gl:frontFace(?GL_CCW).
+
+split_objects([#dlo{src_we=We}=D|Dls], Open, Closed, Lights) when ?IS_ANY_LIGHT(We) ->
+    split_objects(Dls, Open, Closed, [D|Lights]);
+split_objects([#dlo{open=true}=D|Dls], Open, Closed, Lights) ->
+    split_objects(Dls, [D|Open], Closed, Lights);
+split_objects([#dlo{open=false}=D|Dls], Open, Closed, Lights) ->
+    split_objects(Dls, Open, [D|Closed], Lights);
+split_objects([], Open, Closed, Lights) ->
+    {Open, Closed, Lights}.
+
+split_wires([#dlo{src_we=#we{id=Id}, proxy=Proxy}=D|Dls], WOs, Show, Wires, Others,Proxis) ->
+    case gb_sets:is_member(Id, WOs) of
+        true when Proxy -> split_wires(Dls, WOs, Show, Wires, Others, [D|Proxis]);
+        true -> split_wires(Dls, WOs, Show, [D|Wires], Others,Proxis);
+        false when Proxy -> split_wires(Dls, WOs, Show, Wires, Others,Proxis);
+        false when Show -> split_wires(Dls, WOs, Show, Wires, [D|Others], Proxis);
+        false -> split_wires(Dls, WOs, Show, Wires, Others,Proxis)
+    end;
+split_wires([], _, _, Ws, Os, Ps) ->
+    {Ws, Os, Ps}.
+
+render_sel_highlight(Dls, SelMode, false, PM) ->
     gl:disable(?GL_POLYGON_OFFSET_LINE),
     gl:disable(?GL_POLYGON_OFFSET_FILL),
-
-    NeedPush = sel_need_matrix_push(D),
     gl:matrixMode(?GL_PROJECTION),
     gl:pushMatrix(),
     non_polygon_offset(1.0, PM),
+    gl:matrixMode(?GL_MODELVIEW),
     gl:depthFunc(?GL_LEQUAL),
+    _ = [render_sel(D, SelMode, false) || D <- Dls],
+    _ = [render_sel(D, SelMode, true) || D <- Dls],
+    gl:matrixMode(?GL_PROJECTION),
+    gl:popMatrix(),
+    gl:depthFunc(?GL_LESS),
+    gl:matrixMode(?GL_MODELVIEW),
+    %% arbitrary placement in the grand scheme of things
+    _ = [wings_plugin:draw(plain,D,SelMode) || D <- Dls],
+    ok;
+render_sel_highlight(Dls, SelMode, true, _PM) ->
+    _ = [render_sel(D, SelMode, true) || D <- Dls],
+    gl:depthFunc(?GL_LESS),
+    %% arbitrary placement in the grand scheme of things
+    _ = [wings_plugin:draw(smooth,D,none) || D <- Dls],
+    ok.
+
+render_sel(#dlo{drag={matrix,_,_,Matrix}}=D, Mode, Sel) ->
+    gl:pushMatrix(),
+    gl:multMatrixf(Matrix),
+    render_sel_1(D, Mode, Sel),
+    gl:popMatrix();
+render_sel(D, Mode, Sel) ->
+    render_sel_1(D, Mode, Sel).
+
+render_sel_1(#dlo{mirror=none}=D, Mode, Sel) ->
+    render_sel_2(D, Mode, Sel);
+render_sel_1(#dlo{mirror=Matrix}=D, Mode, Sel) ->
+    render_sel_2(D, Mode, Sel),
+    gl:frontFace(?GL_CW),
+    gl:pushMatrix(),
+    gl:multMatrixf(Matrix),
+    render_sel_2(D, Mode, Sel),
+    gl:popMatrix(),
+    gl:frontFace(?GL_CCW).
+
+render_sel_2(#dlo{}=D, SelMode, false) ->
     draw_hard_edges(D, SelMode),
     draw_vertices(D, SelMode),
-    draw_normals(D),
-    NeedPush orelse gl:popMatrix(),
-    case Wire of
+    draw_normals(D);
+render_sel_2(#dlo{src_we=We}=D, _SelMode, true) ->
+    case wire(We) of
 	true ->
 	    gl:disable(?GL_CULL_FACE),
 	    draw_orig_sel(D),
@@ -247,97 +457,12 @@ render_plain_rest(#dlo{}=D, Wire, SelMode, PM) ->
             draw_orig_sel(D),
 	    draw_sel(D)
     end,
-    draw_hilite(D),
-    gl:depthFunc(?GL_LESS),
-    NeedPush andalso gl:popMatrix(),
-    gl:matrixMode(?GL_MODELVIEW),
-    draw_plugins(plain,D,SelMode). %% arbitrary placement in the grand scheme of things
-
-render_smooth(#dlo{work=Work,edges=Edges,smooth=Smooth0,transparent=Trans0,
-		   src_we=We,proxy=Proxy,proxy_data=PD,open=Open}=D,
-	      RenderTrans, PM, SceneLights) ->
-    gl:shadeModel(?GL_SMOOTH),
-    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
-    enable_lighting(SceneLights),
-    gl:enable(?GL_POLYGON_OFFSET_FILL),
-    polygonOffset(2.0),
-
-    case Proxy of
-	true ->
-	    {Smooth, Trans} = wings_proxy:smooth_dl(PD);
-	false ->
-	    Smooth = Smooth0,
-	    Trans  = Trans0
-    end,
-
-    case Trans orelse Open of
-	false -> gl:lightModeli(?GL_LIGHT_MODEL_TWO_SIDE, ?GL_FALSE);
-	true -> gl:lightModeli(?GL_LIGHT_MODEL_TWO_SIDE, ?GL_TRUE)
-    end,
-
-    case RenderTrans of
-	true ->
-	    gl:enable(?GL_BLEND),
-	    gl:blendFunc(?GL_SRC_ALPHA, ?GL_ONE_MINUS_SRC_ALPHA),
-	    gl:depthMask(?GL_FALSE);
-	false ->
-	    gl:disable(?GL_BLEND),
-	    gl:depthMask(?GL_TRUE)
-    end,
-
-    case wings_pref:get_value(show_backfaces) of
-	true when Open -> gl:disable(?GL_CULL_FACE);
-	_ -> ok
-    end,
-    case {Smooth,RenderTrans} of
-	{none,false} ->   wings_dl:call(Work);
-	{[Op,_],false} -> wings_dl:call(Op);
-	{[_,Tr],true} ->  wings_dl:call(Tr);
-	{_,_} -> ok
-    end,
-
-    gl:enable(?GL_CULL_FACE),
-
-    gl:disable(?GL_POLYGON_OFFSET_FILL),
-    gl:depthMask(?GL_TRUE),
-    disable_lighting(),
-    gl:shadeModel(?GL_FLAT),
-    gl:depthFunc(?GL_LEQUAL),
-    case wire(We) of
-	true when Proxy =:= false ->
-	    gl:color3fv(wings_pref:get_value(edge_color)),
-	    gl:lineWidth(1.0),
-	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
-	    wings_dl:call(Edges);
-	true ->
-	    wings_proxy:draw_smooth_edges(D);
-	false -> ok
-    end,
-    NeedPush = sel_need_matrix_push(D),
-    if NeedPush ->
-            gl:matrixMode(?GL_PROJECTION),
-            gl:pushMatrix(),
-            non_polygon_offset(1.0, PM);
-       true -> false
-    end,
-    draw_orig_sel(D),
-    draw_sel(D),
-    draw_hilite(D),
-    gl:depthFunc(?GL_LESS),
-    NeedPush andalso gl:popMatrix(),
-    gl:matrixMode(?GL_MODELVIEW),
-    draw_plugins(smooth,D,none).
+    draw_hilite(D).
 
 wire(#we{id=Id}) ->
     W = wings_wm:get_prop(wireframed_objects),
     gb_sets:is_member(Id, W).
 
-sel_need_matrix_push(#dlo{src_sel={edge,_}})   -> true;
-sel_need_matrix_push(#dlo{src_sel={vertex,_}}) -> true;
-sel_need_matrix_push(#dlo{hilite={edge,_}})    -> true;
-sel_need_matrix_push(#dlo{hilite={vertex,_}})  -> true;
-sel_need_matrix_push(_) -> false.
-
 draw_sel(#dlo{sel=none}) -> ok;
 draw_sel(#dlo{sel=SelDlist,src_sel={edge,_}}) ->
     gl:lineWidth(wings_pref:get_value(selected_edge_width)),
@@ -421,7 +546,7 @@ draw_hard_edges(#dlo{hard=Hard}, SelMode) ->
     gl:lineWidth(hard_edge_width(SelMode)),
     gl:color3fv(wings_pref:get_value(hard_edge_color)),
     wings_dl:call(Hard).
-	
+
 draw_normals(#dlo{normals=none}) -> ok;
 draw_normals(#dlo{normals=Ns}) ->
     gl:color3f(0.0, 0.0, 1.0),
@@ -434,9 +559,6 @@ edge_width(_) -> 1.
 hard_edge_width(edge) -> wings_pref:get_value(hard_edge_width);
 hard_edge_width(_) -> max(wings_pref:get_value(hard_edge_width) - 1, 1).
 
-draw_plugins(Flag,D,Selmode) ->
-    wings_plugin:draw(Flag, D, Selmode).
-
 ground_and_axes(PM,MM) ->
     Axes = wings_wm:get_prop(show_axes),
     GridSize = groundplane(Axes, PM, MM),
@@ -738,32 +860,6 @@ update_bb_center({{Cx,Cy,Cz}=Center,Color}) ->
 	end,
     wings_vbo:new(D, Data).
 
-enable_lighting(SceneLights) ->
-    Progs = get(light_shaders),
-    UseProg = Progs =/= undefined andalso
-	      not SceneLights andalso
-	      wings_pref:get_value(number_of_lights) =:= 2,
-    case UseProg of
-	false ->
-	    gl:enable(?GL_LIGHTING);
-	true ->
-	    NumShaders = wings_pref:get_value(active_shader),
-	    {Prog,_Name} = element(NumShaders, Progs),
-	    %% Reset color. Needed by some drivers.
-	    %% We put it here and not in apply_material, because we
-	    %% can't use some optimizations (e.g. reuse display lists)
-	    %% when drawing selected objects.
-	    gl:color4ub(255, 255, 255, 255), 
-	    gl:useProgram(Prog)
-    end.
-
-disable_lighting() ->
-    gl:disable(?GL_LIGHTING),
-    case get(light_shaders) /= undefined of
-	true -> gl:useProgram(0);
-	false -> ok
-    end.
-
 mini_axis_icon(MM) ->
     case mini_axis_icon_key() of
 	none -> ok;
