@@ -15,6 +15,7 @@
 -module(wings_shapes).
 -export([menu/2,command/2]).
 -export([transform_obj_dlg/0, transform_obj/4, transform_obj/2]).
+-export([tri_sphere/1]).
 -include("wings.hrl").
 
 -import(lists, [map/2,seq/2,seq/3]).
@@ -103,7 +104,7 @@ command(image, St) -> wings_image:create(St).
 build_shape(Prefix, Fs, Vs, #st{onext=Oid}=St) ->
     We = wings_we:build(Fs, Vs),
     Name = Prefix++integer_to_list(Oid),
-    wings_shape:new(Name, We, St).
+    wings_obj:new(Name, We, St).
 
 tetrahedron(Ask, St) when is_atom(Ask) ->
     Q = [{label_column, [
@@ -406,3 +407,123 @@ transform_obj({Rot_X,Rot_Y,Rot_Z}, {Mov_X,Mov_Y,Mov_Z}, Ground, Vs) ->
     Mt = e3d_mat:translate(Mov_X,Y,Mov_Z),
     Mat = e3d_mat:mul(Mt,Mr),
     [e3d_mat:mul_point(Mat, V) || V <- Vs].
+
+
+%%% Other shapes for internal rendering
+
+-define(XPLUS, {1.0,0.0,0.0}).
+-define(XMIN, {-1.0,0.0,0.0}).
+-define(YPLUS, {0.0,1.0,0.0}).
+-define(YMIN, {0.0,-1.0,0.0}).
+-define(ZPLUS, {0.0,0.0,1.0}).
+-define(ZMIN, {0.0,0.0,-1.0}).
+
+-define(octahedron,
+	[{?ZPLUS, ?XPLUS, ?YPLUS},
+	 {?XMIN,  ?ZPLUS, ?YPLUS},
+	 {?ZPLUS, ?XMIN,  ?YMIN },
+	 {?ZPLUS, ?YMIN,  ?XPLUS},
+	 {?YPLUS, ?XPLUS, ?ZMIN },
+	 {?XMIN,  ?YPLUS, ?ZMIN },
+	 {?YMIN,  ?XMIN,  ?ZMIN },
+	 {?XPLUS, ?YMIN,  ?ZMIN }]).
+
+%% func tri_sphere(Options) -> {Size::integer(), Tris::term(), [Extra]}
+%% Replaces glu:quadric
+%% Returns the number of triangles and the triangles in a list
+%% or in a binary if option binary is true.
+%% Extra contents depends on Options.
+%% Options:
+%%     subd    Subdivision level default 1
+%%     binary  All output is binary default false
+%%     ccw     Winding order counter clockwise default true
+%%     scale   Scale output default 1,
+%%     normals Add normals to the extra list default false
+%%     Extra = [NormalsIfIncluded]
+tri_sphere(Opts) when is_map(Opts) ->
+    Subd   = maps:get(subd, Opts, 1),
+    Binary = maps:get(binary, Opts, false),
+    CCW    = maps:get(ccw, Opts, true),
+    Scale  = maps:get(scale, Opts, 1),
+    Normal = maps:get(normals, Opts, false),
+    %% Do the work
+    Tris   = subd_tris(1, Subd, ?octahedron),
+    case Binary of
+	true ->
+	    BinTris = list_to_bin(Tris, CCW, Scale),
+	    Ns = if not Normal -> [];
+		    Scale =:= 1 -> [BinTris];
+		    true -> [list_to_bin(Tris, CCW, 1)]
+		 end,
+	    {size(BinTris) div (9*4), BinTris, Ns};
+	false ->
+	    Scaled = convert_list(Tris, CCW, Scale),
+	    Ns = if not Normal -> [];
+		    Scale =:= 1 -> Scaled;
+		    true -> convert_list(Tris, CCW, 1)
+		 end,
+	    {length(Tris), Scaled, Ns}
+    end.
+
+subd_tris(Level, MaxLevel, Sphere0) when Level < MaxLevel ->
+    Sphere = subd_tris(Sphere0, []),
+    subd_tris(Level+1, MaxLevel, Sphere);
+subd_tris(_,_, Sphere) -> Sphere.
+
+%%	  2             create a, b, c in the middle
+%%	 /\		Normalize a, b, c
+%%	/  \
+%%    c/____\ b		Construct new triangles
+%%    /\    /\		    [0,b,a]
+%%   /	\  /  \		    [a,b,c]
+%%  /____\/____\	    [a,c,2]
+%% 0	  a	1	    [b,1,c]
+%%
+
+subd_tris([{V0,V1,V2}|Rest], Acc) ->
+    A = e3d_vec:norm(midpoint(V0,V1)),
+    B = e3d_vec:norm(midpoint(V1,V2)),
+    C = e3d_vec:norm(midpoint(V0,V2)),
+    T1 = {V0,A,C},
+    T2 = {A,B,C},
+    T3 = {A,V1,B},
+    T4 = {C,B,V2},
+    subd_tris(Rest, [T1,T2,T3,T4|Acc]);
+subd_tris([],Acc) ->
+    Acc.
+
+midpoint({X1,Y1,Z1}, {X2,Y2,Z2}) ->
+    {(X1+X2)*0.5, (Y1+Y2)*0.5, (Z1+Z2)*0.5}.
+
+%%%%%%%%%%%%%% Converters %%%%%%%%%%%
+list_to_bin(Tris, true, 1) ->
+    << <<(X1):?F32,(Y1):?F32,(Z1):?F32,
+	(X2):?F32,(Y2):?F32,(Z2):?F32,
+	(X3):?F32,(Y3):?F32,(Z3):?F32>>
+     || {{X1,Y1,Z1},{X2,Y2,Z2},{X3,Y3,Z3}} <- Tris >>;
+list_to_bin(Tris, false, 1) ->
+    << <<(X1):?F32,(Y1):?F32,(Z1):?F32,
+	(X2):?F32,(Y2):?F32,(Z2):?F32,
+	(X3):?F32,(Y3):?F32,(Z3):?F32>>
+     || {{X1,Y1,Z1},{X3,Y3,Z3},{X2,Y2,Z2}} <- Tris >>;
+list_to_bin(Tris, true, Size) ->
+    << <<(X1*Size):?F32,(Y1*Size):?F32,(Z1*Size):?F32,
+	(X2*Size):?F32,(Y2*Size):?F32,(Z2*Size):?F32,
+	(X3*Size):?F32,(Y3*Size):?F32,(Z3*Size):?F32>>
+     || {{X1,Y1,Z1},{X2,Y2,Z2},{X3,Y3,Z3}} <- Tris >>;
+list_to_bin(Tris, false, Size) ->
+    << <<(X1*Size):?F32,(Y1*Size):?F32,(Z1*Size):?F32,
+	(X2*Size):?F32,(Y2*Size):?F32,(Z2*Size):?F32,
+	(X3*Size):?F32,(Y3*Size):?F32,(Z3*Size):?F32>>
+     || {{X1,Y1,Z1},{X3,Y3,Z3},{X2,Y2,Z2}} <- Tris >>.
+
+convert_list(List, true, 1) ->
+    [V || {V1,V2,V3} <- List, V <- [V1,V2,V3]];
+convert_list(List, false, 1) ->
+    [V || {V1,V2,V3} <- List, V <- [V1,V3,V2]];
+convert_list(List, true, Size) ->
+    [{(X*Size),(Y*Size),(Z*Size)} ||
+        {V1,V2,V3} <- List, {X,Y,Z} <- [V1,V2,V3]];
+convert_list(List, false, Size) ->
+    [{(X*Size),(Y*Size),(Z*Size)} ||
+        {V1,V2,V3} <- List, {X,Y,Z} <- [V1,V3,V2]].
