@@ -59,14 +59,14 @@ yes_no(Question, Yes) ->
 yes_no(Question, Yes, No) ->
     Qs = {vframe_dialog,
 	  [{label,Question,[{break,75*?CHAR_WIDTH}]}],
-	  [{buttons, [yes, no]}, {key, result}]
+	  [{buttons, [yes, no]}, {key, result}, {position, mouse}]
 	 },
     wings_dialog:dialog("", Qs, fun([{result,Res}]) -> yes_no_cancel(Res, Yes, No, ignore) end).
 
 yes_no_cancel(Question, Yes, No) ->
     Qs = {vframe_dialog,
 	  [{label,Question,[{break,75*?CHAR_WIDTH}]}],
-	  [{buttons, [yes, no, cancel]}, {key, result}]
+	  [{buttons, [yes, no, cancel]}, {key, result}, {position, mouse}]
 	 },
     wings_dialog:dialog("", Qs, fun([{result,Res}]) -> yes_no_cancel(Res, Yes, No, ignore) end).
 
@@ -99,6 +99,14 @@ crash_log(WinName, Reason, StackTrace) ->
     F = open_log_file(LogName),
     io:format("Internal Error~n",[]),
     [io:format(Fd, "Version: ~s\n", [?WINGS_VERSION]) || Fd <- [F, group_leader()]],
+    try
+        OsDesc = wx_misc:getOsDescription(),
+        {GLVend, GLRend} = {gl:getString(?GL_VENDOR), gl:getString(?GL_RENDERER)},
+	[io:format(Fd, "OS: ~ts\n", [OsDesc])  || Fd <- [F, group_leader()]],
+	[io:format(Fd, "GPU: ~ts | ~ts\n",[GLVend, GLRend])  || Fd <- [F, group_leader()]]
+    catch
+	_:_ -> ignore
+    end,
     [io:format(Fd, "Window: ~p\n", [WinName])  || Fd <- [F, group_leader()]],
     [io:format(Fd, "Reason: ~p\n\n", [Reason]) || Fd <- [F, group_leader()]],
     report_stacktrace(F, StackTrace),
@@ -110,14 +118,14 @@ report_stacktrace(F, [_|_]=StackTrace) ->
     ShortStackTrace = [{M,N,if
 				is_list(A) -> length(A);
 				true -> A
-			    end} || {M,N,A} <- StackTrace],
-    io:format("Stack trace:\n~P\n\n", [StackTrace, 20]),
+			    end, FL} || {M,N,A,FL} <- StackTrace],
+    format(group_leader(), "Stack trace:\n~P\n\n", [StackTrace, 20]),
     case ShortStackTrace =:= StackTrace of
 	false ->
-	    io:format(F, "Short stack trace:\n~p\n\n", [ShortStackTrace]),
-	    io:format(F, "Long stack trace:\n~p\n\n", [StackTrace]);
+	    format(F, "Short stack trace:\n~p\n\n", [ShortStackTrace]),
+	    format(F, "Long stack trace:\n~P\n\n", [StackTrace, 50]);
 	true ->
-	    io:format(F, "Stack trace:\n~p\n\n", [StackTrace])
+	    format(F, "Stack trace:\n~p\n\n", [StackTrace])
     end;
 report_stacktrace(_, []) ->
     %% Make sure we don't write anything if there is no stacktracke.
@@ -125,6 +133,10 @@ report_stacktrace(_, []) ->
     %% process that runs wings:halt_loop/1.)
     ok.
 
+format(To,F,A) ->
+    %% Format in this process to avoid sending 'St' to another process
+    io:put_chars(To, unicode:characters_to_binary(io_lib:format(F,A))).
+
 caption(#st{file=undefined}=St) ->
     Caption = wings(),
     wings_io:set_title(Caption),
@@ -196,7 +208,7 @@ wings() ->
 wings_branch() ->
     %% Fool dialyzer
     case is_process_alive(self()) of
-        true -> ?wings_branch;
+        true -> {branch, ?wings_branch};
         false -> ""
     end.
 
