@@ -25,7 +25,7 @@
 	 array_is_empty/1,array_entries/1,
 	 mapsfind/3,
 	 wxequal/2, wxset_pid/2, min_wx/1,
-	 nice_float/1,
+	 nice_float/1,nice_vector/1,nice_abs_vector/1,
 	 unique_name/2,
 	 is_name_masked/2,
 	 lib_dir/1,
@@ -35,6 +35,8 @@
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
+-include("e3d.hrl").
+
 -import(lists, [foldl/3,reverse/1,member/2,last/1]).
 
 share(X, X, X) -> {X,X,X};
@@ -186,7 +188,21 @@ array_is_empty(Array) ->
 %%
 array_entries(Array) ->
     array:sparse_foldl(fun(_, _, N) -> N + 1 end, 0, Array).
-			        
+
+-spec nice_abs_vector(e3d_vector()) -> iolist().
+
+nice_abs_vector({X,Y,Z}) ->
+    nice_vector({abs(X),abs(Y),abs(Z)}).
+
+-spec nice_vector(e3d_vector()) -> iolist().
+
+nice_vector({X,Y,Z}) ->
+    ["<",
+     wings_util:nice_float(X),"  ",
+     wings_util:nice_float(Y),"  ",
+     wings_util:nice_float(Z),
+     ">"].
+
 nice_float(F) when is_float(F) ->
     simplify_float(lists:flatten(io_lib:format("~f", [F]))).
 
