@@ -1,3 +1,4 @@
+
 %%
 %%  wings_wm.erl --
 %%
@@ -100,7 +101,7 @@ init(Frame) ->
     put(wm_windows, gb_trees:empty()),
     new(top_frame, Frame, {push, fun wings_frame:forward_event/1}),
     set_dd(top_frame, geom_display_lists), %% Selection mode updates
-    StatusBar = wings_status:start(Frame),
+    StatusBar = wings_status:get_statusbar(),
     new(message, StatusBar, {push, fun message_event/1}),
 
     case wings_pref:get_value(win32_start_maximized) of
@@ -441,8 +442,8 @@ grab_focus() ->
 grab_focus(Name) ->
     case is_window(Name) of
 	true  ->
-%%	    {_, [_,Where|_]} = erlang:process_info(self(), current_stacktrace),
-%%	    io:format("Grab ~p~n",[Where]),
+	    %%{_, [_,Where|_]} = erlang:process_info(self(), current_stacktrace),
+	    %%io:format("Grab focus: ~p~n   ~p~n",[Name, Where]),
 	    case get(wm_focus_grab) of
 		undefined -> put(wm_focus_grab, [Name]);
 		Stack -> put(wm_focus_grab, [Name|Stack])
@@ -453,8 +454,12 @@ grab_focus(Name) ->
 release_focus() ->
     case get(wm_focus_grab) of
 	undefined -> ok;
-	[_] -> erase(wm_focus_grab);
-	[_|Stack] -> put(wm_focus_grab, Stack)
+	[_Name] ->
+            %%io:format("Release focus ~p~n",[_Name]),
+            erase(wm_focus_grab);
+	[_Name|Stack] ->
+            %%io:format("Release focus ~p~n",[_Name]),
+            put(wm_focus_grab, Stack)
     end.
 
 grabbed_focus_window() ->
@@ -708,6 +713,7 @@ dispatch_event(#wx{event=#wxActivate{active=Active}}) ->
     end;
 dispatch_event(#wx{obj=Obj, event=#wxSize{size={W,H}}}) ->
     ?CHECK_ERROR(),
+    erase(current_gl), %% resets wxGLCanvas:setCurrent().
     case W > 0 andalso H > 0 andalso not (wx2win(Obj) =:= none) of
 	true ->
 	    #win{name=Name} = Geom0 = get_window_data(Obj),
@@ -718,6 +724,22 @@ dispatch_event(#wx{obj=Obj, event=#wxSize{size={W,H}}}) ->
 	false ->
 	    true
     end;
+dispatch_event(#wx{obj=Obj, event=#wxFocus{}}) ->
+    case {get_focus_window(), wx2win(Obj)} of
+        {undefined, _} -> ok;
+        {Win, Win} ->
+            update_focus(none);
+        {{grabbed, Win}, Win} ->
+            io:format("Grabbed focus lost: ~p ~p~n", [Obj, Win]),
+            do_dispatch(Win, lost_focus),
+            update_focus(none);
+        {{grabbed, OtherWin}, Win} ->
+            io:format("Grabbed focus lost old?: ~p ~p~n", [OtherWin, Win]),
+            ok;
+        {_OldFocus, _CurrentFocus} ->
+            ok
+    end,
+    true;
 dispatch_event(quit) ->
     foreach(fun(Name) -> send(Name, quit) end, gb_trees:keys(get(wm_windows))),
     true;
@@ -742,19 +764,13 @@ dispatch_event(#wx{obj=Obj}=Event) ->
     end;
 dispatch_event({'EXIT', _Pid, normal}) ->
     true;
-dispatch_event({'EXIT', Pid, {Reason, StackTrace}}) ->
+dispatch_event({'EXIT', Pid, _Reason0}) ->
     Found = [Win || #win{name=Win, obj=Obj} <- gb_trees:values(get(wm_windows)),
 		    (catch wx_object:get_pid(Obj)) =:= Pid],
-    Name = case Found of
-	       [WName] -> WName;
-	       _ ->
-		   case process_info(Pid, registered_name) of
-		       undefined -> Pid;
-		       {registered_name,Reg} -> Reg
-		   end
-	   end,
-    LogName = wings_u:crash_log(Name, Reason, StackTrace),
-    send(geom, {crash_in_other_window,LogName}),
+    case Found of
+        [WName] -> (catch delete(WName));
+        _ -> ignore
+    end,
     true;
 dispatch_event(Event) ->
     case find_active() of
@@ -846,7 +862,12 @@ redraw_all() ->
 redraw_win({Name, #win{w=W,h=H,obj=Obj}}) ->
     DoSwap = case use_opengl(Obj) of
 		 true ->
-		     wxGLCanvas:setCurrent(Obj),
+                     case get(current_gl) of
+                         Obj -> ignore;
+                         _ ->
+                             wxGLCanvas:setCurrent(Obj),
+                             put(current_gl, Obj)
+                     end,
 		     gl:viewport(0,0,W,H),
 		     gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT),
 		     true;
@@ -957,14 +978,18 @@ handle_event(State, Event, Stk) ->
 	    wings_u:message(Error),
 	    Stk;
 	exit:normal ->
-	    exit(normal);
+	    exit(shutdown);
+	exit:shutdown ->
+	    exit(shutdown);
 	exit:{crash_logged, _}=Reason ->
 	    exit(Reason);
 	exit:Exit ->
+            erase(wm_focus_grab),
 	    [#se{h=CrashHandler}] = pop_all_but_one(Stk),
 	    handle_response(CrashHandler({crash,Exit}), Event,
 			    default_stack(this()));
 	error:Reason ->
+            erase(wm_focus_grab),
 	    [#se{h=CrashHandler}] = pop_all_but_one(Stk),
 	    handle_response(CrashHandler({crash,Reason}), Event,
 			    default_stack(this()))
@@ -1127,8 +1152,13 @@ geom_below(Pos) ->
 	true -> none;
 	false ->
             All = windows(),
-            Geoms0 = [geom|[Geom|| {Win,_}=Geom <- All,
-                                   Win =:= geom orelse Win =:= autouv]],
+            Filter = fun(geom) -> true;
+                        ({geom,_}) -> true;
+                        ({autouv,_}) -> true;
+                        ({plugin, {_, geom}}) -> true;
+                        (_) -> false
+                     end,
+            Geoms0 = lists:filter(Filter, All),
             Geoms = [get_window_data(Name) || Name <- Geoms0],
             find_window(Geoms, Geoms, Win0)
     end.
